import com.ibm.websphere.rainmaker.logging.Output;


def createKeystore(keystore, owner = false) {
    System.err.println("createKeystore: " + [keystore, owner].toString());
    return createResource('keystores', keystore, owner)
}

def deleteKeystore(id) {
    def rv =  deleteResource('keystores', id)
    def refs = getResources('domainskeystores', [keystoreid: id])

    refs.each {
        deleteResource('domainskeystores', it.id)
    }
    return rv
}

def getKeystore(id) {
    def rv = getResource('keystores', [id: id], ['id', 'name', 'keystore', 'truststore'])
    System.err.println("getKeystore: " + rv.toString() )
    return rv
}

def getKeystores(filters, owner = false) {
    def _filters = filters.clone()
    def rv = getResources( 'keystores', _filters, false,  ['id', 'name', 'keystore', 'truststore'])

    System.err.println("getKeystores: " + [filters, rv].toString())

    return rv
}

def updateKeystore(keystore) {
    return updateResource('keystores', keystore,  ['id', 'name', 'keystore', 'truststore'])
}

//No longer need to check for trust store files.  Should consider removing
//this groovy method.
def determineTrustStoreFile(dir, file) {
    def lcn = file.name.toLowerCase()

    if (lcn.contains("trust")) {
        return null
    }

    def tnm = file.name.replaceAll(".jks", "Trust.jks")
    tnm = tnm.replaceAll(".p12", "Trust.p12")
    tnm = tnm.replaceAll(".jcek", "Trust.jcek")
    
    def tfile = new File(dir.absolutePath + "/" + tnm)

    Output.info("Testing ${tfile}")
    if (tfile.exists()) {
        return tfile
    }
    return null
}

// Sync up with keystores on filesystem.
def refreshKeystores() {

    try {
        def ks = getKeystores([:], false)
        def keydir = new File(config.app.root[] + "/keystores")

        ks.each {
            def key   = new File(keydir.absolutePath + "/" + it.keystore)
            if (!key.exists()) {
                Output.info("Cleaning keystore ${it.id} for ${it.keystore}")
                deleteKeystore(it.id)
            }
        }
        
        keydir.eachFileMatch(~/.*.(jks|p12|jcek)/) {
        	//Need to update keystore list in case multiple threads are running.
            getKeystores([:], false)
            def ress = getKeystores( ["keystore": it.name], owner = false)
            if (ress.size() == 0) {
                Output.info("Detected new keystore ${it.name}")
                createKeystore(["keystore": it.name, "name": it.name, "truststore": ""])
            }
        }
    } catch (ex) {
        Output.error("Exception while syncing keystores: ${ex.toString()}")
    }
}
            
            
    
