/* TODO: This should move to a xs.app.setup, so it is easier for us to not ship this. */
import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.users.Permissions;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.FileOutputStream;

def taskLogger;
def lang;

def createMonitorAdmin()
{
    def user = [
        username: 'admin', 
        name: MessageUtils.message(lang, 'XSW0111', []),
        email: ''
    ];

        try
        {
            createUser(user, false, config.rainmaker.admin.defaultPassword[]);
        }

        catch (Throwable t)
        {
            logger.FINE { CleanStackTrace.cleanStackTrace(t) };
        }

        addUserRole(user.id, Permissions.SUPER_USER);
        addUserRole(user.id, Permissions.APPLIANCE_ADMIN);
        addUserRole(user.id, Permissions.APPLIANCE_MONITOR);
        addUserRole(user.id, Permissions.APPLIANCE_SECURITY);
        addUserRole(user.id, Permissions.GRID_CREATE);

}

def setupClasspath() {
    def f = new File(config.dependencies.'ibm:xs.app'[] + "/.zero/private/resolved.properties")
    
    def lib = new File(config.dependencies.'ibm:xs.app'[] + "/../lib/ogclient.jar")
    if ( !lib.exists() ) {//WAS installation
       lib = new File(config.dependencies.'ibm:xs.app'[] + "/lib/ogclient.jar")
    }
    
    def fs = new FileInputStream(f.absolutePath)
    def p = new Properties()

    p.load(fs)

    if (!p["classpath.cli"].contains(lib.absolutePath)) {
        p["classpath.cli"] += System.getProperties().getProperty("path.separator") + lib.absolutePath
        
        def os = new FileOutputStream(f.absolutePath)

        p.store(os, "")
    }
}
    

def mkUserTables(newestVersion) {
    def gendir = new File(config.app.root[]);

	broadcastInfo("Creating new XS database using schema version: "+newestVersion);
    create(gendir, ".", "derby", newestVersion);

    broadcastInfo("Creating acl tables");
    create(gendir, config.dependencies.'ibm:rainmaker.acl'[], "derby", "1.0.0.0");
    createResource("Version", [id: 1, number: newestVersion], null, null);
}

def popUserTables() {
    config.xsa.retrieveFromBackupZone = false;
    config.rainmaker.alwaysJoinThreads = true;
    
    log('Creating all-inclusive public group...');        
    createEveryoneGroup();

    log('The public group was created successfully.');
    
    log('Creating admin account...');        
    createMonitorAdmin();
    log('The admin account was created successfully.');
}        


def mkSecretKey() {
    secretKeyConfigFile = new File(config.dependencies.'ibm:xs.app'[] + "/config/key.config");
    def secretKey = zero.core.security.encryption.SecretKeyGenerator.generateSecretKey()
    secretKeyConfigFile.write ("/config/security/secretKey=\"${secretKey}\"")
}

//def supportedCheck() {
//    def p = System.getProperties()

//    if (!p.getProperty("os.name").equalsIgnoreCase("linux")) {
        // Using this property is fine for now, but will need to be updated
        // if JVM support gets expanded before the 64 bit limitation is lifted.
//        if (p.getProperty("com.ibm.vm.bitmode") == "64") {
//            Output.error(MessageUtils.message(lang, 'XSW0153', []))
//            System.exit(1)
//        }
//    }
//}

def migrateTables(newestVersion, allVersions) {
	//No way to role back.  Need to be able to look up version and role back.
	def version = getResource("Version", [id: 1], null);
	if (version == null) {
		broadcastInfo(MessageUtils.message(lang, 'XSW0160', []));
		version = [number: "1.0.0.0"];
	}

    if (isVersionLessThan(version.number, newestVersion)) {
    	broadcastInfo("XS database schema is not at latest version.");
    	migrateTablesHelper(version, newestVersion, allVersions);
    } else {
    	Output.info("XS database schema version: "+version.number);
    }
}

def migrateTablesHelper(version, newestVersion, allVersions) {
    def gendir = new File(config.app.root[]);
	def startMigrating = false;
	try {
		allVersions.each { migrationVersion ->
			if (startMigrating) {
				broadcastInfo("Migrating XS database schema to: "+migrationVersion);
		    	migrate(gendir, ".", "derby", migrationVersion);
		    	migrateData(migrationVersion);
		    	broadcastInfo("Migration completed successfully.");
			}
			
			if (version.number == migrationVersion)
				startMigrating = true;
		}

		if (version.number == "1.0.0.0") {
	    	createResource("Version", [id: 1, number: newestVersion], null, null);
	    	log("Version table added successfully");
		} else {
			updateResource("Version", [id: 1, number: newestVersion], null);
			log("Version table updated to "+newestVersion);
		}
	} catch (e) {
		broadcastError(MessageUtils.message(lang, 'XSW0159', [taskLoggerPath]));
		log(CleanStackTrace.cleanStackTrace(e));
	}
}

def migrateData(version) {
	if (version == "1.0.0.2") {
		migrateData1002();
	}
}

def migrateData1002() {
	def filters = [name:null, order:"name", count:null, page:null] ;
	def servers = getResources( 'servers', filters, false,  ["id", "alias", "name", "listenerport", "jmxport"])
	servers.each { server ->
		if (server.alias == null) {
			server.alias = server.name+":"+server.listenerport;
			updateResource('servers', server,  ["id", "alias", "name", "listenerport", "jmxport"]);
			log("Catalog server "+server.name+" migrated to 1.0.0.2 DB schema.");
		}
	}
}

def isVersionLessThan(version, currentVersion) {
	def normalizedVersion = version.replaceAll("\\.", "");
	def normalizedCurrentVersion = currentVersion.replaceAll("\\.", "");	
	if (normalizedVersion < normalizedCurrentVersion) {
		return true;
	} else {
		return false;
	}
}

def broadcastInfo(message) {
	Output.info(message);
	log(message);
}

def broadcastError(message) {
	Output.error(message);
	log(message);
}

def log(message) {
	taskLogger.append(getNow()+" :: "+message+"\n");
}

def getNow() {
	SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
	return formatter.format(Calendar.getInstance().getTime());
}

def onCliTask() {
	def weird = 
	taskLoggerPath = config.dependencies.'ibm:xs.app'[] + "/logs/startConsoleServer.log"
	lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
	taskLogger = new File(taskLoggerPath);
	def allVersions = ["1.0.0.0", "1.0.0.1", "1.0.0.2"];
	def newestVersion = "1.0.0.2";

	if (taskLoggerPath.indexOf("\\") != -1) {
		taskLoggerPath = taskLoggerPath.replace("\\", "\\\\").replace("/", "\\\\");
	}

    try {
        def f = new File(config.app.root[] + "/databases")
        
        //supportedCheck()
        setupClasspath()
        if (!f.exists()) {
            mkSecretKey()
            mkUserTables(newestVersion)
            popUserTables()
        } else {
        	migrateTables(newestVersion, allVersions)
        }
        
        broadcastInfo(MessageUtils.message(lang, 'XSW0161', [config.http.port[].toString()]));
        broadcastInfo(MessageUtils.message(lang, 'XSW0162', [config.https.port[].toString()]));
        broadcastInfo(MessageUtils.message(lang, 'XSW0163'));
    } catch (Throwable t) {
    	broadcastError(MessageUtils.message(lang, 'XSW0159', [taskLoggerPath]));
		log(CleanStackTrace.cleanStackTrace(t));
    }
}
