/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.config.AdminState;
import com.ibm.bedrock.config.ReturnCode;
import com.ibm.bedrock.config.Value;
import com.ibm.bedrock.config.SimpleValue;
import com.ibm.bedrock.config.StructValue;
import com.ibm.bedrock.config.ServiceUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import zero.json.Json;


def createAggregateInterface(aggregateInterface)
{
	logger.INFO { "createAggregateInterface"};
	logger.INFO { aggregateInterface.asXML()};
		def aggIDValue = aggregateInterface.get(com.ibm.bedrock.appliance.EthernetInterfaces.KEY_NAME);
		if (aggIDValue != null && aggIDValue instanceof SimpleValue) {
			def aggID = ((SimpleValue)aggIDValue).getValue();
			ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
			return serviceUtils.createObject("default", "aggregate-interface", aggID, aggregateInterface, AdminState.kwEnabled ,false);
        } else {
        	logger.INFO{ "Could not create aggregate interface." };
        	return ReturnCode.kwValidationFailed ;
        }
}

def deleteAggregateInterface(id)
{
	logger.INFO { "deleteAggregateInterface"};
		if (id != null && id instanceof String) {
			ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
			return serviceUtils.deleteObject("default", "aggregate-interface", id);
        }
        logger.INFO{ "Failed to delete aggregate interface." };
        return ReturnCode.kwValidationFailed ;
}


def getAggregateInterfaces()
{ 
	logger.INFO { "getAggregateInterfaces"};
    if(BedrockUtils.isBedrock() ){								
        def resultSet = []
        ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
        def interfaceNames = serviceUtils.getNamesofObject("default", "aggregate-interface");
		logger.INFO { "WebSphere XC10 DataPower Aggregate interfaces: " + interfaceNames.size()};
        interfaceNames.each(){ current ->
            if(current){
                resultSet.add(current);
            }
        }
        return resultSet;
    }
}

def isEthernetInterfaceEnabled(id)
{
	logger.INFO { "isEthernetInterfaceEnabled"};
	if(BedrockUtils.isBedrock() ){	
		ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
		AdminState adminState = serviceUtils.getAdminState("default", "ethernet-interface", id);
        if (adminState != null && adminState.getValue() == AdminState.kwEnabled.getValue()){
        	return true;
        }
	}
	return false;
}


def getAggregateInterface(id, lowercaseKeys=true)
{
	logger.INFO { "getAggregateInterface"};
        if(BedrockUtils.isBedrock()){
                ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
                Value aggInterface = serviceUtils.queryObject("default", "aggregate-interface", id);
                if(!aggInterface){
                        return null
                }
                return aggInterface;
         }
}
                    

def updateAggregateInterface(aggregateInterface)
{
	logger.INFO { "updateAggregateInterface"};
	logger.INFO { aggregateInterface.asXML()};
		def aggIDValue = aggregateInterface.get(com.ibm.bedrock.appliance.EthernetInterfaces.KEY_NAME);
		if (aggIDValue != null && aggIDValue instanceof SimpleValue) {
			def aggID = ((SimpleValue)aggIDValue).getValue();
			ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
			return serviceUtils.updateObject("default", "aggregate-interface", aggID, aggregateInterface, false);
        }
        logger.INFO{ "Failed to update aggregate interface" };
        return ReturnCode.kwValidationFailed;
}

def setAdminState(id, adminState)
{
	logger.INFO { "setAdminState"};
	ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
	return serviceUtils.setAdminState("default", "aggregate-interface", id, adminState);
}
