/*s
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.config.AdminState;
import com.ibm.bedrock.config.Value;
import com.ibm.bedrock.config.ArrayValue;
import com.ibm.bedrock.config.SimpleValue;
import com.ibm.bedrock.config.StructValue;
import com.ibm.bedrock.config.ServiceUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import zero.json.Json;

def _convertEthernetID (id) {
	 logger.INFO { "id is: " + id };
	 return id == 'mgmt' ? 'eth4' : id;
}

def getEthernetInterfaces()
{ 
    if(BedrockUtils.isBedrock() ){								
        def resultSet = []
        ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
        def interfaceNames = serviceUtils.getNamesofObject("default", "ethernet-interface");
		// 9004 appliance has 4 interfaces, the 9005 appliance has 12 interfaces
		def applianceType = (interfaceNames.size() > 4 ? "9005" : "9004");
		logger.INFO { "WebSphere XC10 DataPower Appliance Type: " + applianceType };
		logger.INFO { "WebSphere XC10 DataPower Ethernet interfaces: " + interfaceNames.size()};
        interfaceNames.each(){ current ->
            if(current){
                resultSet.add(current);
				
				// This permits us to get the management interfaces as always private for 9005 hardware, by design.
				if(current.startsWith('mgt') && applianceType.equals("9005")) {
					setMgmtInterfacesAsPrivate(current);
				}
            }
        }
        return resultSet;
    }else{
        return getEthernetInterfacesDeprecated();
    }
}

def setMgmtInterfacesAsPrivate(id) {

    ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
    Value ethInterface = serviceUtils.queryObject("default", "ethernet-interface", id);
    if(ethInterface == null){
    	return;
    }else{
    	def userDataValue = ((StructValue)ethInterface).get("userdata");
        if ( userDataValue != null && userDataValue instanceof SimpleValue){
        	String userDataString = ((SimpleValue)userDataValue).getValue();
        	def userData = zero.json.Json.decode(userDataString);

            def private_ip = userData.get("private_ip");
            // enabled is bad, we should be changing that to true.
        	//if (private_ip == null || !(private_ip.equals("enabled") || private_ip.equals("true"))) {
        	if (private_ip == null || !private_ip.equals("true")) {
        		userData.put("private_ip", "true");
                def encodedValue = zero.json.Json.encode(userData, false);  //do not pretty print to keep in on one line
                ((SimpleValue)userDataValue).setValue(encodedValue);
                serviceUtils.updateObject("default", "ethernet-interface", id, ethInterface, true);
        	}
        }
    }
    
}

def getEthernetInterfacesDeprecated()
{
    def response = executeApplianceRequest([
        name: 'soma:get-config',
        content: [
            class: 'EthernetInterface'
        ]
    ]);
    def eth = response.config?.EthernetInterface;
    if(!eth){
    	return null;
    }
	def etherInterfaces = []
	eth.each() { currentEth ->
	        etherInterfaces << currentEth.name ;
	}
	return etherInterfaces;
}

def getEthernetInterface(id)
{
    	if(BedrockUtils.isBedrock()){
    		ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
    		Value ethInterface = serviceUtils.queryObject("default", "ethernet-interface", id);
    		if(ethInterface == null){
            	return null;
            }else{
    			logger.FINE { ethInterface.asXML()};
            	def eth = [:]
            	eth['id'] = id;
                //compatibility with existing CB UI
                AdminState adminState = serviceUtils.getAdminState("default", "ethernet-interface", id);
                if (adminState != null && adminState.getValue() == AdminState.kwEnabled.getValue()){
                	eth['madminstate'] = 'enabled'
                }else{
                    eth['madminstate'] = 'disabled';
                }
    			logger.FINE { ((StructValue)ethInterface).toString()};
                def keys = ((StructValue)ethInterface).keySet().collect { it };
                if(keys.contains("userdata") ){
                	def userData = ((StructValue)ethInterface).get("userdata");
                    if ( userData instanceof SimpleValue){
        				String arrayAsJsonString = ((SimpleValue)userData).getValue();
        	            eth.putAll ((Map)zero.json.Json.decode(arrayAsJsonString));
        	        } 
        	        else if ( userData instanceof ArrayValue){
        	        	String arrayAsJsonString = zero.json.Json.encode(((ArrayValue)userData).subList(0, ((ArrayValue)userData).size()-1));
        	            eth.putAll ((Map)zero.json.Json.decode(arrayAsJsonString));
        	        }
        	        keys.remove("userdata");
                 }
                 //eth['useipv6'] = (keys.contains("ipv6_default_gateway")) ? 'enabled':'disabled';
    	         keys.each {
    	         	def value = ((StructValue)ethInterface).remove(it);
    	            if ( value instanceof SimpleValue ) {
    	            	eth[it.toLowerCase()] = ((SimpleValue)value).getValue();
    	            } else if ( value instanceof StructValue ) { //ip
        	        	def keys1 = ((StructValue)value).keySet();
        	           	for (Iterator iter1 = keys1.iterator();iter1.hasNext(); ) {
        	            	def key1 = iter1.next();
        	                def value1 = ((StructValue)value).get(key1);
        	                if ( value1 instanceof SimpleValue ) {
        	                	if(key1 == "use-dhcp") {
        	                		eth[key1.toLowerCase().replaceAll("-","_")] = ((SimpleValue)value1).getValue();
        	                	} else {
        	    	        		eth[key1.toLowerCase()] = ((SimpleValue)value1).getValue();
        	    	        	}
        	    	        }else if (  value1 instanceof ArrayValue ) { 
        	    	            if ( key1 == "address") { //address - we allow only 1 ipaddress
    								def ipList = [];
        	    	             	for (Iterator<Value> iter3 = ((ArrayValue)value1).iterator();iter3.hasNext();){
        	    	                	Value val3 = iter3.next();
        	    	                	if (val3 != null && val3 instanceof SimpleValue) {
        	    	                   		ipList.add(((SimpleValue)val3).getValue());
        	    	                	}
        	    	             	}
        	    	             	if (keys.contains("ipv6_default_gateway")) {
        	    	             		eth.put("ipv6address", ipList.get(0));
        	    	             	} else {
        	    	             		eth.put(key1.toLowerCase(), ipList.get(0));
        	    	             	}
        	    	             } else if ( key1 == "static-route" ) {
        	    	             //TODO add code to deal with that if needed
        	    	             }
        	    	          }else if ( value1 instanceof StructValue ) { //gateway
        	    	          	def keys2 = ((StructValue)value1).keySet();
        	                    for (Iterator iter2 = keys2.iterator();iter2.hasNext(); ) {
        	                    	def key2 = iter2.next();
        	                        def value2 = ((StructValue)value1).get(key2);
        	                        if ( value2 instanceof SimpleValue ) {
        	    	                	eth[key1.toLowerCase().replaceAll("-","_")] = ((SimpleValue)value2).getValue();
        	    	                }
        	                     }
        	    	          }
        	               }
        	           	}
    				}	     
    	    		logger.FINE {eth.toString()};
    	    		return eth;
    			}
    		}else{
            	id = _convertEthernetID(id);
            	return getEthernetInterfaceDeprecated(id);
			}
    	}

def getEthernetInterfaceDeprecated(id){	
	
    def response = executeApplianceRequest([
        name: 'soma:get-config', 
        content: [
            class: 'EthernetInterface', 
            name: id
        ]
    ]);
    
    def eth = response.config?.EthernetInterface;
    
    if (!eth)
        return null;
        
    eth.remove('intrinsic');
    eth.remove('name');
    
    def keys = eth.keySet().collect { it };
    
    keys.each {
        def value = eth.remove(it);
        eth[it.toLowerCase()] = value;
    }
    
    return eth;
}

def updateEthernetInterface(eth)
{
	logger.INFO { "entry of updateEthernetInterface" };
	logger.FINE { eth.toString(); };
	if(BedrockUtils.isBedrock()){
        ServiceUtils serviceUtils = ServiceUtils.createServiceUtils();
		logger.FINE { "after entry" };
        if(eth.madminstate !=null){
			logger.FINE { "if eth.madminstate != null" };
			AdminState adminState = serviceUtils.getAdminState("default", "ethernet-interface", eth.id);
            if (adminState != null){
            	if(eth['madminstate'] == 'enabled'){
            		adminState = AdminState.kwEnabled;
                }else{
                    adminState = AdminState.kwDisabled;
                }
                logger.INFO {"before setting the AdminState we send adminState = " + adminState };
                def retValueAdmin = serviceUtils.setAdminState("default", "ethernet-interface", eth.id, adminState);
                logger.INFO {"after setting the AdminState we get retValue = " + retValueAdmin };
             }
			logger.FINE { "after eth.madminstate != null" };
        }
        
        Value ethernetInt = serviceUtils.queryObject("default", "ethernet-interface",eth.id);
        Value ipStruct = ((StructValue)ethernetInt).get("ip");
        def needUpdate = false;
		logger.FINE { "ipStruct="+ ipStruct.toString() };
        if (ipStruct != null) {
        	if(eth.defaultgateway !=null ) {
        		needUpdate = true;
        		if (eth.defaultgateway != ''){
					logger.FINE { "if eth.ipv4_default_gateway != null" };
        			Value gatewayValue = ((StructValue)ipStruct).get("ipv4-default-gateway");
        			if (gatewayValue != null) {
        				Value gValue = ((StructValue)gatewayValue).get("gateway");
        				if ( gValue != null) {
        					((SimpleValue)gValue).setValue(eth.defaultgateway);
        					((StructValue)gatewayValue).put("gateway",gValue);
        				}
        			} else {
        				gatewayValue = new StructValue();
        				SimpleValue gValue = new SimpleValue();
        				gValue.setValue(eth.defaultgateway);
        				((StructValue)gatewayValue).put("gateway",gValue);
        			}
        			((StructValue)ipStruct).put("ipv4-default-gateway",gatewayValue);
        			((StructValue)ipStruct).remove("ipv6-default-gateway");	
					logger.FINE { "after eth.ipv4_default_gateway != null" };
				} else {
					((StructValue)ipStruct).remove("ipv4-default-gateway");	
				}
				logger.FINE { "ipStruct="+ipStruct.toString() };
			}
			if(eth.defaultipv6gateway !=null ) {
				needUpdate = true;
				if (eth.defaultipv6gateway != ''){ 
					logger.FINE { "if eth.ipv6_default_gateway != null" };
					Value gatewayValue = ((StructValue)ipStruct).get("ipv6-default-gateway");
					if (gatewayValue != null) {
        				Value gValue = ((StructValue)gatewayValue).get("gateway");
        				if ( gValue != null) {
        					((SimpleValue)gValue).setValue(eth.defaultipv6gateway);
        					((StructValue)gatewayValue).put("gateway",gValue);
        				}
        			} else {
        				gatewayValue = new StructValue();
        				SimpleValue gValue = new SimpleValue();
        				gValue.setValue(eth.defaultipv6gateway);
        				((StructValue)gatewayValue).put("gateway",gValue);
        			}	
        			((StructValue)ipStruct).put("ipv6-default-gateway",gatewayValue);
        			((StructValue)ipStruct).remove("ipv4-default-gateway");	
					logger.FINE { "after eth.ipv6_default_gateway != null" };
				} else {
					((StructValue)ipStruct).remove("ipv6-default-gateway");	
				}
				logger.FINE { "ipStruct="+ipStruct.toString() };
			}
        	if(eth.ipaddress !=null){
        		needUpdate = true;
        		if ( eth.ipaddress != '') {
					logger.FINEST { "if eth.ipaddress != null" };
					SimpleValue addValue = new SimpleValue();
					addValue.setValue(eth.ipaddress);
        			Value addressValue = ((StructValue)ipStruct).get("address");
        			if ( addressValue == null) {
        				addressValue = new ArrayValue();
        				((ArrayValue)addressValue).add(0, addValue);
        			} else {
        				((ArrayValue)addressValue).set(0, addValue);
        			}
        			((StructValue)ipStruct).put("address",addressValue);	
					logger.FINEST { "after if eth.ipaddress != null" };
				} else {
					((StructValue)ipStruct).remove("address");
					logger.FINE { "ipStruct="+ipStruct.toString() };
				}
        	}
			if(eth.ipv6address != null) {
				needUpdate = true;
				if ( eth.ipv6address != '') {
					logger.FINEST { "if eth.ipv6address != null" };
					SimpleValue addValue = new SimpleValue();
					addValue.setValue(eth.ipv6address);
					Value addressValue = ((StructValue)ipStruct).get("address");
        			if ( addressValue == null) {
        				addressValue = new ArrayValue();
        				((ArrayValue)addressValue).add(0, addValue);
        			} else {
        				((ArrayValue)addressValue).set(0, addValue);
        			}
        			((StructValue)ipStruct).put("address",addressValue);
					logger.FINE { "ipStruct="+ipStruct.toString() };
					logger.FINEST { "after if eth.ipv6address != null" };
				} else {
					((StructValue)ipStruct).remove("address");
					logger.FINE { "ipStruct="+ipStruct.toString() };
				}
			}
			if(eth.use_dhcp != null) {
				needUpdate = true;
				logger.FINEST { "if eth.use_dhcp != null" };
				SimpleValue dhcpValue = (SimpleValue)((StructValue)ipStruct).get("use-dhcp");
        		if ( dhcpValue == null) {
        			dhcpValue = new SimpleValue();
        		}
        		dhcpValue.setValue(eth.use_dhcp);
        		((StructValue)ipStruct).put("use-dhcp",dhcpValue);
				logger.FINEST { "after if eth.use_dhcp != null" };
			}
			((StructValue)ethernetInt).put("ip", ipStruct);
			logger.FINE { "ethernetInt="+ethernetInt.toString() };
		}
		
        if( eth['mtu'] !=null){
        	needUpdate = true;
			logger.FINE { "if eth.mtu != null" };
        	Value mtuValue = ((StructValue)ethernetInt).get("mtu");
        	if ( mtuValue != null) {
        		((SimpleValue)mtuValue).setValue(eth.mtu);
        	}
			logger.FINE { "after if eth.mtu != null" };
        }
        if( eth.mode !=null){
        	needUpdate = true;
        	logger.FINE { "if eth.mode != null" };
        	Value modeValue = ((StructValue)ethernetInt).get("mode");
        	if ( modeValue != null) {
        		((SimpleValue)modeValue).setValue(eth.mode);
        	}
			logger.FINE { "after if eth.mode != null" };
        }
        if( eth.private_ip != null)
        {
        	needUpdate = true;
            def userData;
            Value userDataValue = ((StructValue)ethernetInt).get("userdata");
            if ( userDataValue != null) {
            	def userDataString = ((SimpleValue)userDataValue).getValue();
            	userData = zero.json.Json.decode(userDataString);
            	userData.put("private_ip", eth.private_ip);
            	def encodedValue = zero.json.Json.encode(userData, false);	//do not pretty print to keep in on one line
            	((SimpleValue)userDataValue).setValue(encodedValue);
			} else {
				userData = [:];
				userData.put("private_ip", eth.private_ip);
				def encodedValue = zero.json.Json.encode(userData, false);	//do not pretty print to keep in on one line
				SimpleValue dataValue = new SimpleValue();
				dataValue.setValue(encodedValue);
				((StructValue)ethernetInt).put("userdata", dataValue);
			}
        }
		logger.FINE { "after all if checks" };
		if ( needUpdate ) {
 	    	def returnValue = serviceUtils.updateObject("default", "ethernet-interface", eth.id, ethernetInt, true);
			logger.INFO{ "after updating ethernet interface, we return " + returnValue };
        	return returnValue;
        } else {
        	logger.INFO{ "No need for update other than the AdminState." };
        	return true;
        }
		
	}else{
		eth.id = _convertEthernetID(eth.id);
		return updateEthernetInterfaceDeprecated(eth);
	}
}

def updateEthernetInterfaceDeprecated(eth)
{
    
    
    def subchildren = [];
    
    if (eth.madminstate)
        subchildren << [
            name: 'mAdminState', 
            content: eth.madminstate
        ];
    
    if (eth.use_dhcp)
        subchildren << [
            name: 'UseDCHP', 
            content: eth.use_dhcp
        ];
    
    
    if (eth.ipaddress)
        subchildren << [
            name: 'IPAddress', 
            content: eth.ipaddress
        ];
    
    if (eth.mtu)
        subchildren << [
            name: 'MTU', 
            content: eth.mtu
        ];
    
    if (eth.defaultgateway)
        subchildren << [
            name: 'DefaultGateway', 
            content: eth.defaultgateway
        ];
    
    if (eth.mode)
        subchildren << [
            name: 'Mode', 
            content: eth.mode
        ];
    
    
    def response;
    
    // Handle the delete case first.
    if (eth.containsKey("ipaddress") && !eth.ipaddress) {
    	response = executeApplianceRequest([
    	     name: 'soma:del-config',
    	     content: [:],
    	     children: [
    	         [
    	             name: 'EthernetInterface',
    	             content: [name: eth.id]
    	         ]
    	     ]
    	]);
    }
    // If we're not deleting the IP, just modify the part we want.
    else {
	    response = executeApplianceRequest([
	        name: 'soma:modify-config',
	        content: [:],
	        children: [
	            [
	                name: 'EthernetInterface',
	                content: [name: eth.id],
	                children: subchildren
	            ]
	        ]
	    ]);
    }
    
    if (!response?.result?.equals('OK'))
        return false;
        
    saveApplianceConfiguration();
    return true;
}
