/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.appliance.NTPUtils;

def getNtpServers()
{
	if(BedrockUtils.isBedrock()){
		return NTPUtils.getTimeServers();
	}else{
		return getNtpServersDeprecated()
	}
}
def getNtpServersDeprecated()
{
    def response = executeApplianceRequest([
        name: 'soma:get-config', 
        content: [
            class: 'NTPService'
        ]
    ]);
    
    def servers = response?.config?.NTPService?.RemoteServer ?: [];
    
    if (!(servers instanceof List))
        servers = [servers];
    
    return servers;
}
def updateNtpServers(servers){
	if(BedrockUtils.isBedrock()){
		return NTPUtils.setTimeServers(servers as String[]);
	}else{
		return updateNtpServersDeprecated(servers)
	}
}

def updateNtpServersDeprecated(servers)
{
	def theChildren = servers.collect {
        [ 
	         name: 'RemoteServer', 
	         content: it 
         ]
	};
	
    if(theChildren.size() == 0){
        theChildren = [[ name : 'RemoteServer', content: null ]]
    }

	def enableStanza = [ name: 'mAdminState', 
	 	                content: 'enabled'
	 	                ];
	
	theChildren.add(0, enableStanza); // mAdminState MUST be first.
	
	def response = executeApplianceRequest([
        name: 'soma:modify-config',
        content: [:], 
        children: [
            [
                name: 'NTPService', 
                content: [:], 
                children: theChildren
            ]
        ]
    ]);
            
    if (!response?.result?.equals('OK'))
        return false;
        
    saveApplianceConfiguration();
    return true;
}
