/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

// current here for referernce; this is the orignal cloudburst version as is; this will need to be changed, in conjunction with bedrock changes,
// to manipulate config and the brSNMPd process

import com.ibm.websphere.rainmaker.logging.MessageUtils;

def getTrapSeverity() 
{
	return getSNMPSettings()?.TrapPriority;
}

def getTrapSubscribers() {
	def settings = getSNMPSettings();
	
	def targets = settings?.Targets;
	
	if (targets == null)
		return [];
	
	if (!(targets instanceof List))
        targets = [targets];

	/*Temporary Hack: Since we can't delete the last trap subscriber due to a
                          broken DP SOMA call, a bogus trap subscriber is        
                          defined at 127.0.0.1.  This allows all other           
                          subscribers to be deleted.                             
                                                                                 
                          Filter the bogus one out here:            */           
                                                                                 
        def retTargets = [];                                                     
        targets.each { target ->                                                 
           if(!target.Host.equals("127.0.0.1"))                                  
               retTargets << target;                                     
         }                                                            
                                                                      
        return retTargets;  
}

def getTrapSubscriptions() {

	def supportedCodes = [ 	
            [ "0x02220001", "${MessageUtils.message(user.lang[], "RM16050", []) }", "${MessageUtils.message(user.lang[], "RM16055", []) }", false ], 
            [ "0x02240002", "${MessageUtils.message(user.lang[], "RM16052", []) }", "${MessageUtils.message(user.lang[], "RM16056", []) }", false ], 
            [ "0x02220003", "${MessageUtils.message(user.lang[], "RM16050", []) }", "${MessageUtils.message(user.lang[], "RM16057", []) }", false ], 
            [ "0x02220004", "${MessageUtils.message(user.lang[], "RM16050", []) }", "${MessageUtils.message(user.lang[], "RM16058", []) }", false ], 
            [ "0x02220005", "${MessageUtils.message(user.lang[], "RM16050", []) }", "${MessageUtils.message(user.lang[], "RM16059", []) }", false ], 
            [ "0x00850001", "${MessageUtils.message(user.lang[], "RM16053", []) }", "${MessageUtils.message(user.lang[], "RM16060", []) }", false ],
            [ "0x00830002", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16061", []) }", false ],
            [ "0x00830003", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16062", []) }", false ], 
            [ "0x02a30002", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16063", []) }", false ], 
            [ "0x02a30005", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16064", []) }", false ], 
            [ "0x02a30006", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16065", []) }", false ], 
            [ "0x02a30007", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16066", []) }", false ], 
            [ "0x01a30002", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16067", []) }", false ], 
            [ "0x01a30003", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16068", []) }", false ], 
            [ "0x01a50004", "${MessageUtils.message(user.lang[], "RM16053", []) }", "${MessageUtils.message(user.lang[], "RM16069", []) }", false ], 
            [ "0x01a30006", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16070", []) }", false ], 
            [ "0x01a50007", "${MessageUtils.message(user.lang[], "RM16053", []) }", "${MessageUtils.message(user.lang[], "RM16071", []) }", false ], 
            [ "0x01a30009", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16072", []) }", false ], 
            [ "0x01a5000a", "${MessageUtils.message(user.lang[], "RM16053", []) }", "${MessageUtils.message(user.lang[], "RM16073", []) }", false ], 
            [ "0x01a2000e", "${MessageUtils.message(user.lang[], "RM16050", []) }", "${MessageUtils.message(user.lang[], "RM16074", []) }", false ], 
            [ "0x01a6000f", "${MessageUtils.message(user.lang[], "RM16054", []) }", "${MessageUtils.message(user.lang[], "RM16075", []) }", false ], 
            [ "0x01a30014", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16076", []) }", false ],
            [ "0x01a30015", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16077", []) }", false ], 
            [ "0x01a30017", "${MessageUtils.message(user.lang[], "RM16051", []) }", "${MessageUtils.message(user.lang[], "RM16078", []) }", false ], 
            [ "0x01a5001a", "${MessageUtils.message(user.lang[], "RM16053", []) }", "${MessageUtils.message(user.lang[], "RM16079", []) }", false ] 
        ];
	

	def settings = getSNMPSettings();
	
	def systemCodes = settings?.TrapEventCode;
    
	/********************************************/
	/* Mark the currently enabled codes as such */
	/********************************************/
	
	/* O(m * n), where m ~50 and n should be < 100, but could possibly be ~500.  Not good, but this is
	   throwaway code, so not worth optimizing */
	if(systemCodes != null) {
		if (!(systemCodes instanceof List))
        	systemCodes = [systemCodes];
		
		systemCodes.each { code ->
			supportedCodes.each { supportedCode -> 
				if(supportedCode[0].equals(code)) {
					supportedCode[3] = true;
				}
			}
		}
	}
	
	return supportedCodes;
}

def updateTrapSubscribers(targets)
{

    def currentSettings = getSNMPSettings();                                                                 
                                                                                                              
    currentSettings = convertSNMPSettings(currentSettings);  
    def sendSettings = [];

    if(currentSettings == null) {
        logger.FINE { "Error obtaining SNMP settings through SOMA!" };
	return false;
    }

    sendSettings << [ name: 'mAdminState', content: 'enabled' ]; 

    targets.each { target ->
        def children = [ 
            [ name: 'Host',          content: target.Host ],
            [ name: 'Port',          content: target.Port ],
            [ name: 'Community',     content: target.Community ],
            [ name: 'TrapVersion',   content: target.TrapVersion ],
            [ name: 'SecurityName',  content: '' ],
            [ name: 'SecurityLevel', content: 'authPriv' ]
        ];
        sendSettings << [
            name: 'Targets', 
            content: [:], 
            children: children
        ];
        
    } 
  
    currentSettings.each { setting ->
        if(!setting.name.equals("mAdminState") && !setting.name.equals("Targets") )
            sendSettings << setting;
    }

    
    def response = executeApplianceRequest([
        name: 'soma:set-config',
        content: [:], 
        children: [
            [
                name: 'SNMPSettings', 
                content: [:], 
                children: sendSettings
            ]
        ]
    ]);
            
    if (!response?.result?.equals('OK'))
        return false;

    logger.FINE { "Updated snmp trap subscriptions and trap priority" };
    saveApplianceConfiguration();
    return true;
}

def isValidSeverity(severity) {                                                          
   def validSeverities = [ 'critic', 'warn', 'error', 'notice', 'info' ];                      
                                                                                   
   return  validSeverities.find{ it.equals(severity) } != null;                    
}                                                                                     
                                                                                   
def updateTrapSubscriptions(subscriptions)                                                         
{                                                                                              
    def currentSettings = getSNMPSettings();
    currentSettings = convertSNMPSettings(currentSettings);
          
    def sendSettings =  [  [ name: 'mAdminState', content: 'enabled' ] ];

    def setTrapCodes = false;                                                                             

    //Copy all of the existing SNMP settings except the TrapPriority and TrapCodes, obtain those
    //from the input hash (mAdminState is forced to the top above...ignore it as well) 
    
    currentSettings.each { setting ->
       //Maintain other SNMP Settings
       if(!setting.name.equals("mAdminState") && !setting.name.equals("TrapEventCode") &&
          !setting.name.equals("TrapPriority") ) {

           sendSettings << setting;
       }
       //Update the TrapPriority
       else if(setting.name.equals("TrapPriority")) {
           if(isValidSeverity(subscriptions.severity))
               sendSettings << [ name: 'TrapPriority', content: subscriptions.severity ];
           else
               sendSettings << setting;

           setTrapCodes = true;
       }

       //Once the TrapPriority has been added, add the TrapEventCodes.  Check for TrapPriority having
       //been added instead of waiting until TrapEventCode is found because TrapPriority is guaranteed
       //to be in the settings, but there may not be any TrapEventCodes.
       if(setTrapCodes) {
          subscriptions.traps?.each { trap ->
             sendSettings << [name: 'TrapEventCode', content: trap];
          }
          setTrapCodes = false;
       }
    }

    def response = executeApplianceRequest([
        name: 'soma:set-config',
        content: [:],
        children: [
            [
                name: 'SNMPSettings',
                content: [:],
                children: sendSettings
            ]
        ]
    ]);

    if (!response?.result?.equals('OK'))
        return false;

    saveApplianceConfiguration();
    return true;
}