/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package groovy.rainmaker.core;

import java.security.*;
import javax.crypto.KeyGenerator
import javax.crypto.KeyGenerator
import com.ibm.websphere.rainmaker.logging.MessageUtils;

/**
 * Executes ssh-keygen with the given arguments
 */
// def args = ['-t', 'dsa', '-P', '\"testing\"', '-f', 'test.pub'];
 
def keygen(passphrase, outputfile, stdout = null, stderr = null)
{
	if (! stdout)  
		stdout = { line -> logger.INFO { '[ssh-keygen stdout] ' + line } };
	if (! stderr)
		stderr = { line -> logger.WARNING { '[ssh-keygen  stderr] ' + line } }

	if (outputfile.exists() || new File(outputfile.getParentFile(), outputfile.name + '.pub').exists()) {
		throw new Exception(MessageUtils.message('RM15004', [outputfile.absolutePath]));
	}
	
	def cmdargs = ['-t', 'rsa', '-b', '2048', '-P', passphrase, '-f', outputfile.getAbsolutePath()];
	 
	def ret = [:];
	 def args = ([ "/usr/bin/ssh-keygen" ] + cmdargs).collect { it.toString() }
     def pb = new ProcessBuilder(args)
	
     try
     {
         p = pb.start()
         
     def stdoutThread = Thread.start {
        use (groovy.rainmaker.core.InputStreamUtils) {
            p.in.eachLine('ISO-8859-1', stdout)
        }
     }
    
     def stderrThread = Thread.start {
        use (groovy.rainmaker.core.InputStreamUtils) {
            p.err.eachLine('ISO-8859-1', stderr)
        }
     }

         p.waitFor();
         //Output.info("Exit Value: ${p.exitValue()}")
         stdoutThread.join()
         stderrThread.join()
     }
     
     catch (IOException e)
     {
        e.printStackTrace();
     }

     ret.privatekey = outputfile.absolutePath;
     ret.publickey = new File(outputfile.getParentFile(), outputfile.name + '.pub').absolutePath;
     ret.passphrase = passphrase;
     return ret;
}



def fingerprint(key, stdout = null, stderr = null)
{
	if (! stdout)
		stdout = { line -> logger.INFO { '[ssh-keygen stdout] ' + line } }
	if (! stderr)
		stderr = { line -> logger.WARNING { '[ssh-keygen  stderr] ' + line } }

	 def cmdargs = ['-l', '-f', key.getAbsolutePath()];
	 
		def ret = [:];
		 def args = ([ "/usr/bin/ssh-keygen" ] + cmdargs).collect { it.toString() }
	     def pb = new ProcessBuilder(args)
		
	     try
	     {
	         p = pb.start()
	         
	     def stdoutThread = Thread.start {
	        use (groovy.rainmaker.core.InputStreamUtils) {
	            p.in.eachLine('ISO-8859-1', stdout)
	        }
	     }
	    
	     def stderrThread = Thread.start {
	        use (groovy.rainmaker.core.InputStreamUtils) {
	            p.err.eachLine('ISO-8859-1', stderr)
	        }
	     }

	         p.waitFor();
	         stdoutThread.join()
	         stderrThread.join()
	     }
	     
	     catch (IOException e)
	     {
	        e.printStackTrace();
	     }
	     return stdout;

}

def keygen_java() {
	KeyGenerator keygen = KeyGenerator.getInstance("DSA");
	keygen.initialize(2048);
	KeyPair kp = keygen.generateKeyPair();
	PublicKey pub_key = kp.getPublic();
	PrivateKey priv_key = kp.getPrivate();
	
}