/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

import com.ibm.websphere.acl.PermissionsACL;

def onList()
{
    def resource = com.ibm.websphere.acl.AccessControls.parseResourcePath(request.path[]);
    def acl = getAccessControlList(resource.name, resource.id);
    
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = acl;
    render();
}

def onRetrieve()
{
    def entry = request.params.aclId[].tokenize('-');
    def type = entry[0];
    def id = entry[1].toInteger();
    
    def resource = com.ibm.websphere.acl.AccessControls.parseResourcePath(request.path[]);
    
    def acl = getAccessControl(resource.name, resource.id, id, type == 'group');
        
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = acl;
    render();
}

def onPostMember()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def entry = request.params.aclId[].tokenize('-');
    def type = entry[0];
    def id = entry[1].toInteger();
    
	def resource = com.ibm.websphere.acl.AccessControls.parseResourcePath(request.path[]);
    
    request.params.render.resourceType = resource.name;
    request.params.render.resourceID = resource.id;
    
    try
    {
        createAccessControl(resource.name, resource.id, id, type == 'group', PermissionsACL.READ, false);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08055', t);
    }
    
    request.status = 201;
    
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = PermissionsACL.READ;
    render();
}

def onUpdate()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def entry = request.params.aclId[].tokenize('-');
    def type = entry[0];
    def id = entry[1].toInteger();
    
    def permissions = Json.decode(request.input[]);
	def resource = com.ibm.websphere.acl.AccessControls.parseResourcePath(request.path[]);
    
    def success = updateAccessControl(resource.name, resource.id, id, type == 'group', permissions);    
    
    if (!success)
        return com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08001');
    
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = permissions;
    render();
}

def onDelete()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def entry = request.params.aclId[].tokenize('-');
    def type = entry[0];
    def id = entry[1].toInteger();
    
	def resource = com.ibm.websphere.acl.AccessControls.parseResourcePath(request.path[]);
    
    def success = deleteAccessControl(resource.name, resource.id, id, type == 'group');
    
    if (!success)
        return com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08001');
    
    request.status = 204;
}
