/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

def onList()
{
    // TODO - permission checks?

    def groupsid = request.params.groupsid[]?.toInteger();
    def usersid = request.params.usersid[]?.toInteger();

    if (groupsid && usersid) {
        if (isUserInGroup(group, user))
            request.json.output = [ groupsid: groupsid, usersid: usersid ];
        else
            request.json.output = [:]
    }
    else if (groupsid) {
        request.json.output = getGroupMembers(groupsid).collect { user ->
            [ groupsid: groupsid, usersid: user.id ]
        }
    }
    else if (usersid) {
        request.json.output = getUserGroups(usersid).collect { group ->
            [ groupsid: group.id, usersid: usersid ]
        }
    }

    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView(); 
    render();
}

def onRetrieve()
{
    def membership = request.params.groupsusersId[].tokenize('-');
    
    if (membership.size() < 2)
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
    
    def group = membership[0].toInteger();
    def user = membership[1].toInteger();
    
    def inGroup = false;
    
    try
    {
        inGroup = isUserInGroup(group, user);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08005', t);
    }
    
    request.status = inGroup ? 200 : 404;
}

def onCreate()
{
    def membership = null;
    
    try
    {
        membership = Json.decode(request.input[]);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        addGroupMember(membership.groupsid, membership.usersid);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08003', t);
    }
    
    request.status = 201;
}

def onDelete()
{
    def membership = request.params.groupsusersId[].tokenize('-');
    
    if (membership.size() < 2)
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
    
    def group = membership[0].toInteger();
    def user = membership[1].toInteger();
    
    try
    {
        removeGroupMember(group, user);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08005', t);
    }
    
    request.status = 204;
}
