/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.pdq.runtime.exception.DataRuntimeException

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;

def response = [:]

logger.FINE{ 'entering healthcheck' }

// free memory
def memory = Runtime.getRuntime().freeMemory();
response.put('freeMemory', memory);
logger.FINE{ 'free memory: ' + memory  }

// thread related activity
def threadCount = Thread.activeCount()
response.put('activeThreadsCount', threadCount );
logger.FINE{ '# of active threads: ' + threadCount  }

/*
def threads = new Thread[threadCount];
Thread.enumerate(threads);

def mapThreadState = [:]
for (Thread t : threads){
	mapThreadState.put(t.getName(),  t.getState() );
	logger.FINER{ 'Thread '+ t.getName() + ':' + t.getState()  }
}
response.put('activeThreadsState', mapThreadState );
*/

// db related checks
def data;
try{
    
    def tableNames = tmp.rainmaker.tablenames[];
    // create a ThreadLocal copy of the DataManager
    data = groovy.rainmaker.core.SQLUtilsStatics.getManager(config.db.key[], true)
    // start the transaction
    data.beginTransaction();
    if( ! tableNames){
	    def tableMap = []
	    def c = data.retrieveConnection();
	    
	    def md = c.getMetaData();
	    def rs = md.getTables(/*catalog*/null, /*schema*/'APP', /*tablenamePattern*/ "%", /*types*/ null);

	    while (rs.next()) {
	      def tableName = rs.getString(3);
	      def rowCount = getResourceCount(tableName,[:], false)
		  response.put(tableName + '_TABLE_ROW_COUNT', rowCount)
		  tableMap.add(tableName);
		  logger.FINER{ 'table: '+  tableName+ ' rowCount:' + rowCount  }
	    }
	    tmp.rainmaker.tablenames = tableMap;
    }else{
    	for (tableName in tableNames){
    	  def rowCount = getResourceCount(tableName,[:], false)    		
  		  response.put(tableName + '_TABLE_ROW_COUNT', rowCount)
		  logger.FINER{ 'table: '+  tableName+ ' rowCount:' + rowCount  }
    	}
    }
}catch (Throwable t){
	logger.SEVERE{ 'failed to perform health check on db' + t  }
	if( t instanceof DataRuntimeException){
		request.status =  503;
		response.put('Root cause failure', t.getMessage())
	}
	else{
		request.status =  503;
		response.put('Root cause failure', "Unexpected database error"  )
	}
}finally{
	// end transaction and release the manager 
	if(data){
		// release manager first since this is just clearing a thread local
		groovy.rainmaker.core.SQLUtilsStatics.releaseManager() 
		data.endTransaction();
		
	}
}


// everything looks good.. set to 200 and return the state
if(!request.status[]){
	request.status =  200;
}

request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView()
request.json.output=response
render()

logger.FINE{ 'completed healthcheck' }