/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.core.security.util.XOREncoder;
import zero.json.Json;

import com.ibm.websphere.users.Permissions;

def onList()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = config.security.userservice.ldap[];
    render();
}

def onPutCollection()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

    def ldapConfig = null;

    try
    {
        ldapConfig = Json.decode(request.input[]);
    }

    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }

    def password = ldapConfig.remove('value')?.trim();
    def verification = ldapConfig.remove('verification')?.trim();

    if (password)
    {
        if (!password.equals(verification))
            return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08006');

        ldapConfig['jndiSecurityCredentials'] = password;
    }

    if (!storage.rainmaker.security.ldap[])
        storage.rainmaker.security.ldap = [:];

    if (ldapConfig.containsKey('enabled')){
    	storage.rainmaker.security.enabled = ldapConfig.enabled;
    	zput(config.rainmaker.defaultPersistentZone[] + "/rainmaker/security/enabled" , ldapConfig.enabled);
    }

    ldapConfig.each {
        if (it.key == 'enabled')
            return;

        def decoded = it.value.replaceAll('&amp;', '&').trim();

        if (it.key == 'jndiSecurityPrincipal' && decoded.length() == 0)
        {
            _removeConfig('jndiSecurityPrincipal');
            _removeConfig('jndiSecurityCredentials');
            return;
        }

        if (it.key == 'jndiSecurityCredentials')
            decoded = XOREncoder.encode(decoded);

        config.security.userservice.ldap[it.key] = decoded;
        //storage.rainmaker.security.ldap[it.key] = decoded;
        

     }
    zput(config.rainmaker.defaultPersistentZone[] + "/rainmaker/security/ldap" , config.security.userservice.ldap[]);

    logger.FINEST { "The LDAP configuration loaded in memory is:\n${Json.encode(config.security.userservice.ldap[], true)}" };
    logger.FINEST { "The LDAP configuration saved in storage is:\n${Json.encode(config.security.userservice.ldap[], true)}" };
}

def _removeConfig(key)
{
    config.security.userservice.ldap[].remove(key);
    // storage.rainmaker.security.ldap[].remove(key);
    //def newldap = storage.rainmaker.security.ldap[];
    //newldap.remove(key)
    //zput("/storage/rainmaker/security/ldap", newldap);
    zput(config.rainmaker.defaultPersistentZone[] + "/rainmaker/security/ldap", config.security.userservice.ldap[]);
}

