/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rest.RenderUtils;

def onList()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return RenderUtils.renderError(403, 'RM08008');

    def roles;
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    if (userid != null) {
        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
        roles = getUserRoles(id);
    } else if (groupid != null) {
        roles = getGroupRoles(groupid);
    }

    request.view = getRequestedView();
    request.json.output = roles;
    render();
}

def onPostMember()
{
    def session = user.session[];
    def role = request.params.rolesId[].toInteger();
    def resourceType = request.params.render.resourceType[];
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    if (userid != null) {

        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN) || id == session.id)
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot add a role to the user if user is already in a group other than 'Everyone' group
        if (isUserInMoreThanOneGroup(id)) {
            return RenderUtils.renderError(400, 'RM08077');
        }

        addUserRole(id, role);
        invalidateSession(id);
    } else if (groupid != null) {
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN))
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot add a role to 'Everyone' group
        if (groupid.toInteger() == 1) {
            return RenderUtils.renderError(400, 'RM08075');
        }
        addGroupRole(groupid, role);
        // invalidate all group members session except the current user session
        getGroupMembers(groupid.toInteger()).each { user ->
            if (user && user.id != session.id)
                invalidateSession(user.id)
        }
    }

    request.status = 201;
}

def onDelete()
{
    def session = user.session[];
    def role = request.params.rolesId[].toInteger();
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    def success = true;
    if (userid != null) {
        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();

        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN) || id == session.id || id == Permissions.SUPER_USER)
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot remove a role from the user if user is already in a group other than 'Everyone' group
        if (isUserInMoreThanOneGroup(id)) {
            return RenderUtils.renderError(400, 'RM08078');
        }

        success = deleteUserRole(id, role);
        invalidateSession(id);
    } else if (groupid != null) {
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN))
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot remove a role from 'Everyone' group
        if (groupid.toInteger() == 1) {
            return RenderUtils.renderError(400, 'RM08076');
        }

        success = deleteGroupRole(groupid, role);
        // invalidate all group members session except the current user session
        getGroupMembers(groupid.toInteger()).each { user ->
            if (user && user.id != session.id)
                invalidateSession(user.id)
        }
    }

    request.status = success ? 204 : 404;
}
