/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.rest;

import zero.core.events.EventEngine
import zero.core.events.dispatcher.OneDispatcher
import zero.json.Json;


def _capitalize(s)
{
    return s.equals('ips') ? 'IPs' : capitalize(s);
}

def _getResourceScript(type)
{
    return "groovy/rainmaker/${config.rainmaker.resources[type]}/${_capitalize(type)}.groovy";
}


def _logEnter()
{
    request.params.render.startTime = System.currentTimeMillis();
    logger.FINE { "Processing HTTP request: ${request.method[]} ${request.path[]} for user ${user.session[].username}" };
}

def _logExit()
{
    def start = request.params.render.startTime[];
    def end = System.currentTimeMillis();
    logger.FINE { "Completed HTTP request: ${request.method[]} ${request.path[]} for user ${user.session[].username} in ${(end - start) / 1000} seconds." };
}

def handleOnUpdateHasMany(manyres, attr, attrval, newids)
{
	_logEnter();
    def script = _getResourceScript(manyres);
    def currentids = invokeMethod(script, "get${_capitalize(manyres)}", [ "${attr}": attrval ], user.session[].id).collect { it.id };

    def updater = "update${singularize(_capitalize(manyres))}";
    def success = true;

    (currentids - newids).each {
        def updates = [ id: it ];
        updates[attr.toString()] = null;

        success &= invokeMethod(script, updater, updates);
    }

    (newids - currentids).each {
        def updates = [ id: it ];
        updates[attr.toString()] = attrval;

        success &= invokeMethod(script, updater, updates);
    }
    _logExit();
    return success;
}


def handleOnUpdateManyMany(res1, id1, res2, newid2s)
{
	_logEnter();
    def joinTable = [ res1, res2 ].sort().join();

    def res1attr = "${singularize(res1)}id";
    def res2attr = "${singularize(res2)}id";

    def joinRows = getResources(joinTable, [ "${res1attr}": id1 ], false, [ res2attr, 'id' ]);

    joinRows.each { row ->
        if (! newid2s.contains(row[res2attr]))
            deleteResource(joinTable, row.id);
    }

    newid2s.each { newid2 ->
        if (! joinRows.find { it[res2attr] == newid2 })
            createResource(joinTable, [ "${res1attr}": id1, "${res2attr}": newid2 ], false);
    }            
    _logExit();
    return true;
}