/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import com.ibm.websphere.rainmaker.logging.Output;

def root = new File(config.rainmaker.disk.root[]);

if("".equals(config.rainmaker.appName[])){
	if (!root.exists() && onAppliance())
	    Output.fail("Hard drive is not mounted.");
}
else{		// only create the base drouter directory and not children.
	if (!root.getParentFile().exists() && onAppliance())
	    Output.fail("Hard drive is not mounted.");
}

config.rainmaker.directories[].each {
    def subDir = new File(root, it);
    
    if (subDir.exists())
        return;
    
    Output.info("Creating directory ${subDir.absolutePath}...");
    
    if (!subDir.mkdirs())
        Output.fail("Unable to create directory ${subDir.absolutePath}.");
    
    if (subDir.exists()) {
        Output.info("Created directory ${subDir.absolutePath}...");
    }
    else {
        Output.fail("Failed to create ${subDir.absolutePath}...");
    }
}

def target = new File(config.root[]);  
def properties = getResolvedProperties(target);  

// generate secret key per appliance for security
if(properties["ibm:rainmaker.rest.pd"]){
	
	def secretKeyConfigFile = new File (config.rainmaker.disk.shared[], "/config/security-secretkey.config")
	Output.info("secretKeyFile " + secretKeyConfigFile);
	if(!config.security.secretKey[]){
		if(!secretKeyConfigFile.exists()){
			new File (secretKeyConfigFile.getParent()).mkdirs();	//create directories if needed
			secretKeyConfigFile.createNewFile();	// create file
		}
		def secretKey = zero.core.security.encryption.SecretKeyGenerator.generateSecretKey()
		Output.info ("Creating secretKey config");
		secretKeyConfigFile.write ("/config/security/secretKey=\"${secretKey}\"")

//		Output.info ("Updating private secret key file");
		//	 update hidden file to point to drouter config file if necessary
//		def hiddenSecretKeyConfigFile = new File (properties["ibm:rainmaker.rest.pd"], "/config/.security-secretkey.config")
//		hiddenSecretKeyConfigFile.text = "#Auto generated @ " + new java.util.Date() +". Do not check in updates\n#Revert to SVN revision if delete drouter\n@include \"\${/config/rainmaker/disk/shared}/config/security-secretkey.config\"";
	}
}	

if(config.rainmaker.setup.skipdbconfig[])
    return;

def dbConfig = getDatabaseConfig();
def dbType = null;

try
{
    dbType = getDatabaseType(dbConfig['class']);
    Output.info("The selected database vendor is ${getDatabaseVendor(dbType)}.");
}

catch (Throwable t)
{
    Output.fail(t.message);
}

def targetVersion = null;

if (!event.args[].isEmpty())
{
    targetVersion = event.args[][0];
    
    if (!(targetVersion ==~ /^\d+\.\d+\.\d+\.\d+$/))
        Output.fail("The target version must be of the form w.x.y.z (all numeric). You provided ${targetVersion}.");
}

if (!targetVersion){
    targetVersion = config.rainmaker.version[].tokenize('-')[0];
    if (!(targetVersion ==~ /^\d+\.\d+\.\d+\.\d+$/)){
    	if ((targetVersion ==~ /^\d+\.\d+\.\d+$/)){
    		targetVersion = targetVersion + '.0';
    	}
    }
}

Output.info("The target version is ${targetVersion}.");

def currentVersion = null;

//
// TODO: This path has to be replaced with one on the hard disk
//
def dbDir = config.rainmaker.disk.databases[];
def versionFile = new File(dbDir, 'version');

if (versionFile.exists())
    currentVersion = versionFile.text;

def noPreviousSetup = currentVersion == null;

if (currentVersion && !(currentVersion ==~ /^\d+\.\d+\.\d+\.\d+$/))
    Output.fail("The current version has an invalid format - the file at ${versionFile.absolutePath} may have been corrupted.");

else
    Output.info("The current version is ${currentVersion ?: '0.0.0.0 (setup has not been run)'}.");

if (noPreviousSetup)
    currentVersion = '0.0.0.0';

if (currentVersion == targetVersion)
    Output.exit('Target version is the same as the current version - no work will be done.');

def isRollback = targetVersion < currentVersion;
Output.info("The application will be ${isRollback ? 'rolled back' : 'upgraded'} from version ${currentVersion} to ${targetVersion}.");

def names = properties['dependencies'].tokenize(',').reverse();
names << "_self";

Output.info("Searching for SQL files in ${target.name} and its dependencies...");


def versions = [currentVersion, targetVersion] as Set;

try
{
    names.each {
    	if (isLocal(it) || it.equals("_self")){
        	if(properties[it] != null){
        		versions.addAll(getVersions(properties[it], dbType));
        	}
    	}
    }
}

catch (Throwable t)
{
    Output.fail(t.message);
}

versions = versions.flatten().sort();

def index1 = versions.indexOf(currentVersion);
def index2 = versions.indexOf(targetVersion);

if (isRollback)
    index2++;

else
    index1++;

def relevantVersions = versions[index1..index2];

relevantVersions.each { version -> 
    names.each {
    	if (isLocal(it) || it.equals("_self")){
        	if(properties[it] != null){
	            if (isRollback)
	                drop(target, properties[it], dbType, version);
	            
	            else
	                create(target, properties[it], dbType, version);
        	}
        }
    }
}

if (noPreviousSetup && properties['ibm:rainmaker.users'])
{
    try
    {
        invokeScript('groovy/rainmaker/users/LoadSettingsHandler.groovy');
        
        config.rainmaker.alwaysJoinThreads = true;
        
        Output.info('Creating all-inclusive public group...');        
        createEveryoneGroup();
        
        if (properties['ibm:rainmaker.acl'])
        {
            //
            // add all default capabilities to the public group
            //
            Output.info('Setting default permissions for the public group...');
            
            ['scripts', 'fixes', 'templates', 'maintenance', 'patterns', 'part2s', 'instances'].each {
                createAccessControl(it, 0, 1, true, com.ibm.websphere.acl.PermissionsACL.DELETE);
            }
        }

        Output.info('The public group was created successfully.');
        
        Output.info('Creating admin account...');        
        createAdmin();
        Output.info('The admin account was created successfully.');
        
        if (properties['ibm:rainmaker.backup'])
        {
            Output.info('Creating default backup profile...');        
            createBackupProfile([
                name: 'Default Backup Profile'
            ]);
            Output.info('The default backup profile was created successfully.');
        }
    }
    
    catch (Throwable t)
    {
    	Output.info("this is the spot" + t.printStackTrace())
        Output.error(CleanStackTrace.cleanStackTrace(t));
        Output.fail("Could not create default data: ${t.message}");
    }
}

if (!isRollback)
{
    relevantVersions.each { version -> 
        names.each {
        	if (isLocal(it) || it.equals("_self")){
                if(properties[it] != null){
            		sample(target, properties[it], dbType, version);
            	}
            }
        } 
    }
}    
  
Output.info('All SQL files have been executed.');

versionFile.parentFile.mkdirs();
versionFile.text = targetVersion;
 
shutdown();

Output.info('Setup is complete.');
