/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def renderPermissionLink(userID, accessLevel, ownerID, isGroup = false)
{
    return "<span class='default'>[<span id='acl-permission-${isGroup? 'group' : 'user'}-${userID}'>${renderToggleACL(userID, accessLevel, ownerID, isGroup)}</span>]</span>";
}

def renderToggleACL(userID, accessLevel, ownerID, isGroup = false)
{
    def userIsOwner = userID == ownerID;
    def clientIsOwner = (user.session[].id == ownerID) || isAdmin(user.session[].id);
    
    def nlsKey = userIsOwner && !isGroup ? 'RM10031' : getAccessLevelKey(accessLevel);
    def nlsText = MessageUtils.message(user.lang[], nlsKey, []);
    
    if ((userIsOwner && !isGroup) || !clientIsOwner) {
        return nlsText;
    }
    
    def uri = "/resources/${request.params.render.resourceType[]}/${request.params.render.resourceID[]}";
    
    def newAccessLevel = PermissionsACL.NONE;
    
    switch (accessLevel)
    {
        case PermissionsACL.READ:
            newAccessLevel = PermissionsACL.UPDATE;
            break;

        case PermissionsACL.UPDATE:
            newAccessLevel = PermissionsACL.CREATE;
            break;

        case PermissionsACL.CREATE:
            newAccessLevel = PermissionsACL.DELETE;
            break;
            
        case PermissionsACL.DELETE:
            newAccessLevel = PermissionsACL.READ;
            break;
        
        default:
            throw new IllegalStateException("Invalid state transition from '${MessageUtils.message(user.lang[], getAccessLevelKey(accessLevel), [])}'.");
    }
    
    return "<a href=\"javascript:document.dashboard.details.toggleACL('${uri}', ${userID}, ${isGroup}, ${newAccessLevel}, 'acl-permission-${isGroup? 'group' : 'user'}-${userID}');\" title=\"${MessageUtils.message(user.lang[], 'RM10112', [])}\">${nlsText}</a>";
}
