/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def renderAddIPRelationship(htmlId, resourceID)
{
    def progressText = MessageUtils.message(user.lang[], 'RM10643', []);
    return """
        <span class='default'><label for='${htmlId}-add-ip-from'>${MessageUtils.message(user.lang[], 'RM10227', [])}</label> 
        <input type='text' class='relationship-filter-text-ip' id='${htmlId}-add-ip-from'/> &nbsp;
        <label for='${htmlId}-add-ip-to'>${MessageUtils.message(user.lang[], 'RM10228', [])}</label>
        <input type='text' class='relationship-filter-text-ip' id='${htmlId}-add-ip-to'/> &nbsp;
        <a id='${htmlId}-add-ip' href='javascript:void(0);'>${MessageUtils.message(user.lang[], 'RM10229', [])}</a>
        </span>
        <span style='margin-left: 10px;'>
        <img src='/images/progress-indicator-small.gif' id='${htmlId}-add-ip-progress' class='icon-image' style='vertical-align: top; display: none;' 
             alt="${progressText}" title="${progressText}"/>
        </span>
        <br/><br/>
        <div class='error' style='display: none;' id='${htmlId}-add-ip-error'></div>
        """;
}

def renderAddRelationship(htmlId, resourceID)
{
    return """<label for='${htmlId}-add-text-${resourceID}'>
    		<!-- accessible label offscreen to describe input text -->
    		<div class="label-hide">${MessageUtils.message(user.lang[], 'BR10070', [])}</div></label>
    		<input type='text' class='relationship-filter-text' id='${htmlId}-add-text-${resourceID}'/><br/>
    		<div class='add-relationship-results' id='${htmlId}-add-results-${resourceID}' style='display: none; visibility:hidden'></div>"""
}

def renderAddScriptKey(htmlId, resourceID)
{
    return "<span class='default'>${MessageUtils.message(user.lang[], 'RM10244', [])}" + 
           "<label for='${htmlId}-add-scriptkey-name' style='display: none;'> </label><input type='text' class='relationship-filter-text-scriptkey' id='${htmlId}-add-scriptkey-name'/> &nbsp;=" +
           "<label for='${htmlId}-add-scriptkey-value' style='display: none;'> </label><input type='text' class='relationship-filter-text-scriptkey' id='${htmlId}-add-scriptkey-value'/> &nbsp;" + 
           "<a id='${htmlId}-add-scriptkey' href='javascript:void(0);'>${MessageUtils.message(user.lang[], 'RM10229', [])}</a></span>";
}

/**
 *
 * renderBelongsToLink() generates hyperlinked text to a resource that 'owns' a given resource.
 *
 * @param name
 *        The name of the field (column) that is a foreign key to another table.
 *
 * @param value
 *        The ID of resource Y, such that "resource X belongs to resource Y".
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
def renderBelongsToLink(name, value)
{
    //
    // database table = foreign key field - 'id'
    //
    def type = "${name[0..<name.length() - 2]}s";
    
    if (name == 'ownerid')
        type = 'users';
    
    else if (name == 'groupid')
        type = 'groups';
    
    def resource = getCachedResource(type, value);
    
    if (!resource)
        return "<span class='default form-prompt'>${MessageUtils.message(user.lang[], 'RM10067', [])}</span>";

    return renderLink(type, resource);
}

/**
 *
 * renderChildManyManyLinks() generates hyperlinked text to a set of resources that are 
 * related to a specified child resource of the given resource by JOIN tables.
 *
 * @param type
 *        The collection that may have resources related our resource.
 *
 * @param id
 *        The ID of our resource, to be used as a foreign key into the collection.
 *
 * @param owner
 *        The ID of the user who is making the request, for authorization purposes.
 *
 * @param htmlId
 *
 * @param mutable
 *        True if the list should have add/remove links in it.
 *
 * @param childType
 *        The child type of the current resource that may have resources related to the type.
 *
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
// TODO - see if we can collapse with renderManyManyLinks() above
def renderChildManyManyLinks(type, id, owner, htmlId, mutable, childType)
{
    def thistype = request.params.render.resourceType[];
    def thisowner = owner;
    def html = "";

    // Do not consider owner for tables that do not have an owner
    if (!config.rainmaker.ownership[].contains(type))
        thisowner = false;

    // Get the child id for the child join to render
    ArrayList join = getJoinedResources(childType, [:], thisowner, thistype, id);
    if (join != null && join.size() > 0) {

        def childId = join.get(0)?.id;

        // Get the child join and render
        def resources = getJoinedResources(type, [:], thisowner, childType, childId);
        html = renderLinkList(type, resources, mutable, thistype, id);
    }

    if (mutable)
        html = [html, renderAddRelationship(htmlId, id)].join('<br/><br/>');
    
    return html;
}

def renderDeleteHypervisorResource(type1, id1, type2, id2)
{
    def hypervisor = getResource(type2, ['id':id2]);

    // Optionally disable hypervisor remove
    if ([Status.STARTED, Status.STARTING].contains(hypervisor?.currentstatus)) {
        return "";
    }
    return " <span class='default'>[<a href=\"javascript:document.dashboard.details.deleteHypervisor('${type1}', ${id1}, '${type2}', ${id2});\">${MessageUtils.message(user.lang[], 'RM10009', [])}</a>]</span>";
}

def renderDeleteNestedResource(type1, id1, type2, id2)
{
    return " <span class='default'>[<a href=\"javascript:document.dashboard.details.deleteNestedResource('${type1}', ${id1}, '${type2}', ${id2});\">${MessageUtils.message(user.lang[], 'RM10009', [])}</a>]</span>";
}

def renderDeleteRelationship(type1, id1, type2, id2)
{
    return " <span class='default'>[<a href=\"javascript:document.dashboard.details.deleteResourceRelationship('${type1}', '${id1}', '${type2}', ${id2});\">${MessageUtils.message(user.lang[], 'RM10009', [])}</a>]</span>";
}

/**
 *
 * renderHasManyLinks() generates hyperlinked text to a set of resources 'owned' 
 * by a given resource.
 *
 * @param type
 *        The collection that may have resources 'owned' by our resource.
 *
 * @param id
 *        The ID of our resource, to be used as a foreign key into the collection.
 *
 * @param owner
 *        The ID of the user who is making the request, for authorization purposes.
 *
 * @param mutable
 *        True if the list should have add/remove links in it.
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
def renderHasManyLinks(type, id, owner, htmlId, mutable, options = [:])
{
    def fullTypeName = request.params.render.resourceType[];
    def thistype = fullTypeName[0..-2];
    def fkAttr = thistype == 'user' ? 'owner' : thistype;

    def filters = [
        "${fkAttr}id": id
    ];
    
    def thisowner = owner;
    
    // Do not consider owner for tables that do not have an owner
    if (!config.rainmaker.ownership[].contains(type) || type == 'part2s')
        thisowner = false;
    
    else
        filters.order = 'name';

    if (options.orderBy)
        filters.order = options.orderBy;
        
    def canWrite = !config.rainmaker.ownership[].contains(fullTypeName) || canUpdate(fullTypeName, id, user.session[].id);
    
    def resources = getResources(type, filters, thisowner);
    def html = renderLinkList(type, resources, (mutable && canWrite), "${thistype}s", id, options);
    
    if (mutable && canWrite)
        html = [html, "${options.addRenderer ?: 'renderAddRelationship'}"(htmlId, id)].join('<br/><br/>');
    
    else if (resources.size() <= 4)
        html += '<br/><br/>';
        
    return html;
}

/**
 *
 * renderManyManyLinks() generates hyperlinked text to a set of resources that are 
 * related to a given resource by a JOIN table.
 *
 * @param type
 *        The collection that may have resources related our resource.
 *
 * @param id
 *        The ID of our resource, to be used as a foreign key into the collection.
 *
 * @param owner
 *        The ID of the user who is making the request, for authorization purposes.
 *
 * @param mutable
 *        True if the list should have add/remove links in it.
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
def renderManyManyLinks(type, id, owner, htmlId, mutable, options = [:])
{
    def thistype = request.params.render.resourceType[];
    def thisowner = owner;
    
    if (thistype == 'groups' && id == 1)
        return "<span class='default'><em>${MessageUtils.message(user.lang[], 'RM02015', [])}</em></span>";

    // Do not consider owner for tables that do not have an owner
    if (!config.rainmaker.ownership[].contains(type))
        thisowner = false;
        
    def canWrite = !config.rainmaker.ownership[].contains(thistype) || canUpdate(thistype, id, user.session[].id);
    
    def resources = getJoinedResources(type, [:], thisowner, thistype, id);
    def html = renderLinkList(type, resources, (mutable && canWrite), thistype, id, options);
    
    if (mutable && canWrite)
        html = [html, "${options.addRenderer ?: 'renderAddRelationship'}"(htmlId, id)].join('<br/><br/>');
    
    else if (resources.size() <= 4)
        html += '<br/><br/>';
    
    return html;
}

/**
 *
 * renderLink() generates hyperlinked text to a different page.
 *
 * @param type
 *        The server-side type that should be rendered on the destination page.
 *
 * @param resource
 *        The ID of the resource to highlight on the destination page.
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
def renderLink(type, resource, deletable = false, parentType = false, parentID = false, options = [:])
{
    //
    // TODO: put these in zero.config so we don't create the map every time
    //
    def viewsByType = [
        instancesview: 'virtual', 
        instances: 'virtual', 
        patterns: 'patterns', 
        templates: 'catalog/images', 
        scripts: 'catalog/scripts', 
        fixes: 'catalog/fixes', 
        clouds: 'cloud/cloud-groups', 
        hypervisors: 'cloud/hypervisors', 
        storage: 'cloud/storage', 
        subnets: 'cloud/subnets', 
        vlans: 'cloud/vlans', 
        users: 'collective/users', 
        groups: 'collective/user-groups',
        networks: 'cloud/hypervisors'
    ];
    
    //
    // when non-admins look at the cloud monitoring page, they shouldn't 
    // be able to follow links to hypervisor data
    //
    if (type == 'hypervisors' && !user.session[].permissions.contains(Permissions.CLOUD_ADMIN))
        return renderText(resource.name, 'hypervisor-link');
    

    // By default, link is the resource name
    def linkText = resource.name;

    def cssClass = 'default';
    def status = resource.currentstatus;

    //
    // HACK: IPs do not have a name column
    //
    if (! linkText && resource.ipaddress) {
        linkText = resource.ipaddress;
        logger.FINEST { "Looking up the host name for ${resource.ipaddress}..." };
        
        def hostName = InetAddress.getByName(resource.ipaddress).getHostName();
        if (hostName && hostName != resource.ipaddress)
            linkText += " (${hostName})";
        else {
            // force FAILED icon
            status = 'RM01013';
            cssClass = 'error';
            linkText += " (${MessageUtils.message(user.lang[], 'RM10342', [])})";
        }

        logger.FINEST { "The host name for ${resource.ipaddress} is ${hostName ?: 'unknown'}." };
    }
    
    //
    // HACK: show script keys as an equation w/ default value
    //
    if (!linkText && resource.scriptkey)
    {
        def hasDefault = resource.scriptdefaultvalue?.length();
        linkText = "${resource.scriptkey} = ${hasDefault ? resource.scriptdefaultvalue : MessageUtils.message(user.lang[], 'RM10245', [])}";
    }
    
    //
    // HACK: rendering a physicalmachineid on a page, results in an unrecognized type "physicalmachines"
    //       instead of "physicalMachines" in the link
    //
    if (type == "physicalmachines")
        type = "physicalMachines";

    def html = '';

    if (options.statusIcon)
        html += renderStatus(status) + '&nbsp;';

    html += options.plainText ?
        "<span class='${cssClass}'>${linkText}</span>" :
        "<span class='${cssClass}'><a href=\"/dashboard/${viewsByType[type]}/#${resource.id}\" title=\"${MessageUtils.message(user.lang[], 'RM10020', [linkText])}\">${linkText}</a></span>";

        
    //
    // HACK: you can't remove things from the Everyone group
    //
    if (type == 'groups' && resource.id == 1)
        deletable = false;
        
    if (deletable)
        html += "${options.deleteRenderer ?: 'renderDeleteRelationship'}"(parentType, parentID, type, resource.id);

    return html;
}

def renderLinkList(type, resources, mutable = false, parentType = false, parentID = false, options = [:])
{
    //
    // HACK: for IP addresses, sort numerically instead of lexicographically
    //
    if (type == 'ips')
        resources = _sortIPs(resources);
        
    // 
    // HACK: for IP groups, mutability is determined individually for each link in the list
    //
    def links = null;
    if (type == "hypervisors" && parentType == "subnets") {
        links = resources.collect { renderLink(type, it, it.mutable, parentType, parentID, options) };
    } else {
        links = resources.collect { renderLink(type, it, mutable, parentType, parentID, options) };
    }
    
    if (links.isEmpty())
        links << "<span class='default form-prompt'>${MessageUtils.message(user.lang[], 'RM10067', [])}</span>";
    
    if (resources.size() > 4)
    {
        def id = "show-more-${type}-results";
        links[4] = """
            <span class='default'>
            [<a href='javascript:void(0);' onClick="toggleRelationshipSection('${id}', true, '${type}-relationship-toggle');"><span id='${type}-relationship-toggle'>${MessageUtils.message(user.lang[], 'RM10441', [])}</span></a>]
            </span>
            <div id='${id}' style='display: none;'>
            <br/>
        """ + links[4];
        links[-1] += '</div>';
    }
    
    return links.join('<br/><br/>');
}

/**
 *
 * renderParentManyManyLinks() generates hyperlinked text to a set of resources that are 
 * related to a specified parent resource of the given resource by JOIN tables.
 *
 * @param type
 *        The collection that may have resources related our resource.
 *
 * @param id
 *        The ID of our resource, to be used as a foreign key into the collection.
 *
 * @param owner
 *        The ID of the user who is making the request, for authorization purposes.
 *
 * @param htmlId
 *
 * @param mutable
 *        True if the list should have add/remove links in it.
 *
 * @param parentType
 *        The parent type of the current resource that may have resources related to the type.
 *
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
// TODO - see if we can collapse with renderManyManyLinks() above
def renderParentManyManyLinks(type, id, owner, htmlId, mutable, parentType)
{
    def thistype = request.params.render.resourceType[];
    def thisowner = owner;
    def html = "";

    // Do not consider owner for tables that do not have an owner
    if (!config.rainmaker.ownership[].contains(type))
        thisowner = false;

    // Get the child relationships
    ArrayList join = getJoinedResources(type, [:], thisowner, thistype, id);
    if (join != null && join.size() > 0) {
        
        def resources = null;
        join.each {
            def childid = it?.id;

            // Now get the parents of a child and add to results
            def currentRes = getJoinedResources(parentType, [:], thisowner, type, childid);
            if (resources == null) {
                resources = currentRes;
            } else {
                if (currentRes != null && currentRes.size() > 0) {
                    for (int i=0; i < currentRes.size(); i++) {
                        resources.add(currentRes.get(i));
                    }
                }
            }
        }
        // Render final list of parents
        html = renderLinkList(parentType, resources, mutable, thistype, id);
    }

    if (mutable)
        html = [html, renderAddRelationship(htmlId, id)].join('<br/><br/>');

    return html;
}

/**
 *
 * renderSubnetNetworkLinks() generates hyperlinked text to a hypervisor
 * on the subnets page through the networks of the hypervisor.
 *
 * @param type
 *        The collection that may have resources related our resource.
 *
 * @param id
 *        The ID of our resource, to be used as a foreign key into the collection.
 *
 * @param owner
 *        The ID of the user who is making the request, for authorization purposes.
 *
 * @param htmlId
 *
 * @param mutable
 *        True if the list should have add/remove links in it.
 *
 * @param parentType
 *        The parent type of the current resource that may have resources related to the type.
 *
 *
 * @return Hyperlinked text to a destination page showing the desired type/resource.
 *
 */
def renderSubnetNetworkLinks(type, subnetID, owner, htmlId, mutable, options = [:], parenttype)
{
    def thistype = request.params.render.resourceType[];
    def thisowner = owner;

    // Do not consider owner for tables that do not have an owner
    if (!config.rainmaker.ownership[].contains(type))
        thisowner = false;

    // Find the networks that connect to the subnet
    def networks = getResources('networks', [subnetid:subnetID], false);

    // For each network, find it's owning hypervisor
    def hypHash = new HashMap();
    def hypList = new ArrayList();
    networks.each {
        def rela = getResource('hypervisorsphysicalmachines', [physicalmachineid:it?.physicalmachineid]);
        if (rela) {
            def hyp = getResource('hypervisors', ['id':rela?.hypervisorid]);

            if (hyp && !hyp.isEmpty()) {
                hypHash.put(hyp?.id, hyp);
            }
        }
    }

    // Put unique hypervisor hash values into an ArrayList for rendering
    hypList = hypHash.values();
    hypList.each {
        if ([Status.STARTED, Status.STARTING].contains(it?.currentstatus)) {
            it.mutable = false;
        } else {
            it.mutable = true;
        }
    }

    // Build links to the hypervisor
    def html = renderLinkList(parenttype, hypList, mutable, thistype, subnetID, options);
    
    if (hypList.size() <= 4)
        html += '<br/><br/>';

    return html;
}

def _sortIPs(resources)
{
    resources.each { ip -> 
        ip.tokenized = ip.ipaddress.tokenize('.').collect { it.toInteger() }
    }
    
    return resources.sort({ ip1, ip2 ->
        for (i in 0..4) {
            if (ip1.tokenized[i] < ip2.tokenized[i])
                return -1;
            
            else if (ip1.tokenized[i] > ip2.tokenized[i])
                return 1;
        }
        
        return 0;
    });
}
