/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

def _addFiles(pattern, compressed, file, alreadyDone)
{
    if (file.isDirectory())
    {
        info "Searching directory ${file.name}...";

        def matches = [];
        file.eachFileMatch(pattern) { matches << it }

        matches.sort().each {
            _addFiles(pattern, compressed, it, alreadyDone);
        }
    }
    
    else if (!alreadyDone.contains(file))
    {
        info "Adding contents of ${file.name} to compressed file...";
        
        compressed.withWriterAppend {
            it << "/* start of source from file ${file.name}  */\n\n";
            it << file.text;
            it << '\n\n';
        }
        
        alreadyDone << file;
        
        info "Done working on ${file.name}.";
    }
}

def css = [
    'public/style'
];

def javascript = [
    'public/javascript/lib/prototype.js', 
    'public/javascript/lib/effects.js', 
    'public/javascript/lib', 
    'public/javascript/widgets/base-widget.js', 
    'public/javascript/widgets/modal-dialog.js', 
    'public/javascript/widgets/form-dialog.js', 
    'public/javascript/widgets/detail-panel.js', 
    'public/javascript/widgets/property-panel.js', 
    'public/javascript/widgets'
];

def compressedCSS = new File(config.root[], 'public/style/compressed.css');
def compressedJS = new File(config.root[], 'public/javascript/compressed.js');

if (compressedCSS.exists())
    compressedCSS.text = '';

if (compressedJS.exists())
    compressedJS.text = '';

info "Compressing CSS content in ${compressedCSS.name}...";

def alreadyDone = [] as Set;

css.each {
    def file = new File(config.root[], it);
    _addFiles(~/.*\.css/, compressedCSS, file, alreadyDone);
}

info 'Done with CSS compression.';

info "Compressing JavaScript content in ${compressedJS.name}...";

alreadyDone.clear();

javascript.each {
    def file = new File(config.root[], it);
    _addFiles(~/.*\.js/, compressedJS, file, alreadyDone);
}

info 'Done with JavaScript compression.';
