dojo.require("dijit.Tooltip");
var Bedrock = {
 	ScriptFragment: '<script[^>]*>([\\S\\s]*?)<\/script>',

	toArray: function(element) {
	    if ( element && element.split ) {
    		return element.split();
    	}
    	else {
    		return null;
    	}
	},
	
	indexOf: function(obj, k, i) {
		if ( obj.indexOf ) {
	   		return obj.indexOf(k);
	   	}
	   	else if ( dojo.isArray(obj) ) {
  			i || (i = 0);
  			var length = obj.length;
  			if (i < 0) i = length + i;
  			for (; i < length; i++) {
    			if (obj[i] === k) return i;
    		}
  			return -1;
	   	}
	   	else {
	   		return -1;
	   	}
	},

	$A: function(iterable) {
  		if (!iterable) return [];
  		if (Bedrock.toArray(iterable)) return Bedrock.toArray(iterable);
  		var length = iterable.length, results = new Array(length);
  		while (length--) results[length] = iterable[length];
  		return results;
	},

	extractScripts: function(response) {
    	var matchAll = new RegExp(Bedrock.ScriptFragment, 'img');
    	var matchOne = new RegExp(Bedrock.ScriptFragment, 'im');
		return dojo.map(response.match(matchAll) || [], 
			function(_scriptTag) {
    	  		return (_scriptTag.match(matchOne) || ['', ''])[1];
      		}
      	);
    	
  	},

  	evalScripts: function(response) {
  	    var scripts = Bedrock.extractScripts(response);
  	    dojo.forEach(scripts, function(script) {
  	    	//console.log("eval:" + script);
  	    	try {
  	    		eval(script);
  	    	}
  	    	catch ( e ) {
  	    		console.debug( e );
  	    		console.debug("need to convert " + script );
  	    	} 
  	    });
  	},
  	
	delay: function(_func) {
    	var __method = _func, args = Bedrock.$A(arguments), timeout = args.shift() * 1000;
    	return window.setTimeout(function() {
      		return __method.apply(__method, args);
    	}, timeout);
  	},
  	
  	isHidden: function(element) {
    	if ( dojo.style(element, 'display') == 'none') {
    		return true;
    	}
    	else {
			return false;
    	}
  	},

	show: function(element, options) {
		if(!options) options = {}
		
    	if ( element ) {
			dojo.style(element, { 	'visibility' : options.visibility || '',
									'display' : options.display || '', 
									'opacity' :  options.opacity || '100'
									
								}
			);
		}
		else {
	    	console.debug("can't show undefined element");
		}
	},

	hide: function(element) {
		dojo.style(element, 'display', 'none'); 
		dojo.style(element, 'visibility', 'hidden');
	},

	toggle: function(element) {
    	if ( Bedrock.isHidden(element)) {
    		Bedrock.show(element);
    	}
    	else {
    		Bedrock.hide(element);
    	}
  	},
  	
	addProperties: function(destination, source) {
  		for (var property in source) {
    		destination[property] = source[property];
    	}
  		return destination;
	},

	keys: function(object) {
    	var keys = [];
    	for (var property in object)
      		keys.push(property);
    	return keys;
	},

	cleanWhitespace: function(element) {
  		element = dojo.byId(element);
  		var node = element.firstChild;
  		while (node) {
    		var nextNode = node.nextSibling;
    		if (node.nodeType == 3 && !/\S/.test(node.nodeValue)) {
      			element.removeChild(node);
      		}
    		node = nextNode;
 		}
  		return element;
	},

	descendants: function(element) {
  		return $A(dojo.byId(element).getElementsByTagName('*')).each(Element.extend);
	},

	down: function(element, expression, index) {
  		try {
  			element = dojo.byId(element);
  			if (arguments.length == 1) return Bedrock.firstDescendant(element);
  			var descendants = descendants(element);
  			return expression ? Selector.findElement(descendants, expression, index) : descendants[index || 0];
  		}
  		catch ( err ) {
     		console.debug(err);
  		}
	},

	firstDescendant: function(element) {
    	element = dojo.byId(element).firstChild;
    	while (element && element.nodeType != 1) element = element.nextSibling;
    	return dojo.byId(element);
	},

	getEntry: function(element, index) {
    	return element.firstChild.childNodes[index];
	},

	render: function(entryCount, element) {
    	if(entryCount > 0) {
      		for (var i = 0; i < entryCount; i++) {
        		// this.index==i ? Bedrock.addClassName(getEntry(element,i),"selected") : Bedrock.removeClassName(getEntry(element, i),"selected");
      			//if(this.hasFocus) {
        		Bedrock.show(element);
        		//this.active = true;
        		//}
      		}
    	} else {
      		//this.active = false;
      	Bedrock.hide(element);
    	}
	},

	addClassName: function(element, className) {
    	if ( element ) {
			dojo.addClass(element, className); 
    	}
    	else {
    		console.log("addClassName, element is undefined");
    	}
	},

	removeClassName: function(element, className) {
		dojo.removeClass(element, className); 
	},

	fadeSplash: function() {
		return Bedrock.fadeCommon('splash-screen');
	},

	fadeCommon: function(nodeName) {
		dojo.fadeOut({
	    	node: nodeName,
    		duration: 1000,
	    	onEnd: function(){
	        	dojo.style(nodeName, 'display', 'none');
    		}
		}).play();

	},

	strip: function(object) {
    	return object.replace(/^\s+/, '').replace(/\s+$/, '');
	},

	isNumber: function(object) {
    	return typeof object == "number";
  	},

	isEmpty: function(str) {
    	return str == '';
	},
	
	createTooltip: function(target_id, content) {
		// prepending 'tt_' to distinguish between target and tooltip id's
		console.log("using Bedrock.tooltips");
		var obj = document.getElementById('tt_'+target_id);
		if(obj != null){
	        obj.parentNode.removeChild(obj);
		}
	
    	var tooltip = new dijit.Tooltip({
        	label: content,
   	    	connectId: [target_id]
   		});
   		
	 	dojo.byId(target_id).setAttribute("tabIndex", 0);
	 	
  		tooltip.domNode.id = 'tt_'+target_id;
   		document.body.appendChild(tooltip.domNode);
	},
	hashCode: function(stringInstance){
		var hash = 0;
		if ((!stringInstance.length)  || stringInstance.length == 0) return hash;
		for (i = 0; i < stringInstance.length; i++) {
	        char = stringInstance.charCodeAt(i);
	        hash = 31*hash+char;
	        hash = hash & hash; // Convert to 32bit integer
	    }
	    return hash;
	},	
	_lastFunction: function() {
		// no function after this one.
	}
};
