/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// DetailPanel is a base class for all panels that appear inside the right-hand side of 
// a page (in DetailView). It provides all of the generic mechanisms for AJAX requests, 
// scrolling, and changes in what resource is being displayed.
//
dojo.provide('javascript.widgets.DetailPanel');
dojo.require('javascript.widgets.BaseWidget');
dojo.declare('javascript.widgets.DetailPanel', javascript.widgets.BaseWidget, {   
    
    _createFieldsQueryString: function(fields) {
        //
        // specify fields to show in the panel using the following query param:
        //
        // fields={key1}:{value1}:{mutable1};{key2}:{value2}:{mutable2};...
        //
        var keys = this._getFieldQueryStringKeys();
        
		var tokens = dojo.map(fields, function(field){

            var param = "";
            
            for (var i = 0; i < keys.length; ++i)
            {
            	var value = field[keys[i]];
                param += value;
                
                if (i < keys.length - 1) {
                    param += ':';
                }
            }
            
            return param;
        });
        return tokens.join(';');
    },
    
    _getFieldQueryStringKeys: function() {
		// moving relatedType to last...
        return ['key', 'label', 'mutable', 'fieldType', 'relatedtype'];
    },
    
    _getScrollableElement: function() {
        return this.element;
    },
    
    _getScrollPosition: function() {
        return this.scrollPosition;
    },
    
    initialize: function(parent, name, mimeType, options) {
		console.log("detail-panel initialize called");
        this.inherited(arguments, [parent, { 
            id: name, 
            'class': 'dashboard-detail-panel' 
        }]);
 		
        this.options = {
            autoRefresh: false, 
            display: true
        };
        Bedrock.addProperties(this.options, options || {});
        
        this.name = name;
        this.mimeType = mimeType;
        
        this._updateFrequency = 10 * 1000;
        
        this.scrollPosition = {
            top: 0, 
            left: 0 
        };
        
        //
        // create backwards link from panel (child) to view (parent)
        //
        if (this.options.display) {
            parent.panel = this;
        }
    },
    
    constructor: function(parent, name, mimeType, options) {
    	if ( this._initialize ) {
    		this._initialize = false;
	        this.initialize(parent, name, mimeType, options); 
    	}
    },
    
    onScroll: function() {
        var element = this._getScrollableElement();
        this.scrollPosition.top = element.scrollTop;
        this.scrollPosition.left = element.scrollLeft;
    },
    
    prepareForDisplay: function(dontShowAfter) {
        this._resetScrollPosition();
        
        //
        // update icon for item in lefthand list after all 
        // JS from HTML update has been executed. if we do 
        // this immediately, the status values won't be 
        // updated to the new values yet (because the DOM 
        // hasn't been parsed and script executed).
        //
        setTimeout(dojo.hitch(this, this._updateStatusIcon), 1500);

        
        var detailSplash = dojo.byId('detail-splash-screen');
        
        if (dojo.style(detailSplash, 'display') != 'none'){
            Bedrock.hide(detailSplash);
        }
            
        if (!dontShowAfter){
            Bedrock.show(this.element, {'display': 'block'} );
		}

        
        var pageSplash = dojo.byId('splash-screen');
        
        if (dojo.style(pageSplash, 'display') != 'none'){
		    dojo.fadeOut({
				node: pageSplash,
				duration: 1000
		    }).play();
		}
    },
    
    prepareForFailure: function(response) {
        //
        // HACK: wrap response HTML in a div that will give us the alignment we want. we 
        //       can't use DOM/CSS APIs directly because we don't get the opportunity to 
        //       modify the DOM in between the time it's parsed by prototype and the time 
        //       it's rendered on screen
        //
        response.responseText = "<div class='error-with-margins'>" + response.responseText + "</div>";
    },

    prepareForUpdate: function(response, ioArgs) {
    	var _update = true;
    	if ( ioArgs ) {
    		if ( ioArgs.xhr.status == 201 ) {
    			_update = false;
    			console.log("live updates are done");
    			// add call to refresh view.
    			if ( this.listUpdate ) {
    				console.log("call list update");
    				this.listUpdate();
    			}
    		}
    	}
		if ( _update && this.options.autoRefresh ) {
			clearTimeout(this._autoUpdate);
			this._autoUpdate = setTimeout(dojo.hitch(this, this._update, this.element, this.ajaxOptions ), this.ajaxOptions.frequency );
	    }
    },
    
    setListUpdate: function(update) {
    	this.listUpdate = update;
    },
    
    _resetScrollPosition: function(top, left) {
        var element = this._getScrollableElement();
        
        if (element)
        {
            element.scrollTop = top != undefined ? top : this.scrollPosition.top;
            element.scrollLeft = left != undefined ? left : this.scrollPosition.left;
        }
    },
        
    restore: function() {
        this.isMaximized = false;
    },
    
    scroll: function(top, left) {
        this._resetScrollPosition(top, left);
        this.onScroll();
    },
    
    //
    // selecteResource() modifies the continuous AJAX request to point to 
    // a new URI/ID and asks for an immediate refresh.
    //
    selectResource: function(uri, id, fields, dontShowAfter) {
    	console.log("detail-panel, selectResource");
        var fieldsParam = this._createFieldsQueryString(fields);
        
        //
        // ID indicates we're targeting an individual resource - adjust URI
        //
        this.uri = uri;
        if (id != null)
            this.uri += '/' + id;
        
        //
        // if this isn't the first time we're selecting a resource, 
        // we have to stop AJAX before making changes
        //
       	console.log("_startUpdates");
        this._startUpdates(this.uri, fieldsParam, dontShowAfter);
    },
    
    showDefault: function(uri) {
        this.element.innerHTML = '';
    },

    _update: function(element, options){
    	var params = options.content;
   		var section = dojo.byId('form-table-messages-section');
   		if ( section ) {
   			console.log("found section");
	    	if (dojo.style(section, 'display') == 'none') {
   				console.log("keep section collapsed");
   				params['messageSection'] = 'collapsed';
			}
			else {
   				console.log("keep section expanded");
   				params['messageSection'] = 'expanded';
			}
		}
		else {
   			console.log("did not find section");
		}
        console.log("params", params);
                
		dojo.mixin(options, {
	    	load: function(response, ioArgs){
	    	    if ( options.evalScripts ) {
	    	        try {
	    	        	window.setTimeout(function() {Bedrock.evalScripts(response)}, 1000);
	    	    	}
	    	    	catch ( e ) {
	    	    		console.debug(e);
	    	    	}
	    	    }
	    	    // can we check the response status?
	    	    //console.log("response is ", response);
	    	    var priorResponseHashCode = element.hashCode;
	    	    var currentReponseHashCode = Bedrock.hashCode(response);
	    	    console.log('DetailPanel._update() priorResponseHashCode', priorResponseHashCode);
	    	    console.log('DetailPanel._update() currentReponseHashCode', currentReponseHashCode);
	    	    if(priorResponseHashCode != currentReponseHashCode){ 
					element.innerHTML = response;
					element.hashCode = currentReponseHashCode;
				}else{
					console.log('DetailPanel._update() content has not changed!');
				}
				this.onSuccess(response, ioArgs);
	    	},
	    	//error: function(response, ioArgs){
	    	//	console.log('DetailPanel._update() error continue requesting');
	    	//	this.onSuccess(response, ioArgs);
	    	//},
	    	content: params
		});
		console.log("options", options);
		dojo.xhrGet(options);
    },

    
    _startUpdates: function(uri, fields, dontShowAfter) {
        var params = {}; //this._getAjaxParameters();
        
        if (fields)
            params['fields'] = fields;
                    
        this.ajaxOptions = {
            frequency: this._updateFrequency,
            url: uri, 
            decay: 2, 
			headers: {
                'Accept': this.mimeType 
            },
            //parameters: params,
		    content: params,
            evalScripts: true, 
            handle: dojo.hitch(this, this.prepareForDisplay, dontShowAfter),  // go back there after refresh
            onSuccess: dojo.hitch(this, this.prepareForUpdate),
            onFailure: dojo.hitch(this, this.prepareForFailure)
        };

		if ( this.options.autoRefresh ) {
			clearTimeout(this._autoUpdate);
			this._autoUpdate = setTimeout(dojo.hitch(this, this._update, this.element, this.ajaxOptions ), this.ajaxOptions.frequency );
	    }
	    
    	this._update(this.element, this.ajaxOptions);

    },
    
	destroy: function() {
		clearTimeout(this._autoUpdate);
	},
	
    
    _updateStatusIcon: function() {
        var id = this.uri.split('/').pop();
       	var statusIcon = dojo.byId('dashboard-list-view-status-' + id);
        
       	if (statusIcon && this.parent.currentStatusIcon)
       	{
           	statusIcon.src = this.parent.currentStatusIcon;
           	statusIcon.alt = statusIcon.title = this.parent.currentStatusMessage;
       	}
    }
    
});
