/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

//
// DetailView represents the right-hand side of a page, where information is displayed 
// about a particular resource. This view contains one or more panels which modify the 
// way the information is presented, with some panels being more appropriate for certain 
// tasks than others.
//

dojo.provide('javascript.widgets.DetailView');
dojo.require('javascript.widgets.ActionMenu');
dojo.require('javascript.widgets.BaseWidget');
dojo.require('javascript.widgets.ModalDialog');

dojo.declare('javascript.widgets.DetailView', javascript.widgets.BaseWidget, {   
    
    addEnumeration: function(uri, key, label, mutable, level) {
        this.addField(uri, 'enum-' + key, label, mutable, level);
    },
    
    addExternalField: function(uri, key, label, level) {
        this.addField(uri, 'external-' + key, label, false, level);
    },
    
    //
    // addField() ensures that a field in a resource (JSON object) is 
    // displayed somewhere in the panels. The last parameter is a flag 
    // that tells the renderer whether the field should have an HTML 
    // input widget associated with it or just plain text.
    //
    addField: function(uri, key, label, mutable, level, relatedType, fieldType) {
        if (!this.fields[uri]) {
            this.fields[uri] = [];
        }
        console.log("fieldType is " + fieldType);
        
        if (relatedType == null) {
            relatedType = '';
        }

        if (fieldType == null) {
            fieldType = 'normal';
        }

        this.fields[uri].push({
            'key': key, 
            'label': label, 
            'mutable': mutable, 
            'level': level,
            'relatedtype': relatedType,
            'fieldType':fieldType
        });
    },

    //
    // addHasManyRelationship() is a convenience method that calls addField() 
    // with additional metadata to indicate a has-many relationship between 
    // the resource and the objects represented by the field.
    //
    addHasManyRelationship: function(uri, key, label, mutable, options) {
        options = options || {};

        var prefix = 'hasmany';

		for(var k in options){
			prefix = prefix + '@' + k + '=' + options[k];
        }

        this.addField(uri, prefix + '-' + key, label, mutable, options.level);
    },
    
    //
    // addManyManyRelationship() is a convenience method that calls addField() 
    // with additional metadata to indicate a many-to-many relationship between 
    // the resource and the objects represented by the field.
    //
    addManyManyRelationship: function(uri, key, label, mutable, options) {
        options = options || {};

        var prefix = 'manymany';

		for(var k in options){
			prefix = prefix + '@' + k + '=' + options[k];
        };


        this.addField(uri, prefix + '-' + key, label, mutable, options.level);
    },
    
    //
    // addChildManyManyRelationship() is a convenience method that calls addField() 
    // with additional metadata to indicate a many-to-many relationship between 
    // a child of the resource of a given type, and the objects represented by the
    // field.
    //
    addChildManyManyRelationship: function(uri, key, label, childtype, mutable, level) {
        this.addField(uri, 'childmanymany-' + key, label, mutable, level, childtype);
    },
    
    //
    // addParentManyManyRelationship() is a convenience method that calls addField() 
    // with additional metadata to indicate a many-to-many relationship between 
    // a parent of the resource of a given type, and the objects represented by the
    // field.
    //
    addParentManyManyRelationship: function(uri, key, label, parenttype, mutable, level) {
        this.addField(uri, 'parentmanymany-' + key, label, mutable, level, parenttype);
    },
    
    //
    // addSubnetNetworkRelationship() is a convenience method that calls addField() 
    // with additional metadata to indicate a has-many relationship between 
    // a subnet and a network.
    //
    addSubnetNetworkRelationship: function(uri, key, label, parenttype, mutable, options) {
        options = options || {};
        this.addField(uri, 'subnetnetwork-' + key, label, mutable, options.level, parenttype);
    },
    
    addResourceRelationship: function(type1, id1, type2, id2) {
    	console.log("addResourceRelationship, type1: " + type1 + ", id1: " + id1 + "type2: " + type2 + ", id2: " + id2);
        var joinTable = [type1, type2].sort().join('');
        var relationship = {};
        relationship[type1 + 'id'] = id1;
        relationship[type2 + 'id'] = id2;
        
        var selectResource = dojo.hitch(this, this.selectResource, '/resources/' + type1, id1);

        
		dojo.xhrPost({
		    url: '/resources/' + joinTable,
		    headers: {
			'Content-Type': 'application/json'
		    },
		    postData: dojo.toJson(relationship),
	            handle: selectResource
         });
    },
    
    addToACL: function(type, id, userID, isGroup) {
    	console.log("addToACL", type, id, userID, isGroup);
		dojo.xhrPost({
		    url: '/resources/' + type + '/' + id + '/acl/' + (isGroup ? 'group' : 'user') + '-' + userID,
            headers: {
                 'Accept': 'application/rainmaker+acl+entry' 
             },
            load: function(response) {
                 dojo.byId('acl-next-entry-' + id).innerHTML += response;
             }
        });
    },

    deleteNestedResource: function(type1, id1, type2, id2) {
        var selectResource = dojo.hitch(this, this.selectResource, '/resources/' + type1, id1);

		dojo.xhrDelete({
		    url: '/resources/' + type1 + '/' + id1 + '/' + type2 + '/' + id2,
	    	handle: selectResource
         });
    },

    deleteResourceRelationship: function(type1, id1, type2, id2) {
        var joinTable = [type1, type2].sort().join('');        
        var selectResource = dojo.hitch(this, this.selectResource, '/resources/' + type1, id1);

        
        //
        // HACK: for deleting relationships, we create a URI where the ID is the 
        //       concatentation of the IDs of the two resources involved in the 
        //       relationship. the IDs must be in the right order, with the first 
        //       ID belonging to the type that is first in the join table name
        //
        if (type2 == joinTable.substring(0, type2.length))
        {
            var swap = id1;
            id1 = id2;
            id2 = swap;
        }
        
		dojo.xhrDelete({
		    url: '/resources/' + joinTable + '/' + id1 + '-' + id2,
		    handle: selectResource
         });
    },
    
    //
    // deleteResource() pops up a dialog confirming if a user wants to remove 
    // an object from the system.
    //
    deleteResource: function(uri, id) {
        if (document.dialog) {
            return;
        }
            
        var submitHandler = dojo.hitch(this, this._onDeleteRequest, uri, id);
        this.dialog = new javascript.widgets.ModalDialog('modal-dialog', messages['RM11001'], submitHandler);
        this.dialog.show();
    },
    
    getFields: function(uri) {
        return this.fields[uri];
    },
    
    initialize: function(parent, details) {
		console.log("detail-view initialize called");
		this.inherited(arguments );
    },
    
    constructor: function(parent, defaults) {
		console.log("detail-view ctr called");
        this.initialize(parent, { 
            id: 'detail-view', 
            'class': 'dashboard-detail-view'
        });        
        
        var options = {
            contentDefault: messages['RM11006']
        };
        Bedrock.addProperties(options, defaults || {});
        
        this.fields = {};
        this.panel = null;
        
        this.splash = document.createElement('div');
		this.splash.setAttribute('id', 'detail-splash-screen');
		dojo.addClass(this.splash, 'dashboard-detail-splash-screen');
        
        //this.splash = new Element('div', {
        //    id: 'detail-splash-screen',
        //    'class': 'dashboard-detail-splash-screen'
        //});
        
        var progressIcon = document.createElement('img');
		dojo.addClass(progressIcon, 'icon-image');
		progressIcon.setAttribute('src', '/images/progress-indicator-large.gif');
		progressIcon.setAttribute('alt', messages['RM11006']);
		
        //var progressIcon = new Element('img', {
        //    'class': 'icon-image', 
        //    'src': '/images/progress-indicator-large.gif',
		//		'alt': messages['RM11006']
        //});
        this.splash.appendChild(progressIcon);
        
        this.element.appendChild(this.splash);
        
        //this.splash.hide();
        Bedrock.hide(this.splash);
        
        this.defaultMessage = document.createElement('div');
        dojo.addClass(this.defaultMessage, 'dashboard-detail-view-default-message');
		this.defaultMessage.setAttribute('id', 'detail-default-message');

        this.defaultMessage.innerHTML = "<p align='center'><span class='dashboard-detail-view-default-message-text'>" + options.contentDefault + "</span></p>"        
        this.element.appendChild(this.defaultMessage);
        
        this.actions = new javascript.widgets.ActionMenu(this, options);
        
        parent.details = this;
        
        this.currentStatusIcon = null;
        this.currentStatusMessage = null;
    },
    
    _onDeleteRequest: function(uri, id, event) {
        var onDeleteResponse = dojo.hitch(this, this._onDeleteResponse);
        var onDeleteFailure = dojo.hitch(this, this._onDeleteFailure, uri, id);

		dojo.xhrDelete({
		    url: uri + '/' + id,
		    handleAs: 'json',
			load: onDeleteResponse,
			error: onDeleteFailure
         });

        
        if (event) {
            dojo.stopEvent(event);
		}
    },
    
    _onDeleteResponse: function() {
        this.dialog.hide();
        this.showDefault();
        this.parent.list.updateResources();
    },

    _onDeleteDismiss: function() {
    	console.log("_onDelete");
        this.dialog.hide();
        this.showDefault();
        this.parent.list.updateResources();
    },

    _onDeleteFailure: function(uri, id, response) {
    	try {
	    	console.log("_onDeleteFailure", response);
    	    var submitHandler = dojo.hitch(this, this._onDeleteDismiss);
    	    this.dialog = new javascript.widgets.ModalDialog('modal-dialog', response.responseText, submitHandler);
        	this.dialog.show();
    	}
    	catch ( e ) {
    	   alert(e);
    	}
    },
    
    removeFromACL: function(type, id, userID, isGroup) {
		dojo.xhrDelete({
		    url: '/resources/' + type + '/' + id + '/acl/' + (isGroup ? 'group' : 'user') + '-' + userID,
		    load: function(response){
		    	console.log("removeFromACL response is ", response);
		    	var _id = 'acl-entry-' + id + '-' + (isGroup ? 'group' : 'user') + '-' + userID;
                var entry = dojo.byId(_id);
                entry.parentNode.removeChild(entry);
             } 
         });

    },
    
    //
    // selectResource() modifies the data shown in the detail panels to be that 
    // of the resource with the given URI/ID. The action toolbar will also be 
    // updated to reflect the change.
    //
    selectResource: function(uri, id, dontShowAfter) {
        console.log("detail-view, selectResource");
        Bedrock.hide(this.defaultMessage);
        
        if (dojo.style('splash-screen', 'display') == 'none' && !dontShowAfter)
        {
            if (this.panel) {
                this.panel.hide();
                //Bedrock.hide(this.panel);
            }    
            //this.splash.show()
            Bedrock.show(this.splash);
        }
        
        this.actions.selectResource(uri, id);

        //
        // update resource in every panel so that when users switch, 
        // there's no lag in the rendering
        //
        var fields = this.fields[uri] || new Array();
        
        if (this.panel)
            this.panel.selectResource(uri, id, fields, dontShowAfter);
    },
    
    setCurrentStatus: function(icon, message) {
        this.currentStatusIcon = icon;
        this.currentStatusMessage = message;
    },
    
    showDefault: function(uri) {
        this.actions.showDefault(uri);
        
        if (this.panel)
            this.panel.showDefault(uri);
        
        Bedrock.show(this.defaultMessage);
    },
    
    toggleACL: function(uri, user, isGroup, access, htmlID) {
    	console.log("toggleACL", uri, user, isGroup, access, htmlID);
		dojo.xhrPut({
		    url: uri + '/acl/' + (isGroup ? 'group' : 'user') + '-' + user,
		    putData: dojo.toJson(access),
		    headers: {
				'Content-Type': 'application/json',
				'Accept': 'application/rainmaker+acl+permissions'
             },
             load: function(response) {
                dojo.byId(htmlID).innerHTML = response;
             }
        });    
    },
     
    //
    // updateResource() sends an HTTP PUT to the given URI along with the 
    // given JSON representation. The updated state is then shown in the 
    // selected panel.
    //
    updateResource: function(uri, id, json) {
        var callback = dojo.hitch(this, function(uri, id) {
            this.selectResource(uri, id, true);
            this.parent.list.updateResources();
        });
        
		dojo.xhrPut({
		    url: uri + '/' + id,
		    postData: json,
		    headers: {
				'Content-Type': 'application/json'
		    },
		    load: callback
         });
    },
    
    updateResourceField: function(uri, id, field) {
        var json = {};
        json[dojo.byId(field).name] = dojo.attr(field, 'value');
        this.updateResource(uri, id, dojo.toJSON(json));
    }
    
});
