/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// FormDialog is an extension of ModalDialog that has the ability to render form elements
// and submit them to a given REST API when the user clicks OK. It augments the ModalDialog
// <div/> by adding a <table/> with the form elements; these elements are removed upon
// submission of the form.
//

dojo.provide('javascript.widgets.FormDialog');
dojo.require("dijit.Tooltip");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.TextBox");
dojo.require("javascript.widgets.IPAddress");
dojo.require("javascript.widgets.ModalDialog");
dojo.require("dojo.NodeList-traverse");
dojo.require("dojox.validate._base");
dojo.require("dojox.validate.web");
dojo.declare('javascript.widgets.FormDialog', javascript.widgets.ModalDialog, {   

    _addValidationTest: function(inputElement, validation, callback) {
    	console.log("_addValidationTest");
        if (!inputElement.tests)
            inputElement.tests = new Array();
        
        if (!inputElement.messages)
            inputElement.messages = new Array();
        
        var builtIn = this._getBuiltInTest(validation);

        var test = builtIn ? builtIn.test : validation.test
        inputElement.tests.push(test);

        inputElement.messages.push(builtIn ? builtIn.message : validation.message);
        
        inputElement.callback = callback;

        inputElement.onblur = function() {
            var result = true;
            
            if (dojo.hasClass(this,'error'))
                result = false;
            
            else
            {
                if (dojo.hasClass( this, 'form-prompt'))
                {
                    this.value = '';
                    //this.removeClassName('form-prompt');
                    dojo.removeClass(this, 'form-prompt');
                }
                
                this.value = Bedrock.strip(this.value);
                
                for (var n = 0; n < this.tests.length && result; ++n)
                {
                    if (!this.tests[n](this))
                    {
                        this.value = this.messages[n];
                        //this.addClassName('error');
                        dojo.addClass(this, 'error');
                        
                        if (this.type != 'text') {
                            toggleTextBoxType(this, false);
                        }    
                        result = false;
                    }
                }
            }
            
            if (this.callback)
                this.callback(this);
                
            return result;
        }
    },

    _addValidationTests: function(inputElement, validations, callback) {
    	console.log("_addValidationTests");
        for (var i = 0; i < validations.length; ++i)
            this._addValidationTest(inputElement, validations[i], callback);
    },
    
    _adjustMemSize: function(element) {
    	console.log("_addMemSize");
        var size = parseInt(element.value);
        
        if (size < 1024)
            size = 1024;
        
        size -= size % 4;
        element.value = size;
        
        return true;
    },

    cleanup: function(id) {
    	console.log("cleanup");
        this.inherited(arguments);

        // remove the form data we added to the dialog
        var extensionRows = dojo.query('.' + id + '-dialog-extension-row');
		dojo.forEach(extensionRows, function(row) {
            row.parentNode.removeChild(row);
        });
				
		//remove span so it can be re-created later
		console.log("exit cleanup");
    },

    _createForm: function(id, uri, fields, alwaysScroll) {
    	//console.log("_createForm");
    	var f = this._nonHidden(fields);
    	//console.log("get non hidden fields", f); 
        if (f.length > 0) {
            this._createTable(id, uri, fields, alwaysScroll);
        }
    },

    _createJSON: function(fields, uri) {
    	//ignoring 3rd parameter passed in, "this" scope
    	console.log("_createJSON _jsonBuilder, entered");
        var json = {};

        //        
        // retrieve JSON data from the form elements
        //
        for (var i = 0; i < fields.length; i+=1) {
        	var name = fields[i].name;
        	console.log("_createJSON 0 fields.length is: ", fields.length); 
        	console.log("_createJSON 1 fields[",i,"].name is: ", name); 
        	if((uri== '/resources/users')&&(name == 'name'))
        	{
        		json[name] = Bedrock.strip(dojo.byId(name).value).length == 0 ? Bedrock.strip(dojo.byId('username').value) : Bedrock.strip(dojo.byId(name).value);
        	}
        	else if ((uri=='/resources/communities')&&(name == 'network')) {
        		// this is where we are fetching the values
        		var listOfURIs = this.hosts;
        		var commaSeparatedString = "";
        		if (listOfURIs.length > 0) {
	        		console.log("_createJSON 2 our final host values are: ", listOfURIs);
	        		console.log("_createJSON 3 length is: ", listOfURIs.length);
	        		for (var j=0; j<listOfURIs.length; j+=1) {
	        			if (j == 0) {
	        		      commaSeparatedString += listOfURIs[j];
	        			} else {
	        			  commaSeparatedString = commaSeparatedString + "," + listOfURIs[j];
	        			}
	        		}
	        		console.log("_createJSON entering network JSON area 4. commaSeparatedString is: ", commaSeparatedString);
	        		json[name] = Bedrock.strip(commaSeparatedString);
	        		console.log("entering network JSON area 5");
        		} else {
        			json[name] = '';
        		}
        	} else if ((uri=='/resources/appliances') && (name == 'hostName')){
        		var jsonVal = fields[i].hidden ? fields[i].value : Bedrock.strip(dojo.byId(name).value);
        		console.log("_createJSON: jsonVal for /resources/appliances hostName (before): ", jsonVal);
        		// convert IPv6 addresses to URL/runtime-friendly form surrounded by square brackets
        	    if(jsonVal.indexOf(":") != -1) {
        	    	jsonVal = "[" + jsonVal + "]";
        	    }
        		console.log("_createJSON: jsonVal for /resources/appliances hostName (after): ", jsonVal);
            	json[name] = jsonVal;
        	} else {
            	json[name] = fields[i].hidden ? fields[i].value : Bedrock.strip(dojo.byId(name).value);
        	}
        	console.log("_createJSON json[", name ,"]", json[name]);
		}
        // outputing the JSON associative array
        console.log("_createJSON exiting with full json map: ", json);
        return json;
    },
    
 
	_createHostRestrictionEditor: function (fieldName, fieldTitle, noValue, defaultValue) {
		console.log("entered _createHostRestrictionEditor with arguments: ", arguments);
		var priorEditor = dijit.byId(fieldName);
		if(priorEditor){
			var priorEditorParent = dojo.byId('editorCell');
	        
			// debug output
	        var cellChild = dojo.query("[id=editorCell]").children();
	        if (cellChild) {
			  dojo.forEach(cellChild, function(child) {
	              console.log("(A) an iteration of editorCell: ", child);
	          });
	        }
			// remove event listener and blow away the DOM node		
			priorEditor.disconnect();
			priorEditor.destroy();
			
			// debug output
	        var cellChild = dojo.query("[id=editorCell]").children();
	        if (cellChild) {
			  dojo.forEach(cellChild, function(child) {
	              console.log("(B) an iteration of editorCell: ", child);
	          });
	        }			
			
			// debug output
	        var cellChild = dojo.query("[id=editorCell]").children();
	        if (cellChild) {
			  dojo.forEach(cellChild, function(child) {
	              console.log("(C) an iteration of editorCell: ", child);
	          });
	        }
		}
		var scope = this;
		var domNode = dojo.byId(fieldName);
		var eb = new dijit.InlineEditBox({
			editor: "dijit.form.TextBox",
			autoSave: true,
			noValueIndicator: noValue,
			title: fieldTitle,
			value: defaultValue,
			style: "background-color: #ffffff;",
			'class': "inplace-form-table"
		},
		fieldName);
		eb.connect(eb, "onChange", dojo.hitch(scope, scope.addHostElement, noValue));
		console.log("exiting _createHostRestrictionEditor");
		return eb;
	},
	
	addHostElement: function(noValueProvided, value) {
		console.log("entered addHostElement with (noValueProvided, value):", arguments);
		var validationFlag = this._specialValidator('hostItemAdd', this, false);
		console.log("validation is: ", validationFlag);
		if (validationFlag == true) {			
			if (value == noValueProvided) {
			  console.log("addHostElement exiting, no value was provided");
	          return;
			}
		  
			value = Bedrock.strip(value);
			
			if(value == ''){
				console.log("addHostElement exiting, only whitespace was provided");
				return;
			}
			var scope = this;
		
		    // call getCurrentHostsTable
		    scope.addToCurrentHostsTable(value);
		} else {
			console.log("addHostElement skipped, validation flag is reset to: ", this.validationFlag);
		}
		console.log("exiting addHostElement");
	},
	
	validateURI: function(elementId) {
		// this is a special validation method for the 
		// host restriction field only, circumvents the other validation
		console.log("entered validateURI with element ID:", elementId);
        var result = true;
        var editor = dijit.byId(elementId);
        var elementRepeated = false;
        
        console.log("editor: ", editor);
        console.log("editor.value: ", editor.value);
    	console.log("entered validateURI 1");

    	editor.value = Bedrock.strip(editor.value);
    	    	
    	//var editorDisplayable = dojo.byId('hostItemAdd');
    	var editorDisplayable = dojo.byId(elementId);
        console.log("editorDisplayable: ", editorDisplayable);

        console.log("editorDisplayable.lastChild.nodeValue: ", editorDisplayable.lastChild.nodeValue);
    	console.log("entered validateURI 2");

		for (var i=0; i<this.hosts.length; i+=1) {
			if (this.hosts[i] == editor.value) {
				console.log("duplicate host entry found, validation fails");
				elementRepeated = true;
			}
		}		
		
		var isValidIP = dojox.validate.isIpAddress(editor.value, {allowDecimal: false}); 
		console.log("isValidIP: ", isValidIP);
		
		// RFC 1738 says that host can be (hostname | hostnumber)
		// hostnumber = digits "." digits "." digits "." digits

		var strArray = editor.value.split(".");
		var strArray2 = editor.value.split(":");
		var strArray3 = editor.value.split("::");
		
		var ipLike = true;
		var startsEndsWithUnsafeChar = false;
		var entryLength = editor.value.length;
			
		console.log("entryLength: ", entryLength);
		console.log("index of last char: ", editor.value.lastIndexOf('.'));
		if(editor.value.indexOf('/') == 0 || editor.value.indexOf('-') == 0
				|| editor.value.indexOf('_') == 0 || editor.value.indexOf('.') == 0
				|| editor.value.lastIndexOf('/') == entryLength - 1 || editor.value.lastIndexOf('-') == entryLength - 1
				|| editor.value.lastIndexOf('_') == entryLength - 1 || editor.value.lastIndexOf('.') == entryLength - 1) {
				startsEndsWithUnsafeChar = true;
		}
		
		console.log("startsEndsWithUnsafeChar: ", startsEndsWithUnsafeChar);
		
		var ipV4Like = true;
		var ipV6Like = true;
		
		var j=0;
		
		//we check to see if the provided entry is intended as a valid IPv4 address
		for(j=0; j<strArray.length; j+=1) {
			var octet = strArray[j];
			console.log("octet[",j,"]: ", octet);
			
			if (!strArray[j].match(/^([\d]*)$/)) {
				ipV4Like = false;
				//ipLike = false;
				break;
			}
			//if (isNaN(octet)) {
			//	ipLike = false;
			//	break;
			//}
		}

		var k=0;
		var hexit = "";
		console.log("strArray2 length: ", strArray2.length);
		console.log("strArray3 length (should be 2 or less): ", strArray3.length);
		
		//we check to see if the provided entry is intended as a valid IP address
		var pattern2 = /^([\da-fA-F]{1,4})(%[\w\d]+)?(\/[\d]+)?$/;
		
		// this pattern is kept for reference, we use pattern2, which is the 
		// same pattern, but in a different notation
		var patternIPv6 = "([\\da-fA-F]{1,4})(%[\\w\\d]+)?(\\/[\\d]+)?";
		
		// declaring the regexp object doesn't seem to work			
		var regExpIPv6 = new RegExp(patternIPv6, "g");
		
		for (k=0; k<strArray2.length; k+=1) {
			hexit = strArray2[k];
			console.log("hexit[", k ,"]: ", hexit, ". Matches: ", 
					pattern2.test(strArray2[k]), ". Matches regexp: ",  
					regExpIPv6.test(strArray2[k]), ". Matches blank: ", (hexit==""));
			
			// logic to detect more than one "::", which makes it invalid as IPv6 address
			if(strArray3.length > 2) {
				ipV6Like = false;
				break;
			}
 		
			if (!pattern2.test(hexit) && hexit != "") {
				ipV6Like = false;
				break;
			}
		}
		console.log("ipV4Like: ", ipV4Like);
		console.log("ipV6Like: ", ipV6Like);
		ipLike = (ipV4Like || ipV6Like);
		console.log("ipLike: ", ipLike);
        /*    			
		// /^((https?:\/\/)?[\w\d:#@%\/;$()~_?\+\-=\\\.&]*)$/
		*/
		
		// the below regex will prevent use of scheme (http(s)://), ":", "?", "&" and 
		// other character that net-snmp would not expect for a host restriction
		if (!editor.value.length == 0 && !ipV6Like 
				&& !editor.value.match(/^([\w\d\/_\-\.]*)$/)) {
    		console.log("entered validateURI 3a");
        	editorDisplayable.lastChild.nodeValue = messages['RM11098'];
            editor.value = '';
            dojo.addClass(editorDisplayable, 'error');               
            result = false;
        } else if (!editor.value.length == 0 && startsEndsWithUnsafeChar) {
    		console.log("entered validateURI 3b");
        	editorDisplayable.lastChild.nodeValue = messages['RM11098'];
            editor.value = '';
            dojo.addClass(editorDisplayable, 'error');               
            result = false;        	
        } else if (!editor.value.length == 0 && elementRepeated) {
    		console.log("entered validateURI 3c");
        	editorDisplayable.lastChild.nodeValue = messages['RM11095'];
            editor.value = '';
            dojo.addClass(editorDisplayable, 'error');
            result = false;
        } else if (!editor.value.length == 0 && ipLike && !(isValidIP || ipV6Like)) {
    		console.log("entered validateURI 3d");
        	editorDisplayable.lastChild.nodeValue = messages['RM11099'];
            editor.value = '';
            dojo.addClass(editorDisplayable, 'error');
            result = false;
        } else {
			// if the validation passed, remove the "error" class from the widget
        	console.log("entered validateURI 3, validation passes");
			//var elementDisplayable = dojo.byId('hostItemAdd');
        	var elementDisplayable = dojo.byId(elementId);
            if (dojo.hasClass(elementDisplayable, 'error'))
            {
                dojo.removeClass(elementDisplayable, 'error');
            }
        }     
		console.log("exited validateURI with: ", result);
		this.validationFlag = result;
        return result;
	},
	
	addToCurrentHostsTable: function(newHostAddress){
		console.log("entered addToCurrentHostsTable(newHostAddress): ", newHostAddress);
		var scope = this;
		var table = dojo.query("[id=hostsTableBody]");
		console.log("Found ", table.length, " nodes");
		console.log("the table found: " + table);
		if (table.length == 1) {	      
		  //create the new row
		  var newRowFromValue = this.createNewRow(newHostAddress);	
		  
		  //get reference to last row
		  var editorRow = dojo.byId('editorRow');
		  
		  //insert the new row
		  if (editorRow) {
		    table[0].insertBefore(newRowFromValue, editorRow);
		    console.log("found editorRow, inserting new row before this one");
		  }
		  
		  // hostItemAdd
		  var editorSpan = dojo.byId('hostItemAdd');
		  
		  if (editorSpan) {
		    //reset the editor to the default text
		    //Add the new host address to the array
			  if(this.isMember)
			  {
				  this.member.push(newHostAddress);  
			  }else{
				  this.hosts.push(newHostAddress);
			  }
		    
		    //This reverts the displayed value
        	var defaultTextForEditor = messages['RM11094'];
        	if(this._defaultInlineValue) {
        		defaultTextForEditor = this._defaultInlineValue;
        	}
		    //editorSpan.lastChild.nodeValue = messages['RM11094'];
        	editorSpan.lastChild.nodeValue = defaultTextForEditor;
		    
		    var editor = dijit.byId('hostItemAdd');
		    if (editor) {
		    	console.log("found editor with value: ", dojo.attr(editor, 'value'));
		    	console.log("found editor with displayedValue: ", dojo.attr(editor, 'displayedValue'));
		    	// this reverts the editor to the default text when clicking on it, instead of the 
		    	// previous value
		    	//dojo.attr(editor, 'value', messages['RM11094']);
		    	dojo.attr(editor, 'value', defaultTextForEditor);
		    }
		    console.log("added ", newHostAddress, " to array: ", this.hosts, ", array length: ", this.hosts.length);
		  } else {
			  console.log("failed to find editor span with dojo.byId(\'hostItemAdd\'), instead got: ", dojo.byId('hostItemAdd'));
		  }
		  console.log("done adding row");
		}

		console.log("exiting addToCurrentHostsTable");
    },
    
	deleteHostElement: function(rowIdToDelete) {
		console.log("entered deleteHostElement with (rowIdToDelete):", arguments);
	    var scope = this;	    
	    var row = dojo.byId(rowIdToDelete);	   
	    //'hostRow-' + newHostAddress
	    var pos = rowIdToDelete.indexOf('-');	    	 
	    var valueToRemove = rowIdToDelete.substring(pos+1);
	    console.log("value to remove is: ", valueToRemove, ", array length is: ", this.hosts.length);
	    
	    var indexToDelete = 0;
	    //find index of array to remove
		if(this.isMember){
			for (var i = 0; i < this.member.length; i+=1) {
				if (this.member[i] == valueToRemove) {
					indexToDelete = i;
					break;
				}
			}
			this.member.splice(indexToDelete, 1);
		}else{
			for (var i = 0; i < this.hosts.length; i+=1) {
				if (this.hosts[i] == valueToRemove) {
					indexToDelete = i;
					break;
				}
			}
			this.hosts.splice(indexToDelete, 1);
		}
		    
	    
	    console.log("found item to delete at index: ", indexToDelete);

	    console.log("deleted ", valueToRemove, " from the array: ", this.hosts, ", new length: ", this.hosts.length);
	    row.parentNode.removeChild(row); 
	    
		console.log("exiting deleteHostElement");
	},
	
    _createTable: function(id, uri, fields, alwaysScroll) {
    	console.log("entered _createTable");
    	console.log("alwaysScroll is " + alwaysScroll);
        var submitRow = dojo.byId(id + '-submit-row');
        var tableBody = submitRow.parentNode;

        var extensionRow = document.createElement('tr');
        dojo.addClass(extensionRow, id + '-dialog-extension-row');
        tableBody.insertBefore(extensionRow, submitRow);

        var extensionColumn = document.createElement('td');
        extensionRow.appendChild(extensionColumn);

        //
        // wrap the form in a scrollable <div/> so the dialog doesn't
        // become as big as the window itself
        //
        var scrollArea = document.createElement('div');
        dojo.addClass(scrollArea, 'alt-scrollable');
        scrollArea.alwaysScroll = alwaysScroll;
        
		dojo.style(scrollArea, { 
            overflowY: 'auto',
            overflowX: 'hidden',
            maxHeight: (this.defaultScrollLength * 26) + 'px',
            paddingRight: 10 + 'px',
            paddingTop: 10 + 'px'
        });
        
        extensionColumn.appendChild(scrollArea);

        var form = document.createElement('form');
		form.setAttribute('id', id + '-form');
		form.setAttribute('onsubmit', this.onSubmit);
        scrollArea.appendChild(form);

        //
        // here is the table with all of the form input elements
        //
        var fieldTable = document.createElement('table');
        
        // accessibility since table is purely for layout 
        fieldTable.setAttribute ("role", "presentation" );
        
        dojo.addClass(fieldTable, 'dialog-form-table');
        form.appendChild(fieldTable);

        //IE requires the 'tbody' to be created and appended...
        var fieldTableBody = document.createElement('tbody');
        fieldTable.appendChild(fieldTableBody);
        
        var tooltips = [];

        for (var i = 0; i < fields.length; ++i)
        {
            if (fields[i].hidden)
                continue;

  		    var row = document.createElement('tr');
            fieldTableBody.appendChild(row);
            if (fields[i].inlinetextedit) {
            	row.setAttribute('valign','top');
            }

  		    var requiredColumn = document.createElement('td');
            row.appendChild(requiredColumn);
            
            if (fields[i].required) {
                dojo.byId(requiredColumn).innerHTML = "<span class='default required-field-marker' title='" + messages['RM11082'] + "'>*</span>";
			}
			
            // appends the label to the row of the table
  		    var nameColumn = document.createElement('td');
            row.appendChild(nameColumn);

            // appends the text box column to the row of the table
  		    var valueColumn = document.createElement('td');
            row.appendChild(valueColumn);
            
            var lockColumn = fields[i].showLock ? new Element('td') : null;

            if (lockColumn) {
                row.appendChild(lockColumn);
			}
			var inner = "<span class='default'><label for='"+fields[i].name+"'>" + fields[i].label + this.labelSeparator + "</label></span>";
			console.log("tooltip is " + fields[i].tooltip );
			if ( fields[i].tooltip != null ) {
				var tooltipId = "c_" + fields[i].name + "-" + i;
				inner += "<span class='dashboard-detail-view-tt' id=" + tooltipId + "></span>"
				tooltips.push({tooltipId : tooltipId, tooltip : fields[i].tooltip});
			}
            dojo.byId(nameColumn).innerHTML = inner;
            
            // this portion of the code handles adding the dropDown list
            if (fields[i].values)
            {
            	//the drop down list
	  		    var input = document.createElement('select');
	  		    dojo.addClass(input, 'dialog-drop-down-menu');
				input.setAttribute('id', fields[i].name);

				if (fields[i].required) {
					input.setAttribute('aria-required', true);
				}
				
				if (fields[i].multiple) {
                    input.setAttribute('multiple', 'multiple');
                    input.setAttribute('size', (fields[i].values.length<10) ? fields[i].values.length : 10);
                }
                
                if (fields[i].type == 'boolean' || fields[i].type == 'integer') {
                    input.style.width = '100px';
                }

                for (var j = 0; j < fields[i].values.length; ++j)
                {
                    var values = fields[i].values[j];
                    input.options[j] = new Option(values.name, values.value);
                    
                    if (fields[i].value && fields[i].value == values.value)
                        input.options[j].selected = true;
                }
            } else if (fields[i].inlinetextedit) {
            	// here we create a <div> that has a table that has a tbody
            	// that has a tr that has a td that
            	// has a span that has the editor widget 
            	if (fields[i].name == 'member'){
            		this.isMember = true;
            	}         	
            	
            	console.log("Creating div wrapping hosts table");
            	var input = document.createElement('div');
            	var divId = "configure-hosts-restrictions-section";
            	input.setAttribute('id', divId);
				if (fields[i].required) {
					input.setAttribute('aria-required', true);
				}
            	
            	//adjust the default inlineeditor default value text accordingly
            	var defaultTextForEditor = messages['RM11094'];
            	console.log("defaultInlineValue from createTable method: ", this._defaultInlineValue);
            	if(this._defaultInlineValue) {
            		console.log("Found passed in default value: ", this._defaultInlineValue);
            		defaultTextForEditor = this._defaultInlineValue;
            	}
            	            	
            	// Create <table> for Host restrictions
            	var hostsTable = document.createElement('table');
				hostsTable.setAttribute('role', 'presentation');
				hostsTable.setAttribute('id', 'hostsTable');
				dojo.addClass(hostsTable, 'default');
				
				// add table to the main <div>
				input.appendChild(hostsTable);
				
		        // IE requires the 'tbody' to be created and appended
				// Later, we will search for this <tbody>
		        var hostsTableBody = document.createElement('tbody');
				hostsTableBody.setAttribute('id', 'hostsTableBody');
		        hostsTable.appendChild(hostsTableBody);
					  		    
	  		    // the new row for the user to enter the host restriction
	  		    var rowNew = document.createElement('tr');	  	
	  		    rowNew.setAttribute('id', 'editorRow');
	  		    hostsTableBody.appendChild(rowNew);
	  		    
	  		    // <td colspan='2'>
	  		    var columnNew = document.createElement('td');
	  		    columnNew.setAttribute('id', "editorCell");
				columnNew.setAttribute('colspan', '2');
	  		    rowNew.appendChild(columnNew);
				
	  		    // the <span> housing the inlineEditor widget
				var spanNew = document.createElement('span');
				// the id of the element the widget will bind to
				var spanNewID = "hostItemAdd";
				spanNew.setAttribute('id', spanNewID);
				dojo.addClass(spanNew, 'default inplace-form-text');
				dojo.style(spanNew, {
					'backgroundColor':'transparent'
				});
				//spanNew.setAttribute('title', messages['RM11094']);	  		      
				spanNew.setAttribute('title', defaultTextForEditor);
				
				// normally this call would be after the if-else if-else, but for some reason
				// it doesn't work unless it is here for this "else if" case.
	  		    columnNew.appendChild(spanNew);									
	  		    
            	console.log("created span");                

            	valueColumn.appendChild(input);       
            	
            	console.log("calling createHostRestrictionEditor");
            	console.log("dojo.byId(spanNewID)", dojo.byId(spanNewID));
            	console.log("document.getElementById(spanNewID)", document.getElementById(spanNewID));
            	
            	console.log("entering createHostRestrictionEditor with fields[", i ,"].name=", fields[i].name,
            			" and fields[", i, "].label=", fields[i].label);
            	
            	// create edit box and attach it to the span for the new host item, make it responsive to onChange event
            	this._createHostRestrictionEditor(spanNewID, defaultTextForEditor, messages['RM11093'], defaultTextForEditor);
            } else {
            	// this creates either the an input box, or an input box for a password
                var isPassword = fields[i].name.match(/password/);
                var textType = isPassword ? 'password' : 'text';
                
	  		    var input = document.createElement('input');
	  		    
				input.setAttribute('type', textType);
				dojo.addClass(input, 'modal-dialog-text');
				input.setAttribute('id', fields[i].name);
				
				if (fields[i].required) {
					input.setAttribute('aria-required', true);
				}
				
	  		    this.connectToDialogOK(input);
				
				
				if ( fields[i].maxlength ) {
					console.log("set field maxlength to " + fields[i].maxlength);
					input.setAttribute('maxlength', fields[i].maxlength);
				}

                //
                // add default value, if one exists
                //
				console.log("createTable: fields[",i,"].value: ", fields[i].value);
                if (fields[i].value && !fields[i].value.match(/^RM\d{5}$/)) {
                    input.value = fields[i].value;
                    console.log("createTable: fields[",i,"].value: ", fields[i].value);
				}
                else if (fields[i].help)
                {
                    input.value = input.help = fields[i].help;
                    //input.addClassName('form-prompt');
                    dojo.addClass(input, 'form-prompt');
                }
                
                if (isPassword)
                    input.onfocus = function() {
                        if (dojo.hasClass(this, 'error'))
                        {
                            this.value = '';
                            //this.removeClassName('error');
                            dojo.removeClass(this, 'error');
                        }
                        
                        toggleTextBoxType(this, true);
                        return true;
                    }
                else if (fields[i].inlinetextedit)
                    input.onfocus = function() {
                    if (dojo.hasClass(this, 'error'))
                    {
                        this.value = '';
                        dojo.removeClass(this, 'error');
                    }                    
                    //toggleTextBoxType(this, true);
                    //return true;
                }
                else {
                    input.onfocus = function() {
                        if (this.value == this.help || dojo.hasClass( this, 'error'))
                        {
                            this.value = '';
                            //this.removeClassName('form-prompt');
                            //this.removeClassName('error');
                            dojo.removeClass(this, 'error');
                            dojo.removeClass(this, 'form-prompt');
                        }
                    }
                }

                //
                // temporarily disactivate validation for network
                // enforce validation if present
                //
                console.log("iteration i=", i);
                if (fields[i].validation 
                		&& (!("inlinetextedit" in fields[i]) || !fields[i].inlinetextedit)) {
    		    	console.log("entering first validation check");
                	this._addValidationTests(input, fields[i].validation);
				}                 
            } // end else clause of creation of all fields in the table

            //
            // turn off editing, if desired
            //
            if (fields[i].immutable)
                input.disabled = true;

            // input box is appended to table, whether it's an Option box, text box, password box, etc.
            if (!fields[i].inlinetextedit) {
              valueColumn.appendChild(input);
            }
            
            if (lockColumn)
            {
	  		    var lockImage = document.createElement('img');
	  		    lockImage.setAttribute('id', fields[i].name + '-lock');
	  		    lockImage.setAttribute('src', fields[i].locked ? '/images/lock-icon.gif' : '/images/unlocked.gif');
	  		    lockImage.setAttribute('alt', fields[i].locked ? fields[i].lockedTooltip : fields[i].unlockedTooltip);
	  		    lockImage.setAttribute('title', fields[i].locked ? fields[i].lockedTooltip : fields[i].unlockedTooltip );

                lockImage.locked = fields[i].locked;
                lockImage.lockedTooltip = fields[i].lockedTooltip;
                lockImage.unlockedTooltip = fields[i].unlockedTooltip;

                if (fields[i].lockImmutable)
                {
                    lockImage.setOpacity(0.5);
                    lockImage.alt = lockImage.title = messages['RM11062'];
                    lockColumn.appendChild(lockImage);
                }

                else
                {
		  		    var lockLink = document.createElement('a');
		  		    lockLink.setAttribute('href', 'javascript:void(0);');
                    lockLink.image = lockImage;
                    lockLink.onclick = dojo.hitch(lockLink, function() {
                        this.image.setAttribute('locked', ! this.image.locked);
                        this.image.setAttribute('src', this.image.locked ? '/images/lock-icon.gif' : '/images/unlocked.gif');
                        this.image.setAttribute('alt', this.image.locked ? this.image.lockedTooltip : this.image.unlockedTooltip);
                        this.image.setAttribute('title', this.image.locked ? this.image.lockedTooltip : this.image.unlockedTooltip);
                    });

                    lockLink.appendChild(lockImage);
                    lockColumn.appendChild(lockLink);
                }

            }
        }  // end for loop iterating through the fields
        for (var i = 0; i < tooltips.length; ++i) {
			this.createTooltip(tooltips[i].tooltipId, tooltips[i].tooltip);
        }

    },
    
    createNewRow: function(newHostAddress) {
    	console.log("entered createNewRow(newHostAddress) with: ", newHostAddress);
    	var rowStored = document.createElement('tr');
    	rowStored.setAttribute('id', 'hostRow-' + newHostAddress);
    	
        // Create <td> for text field of already stored IP Addresses
        var column1 = document.createElement('td');
        column1.setAttribute('valign', 'top');
        rowStored.appendChild(column1);
    
        var spanHostCol = document.createElement('span');
        dojo.addClass(spanHostCol, 'default');
        column1.appendChild(spanHostCol); 
      
        var spanHostColValue = document.createTextNode(newHostAddress);
        spanHostCol.appendChild(spanHostColValue);	  		    	  		      
    	
	    // Create <td> for the x icon
	    // <td valign="top">
	    // <span class='default-with-margin'>   
	    // <a href="javascript:void(0);" onClick='${request.view.logLevelType[]}.deleteTraceElement("${loggingLevel.key}-${request.view.logLevelType[]}", "${loggingLevel.key}");' style="text-decoration: none;">
	    // <img src='/images/delete-icon-small.gif' class='icon-image' alt="${deleteText}" title="${deleteText}"/>
	    // </a>
	    var column2 = document.createElement('td');
	    column2.setAttribute('valign', 'top');
    	
	    //span for the column containing the "x"
	    var spanXcol = document.createElement('span');
	    dojo.addClass(spanXcol, 'default-with-margin');
	    spanXcol.setAttribute('id', newHostAddress + '-span');
	    column2.appendChild(spanXcol);
    	
	    var anchorX = document.createElement('a');
	    dojo.style(anchorX, {
		  'textDecoration': 'none'
		  //,'border' : '0'
	    });
	    anchorX.setAttribute('href', 'javascript:void(0);');
    	
	    // this is not a real method yet "_deleteHost"
	    anchorX.onclick = dojo.hitch(this, this.deleteHostElement, 'hostRow-' + newHostAddress);
    	
	    spanXcol.appendChild(anchorX);
    	
	    var imgXicon = document.createElement('img');
	    // not a real url to an image yet
	    imgXicon.setAttribute('src', '../../../../images/delete-icon-small.gif');
	    // IE 8 no likey: dojo.style(imgXicon, 'icon-image');
	    imgXicon.setAttribute('title', messages['RM11092']);
	    imgXicon.setAttribute('alt', messages['RM11092']);
	    
	    anchorX.appendChild(imgXicon);
	    rowStored.appendChild(column2);
	    
	    console.log("exiting createNewRow with the new rowStored: ", rowStored);
	    return rowStored;
    },
    
	createTooltip: function(target_id, content) {
		console.log("target id is " + target_id + " and content is " + content);
		// prepending 'tt_' to distinguish between target and tooltip id's
		var obj = document.getElementById('ctt_'+target_id);
		if(obj != null) {
		    console.log("found tt, need to remove");
	        obj.parentNode.removeChild(obj);
		}
		
	   	var tooltip = new dijit.Tooltip({
	       	label: content,
	    	connectId: [target_id]
		});
 
 		dojo.byId(target_id).setAttribute("tabIndex", 0);
 		
		tooltip.domNode.id = 'ctt_'+target_id;
		document.body.appendChild(tooltip.domNode);
	},
				

    _getBuiltInTest: function(test) {
    	console.log("_getBuiltInTest");
        return this._builtInTests[test];
    },
    
    initialize: function(id, text, uri, fields, callback, ajaxMethod, jsonBuilder) {
		var onSubmit = dojo.hitch(this, "_onPostRequest", uri, fields);
		this.inherited(arguments, [id, text, onSubmit]);
        console.log("form-dialog initialize called");
    },

    constructor: function(id, text, uri, fields, callback, ajaxMethod, jsonBuilder, defaultValue, inlineEditValidator) {
   		this.initialize(id, text, uri, fields, callback, ajaxMethod, jsonBuilder);
		//console.debug("form-dialog constructor called, id is " + id + " jsonBuilder is " + jsonBuilder);
   		console.log("form-dialog constructor called, id is " + id + " jsonBuilder is " + jsonBuilder);
   		console.log("defaultValue is: ", defaultValue);
   		console.log("inlineEditValidator is: ", inlineEditValidator);
        //$super(id, text, this._onPostRequest.bind(this, uri, fields));

        this._builtInTests = {
            'notBlank': {
                test: dojo.hitch(this, this._notBlank),
                message: messages['RM11025']
            },
            'isAlphaNumeric': {
                test: dojo.hitch(this, this._isAlphaNumeric),
                message: messages['RM11026']
            },
            'isBRName': {
                test: dojo.hitch(this, this._isBRName),
                message: messages['RM11026']
            },
            'isNumeric': {
                test: dojo.hitch(this, this._isNumeric),
                message: messages['RM11027']
            },
            'isEmail': {
                test: dojo.hitch(this, this._isEmail),
                message: messages['RM11028']
            },
            'isURI': {
                test: dojo.hitch(this, this._isURI),
                message: messages['RM11029']
            },
         	'isSNMPURI': {
                test: dojo.hitch(this, this._isSNMPURI),
                message: messages['RM11029']
            },
            'isHost': {
            	test: dojo.hitch(this, this._isHost),
            	message: messages['RM11029']
            },
            'isVersion': {
                test: dojo.hitch(this, this._isVersion),
                message: messages['RM11030']
            },
            'isIPMask': {
                test: dojo.hitch(this, this._isIPMask), 
                message: messages['RM11028']
            },
            'isIP': {
                test: dojo.hitch(this, this._isIP), 
                message: messages['RM11028']
            },
            'isValidIP' : {
            	test: dojo.hitch(this, this._isValidIP),
            	message: messages['RM11028']
            },
            'adjustMemSize': {
                test: dojo.hitch(this, this._adjustMemSize), 
                message: null
            },
            'validatePassword': {
            	test: dojo.hitch(this, this._validatePassword),
            	message: messages['BR11088']
            },
            'isEthernetInterface': {
            	test: dojo.hitch(this, this._validateEthernetInterface),
            	message: messages['RM11107']
            },
            'validateURI': {
            	test: dojo.hitch(this, this._validateURI),
            	message: messages['RM11119']
            }
        };
        
        this._defaultInlineValue = defaultValue;
        this.isSubmitting = false;
        this.labelSeparator = messages['RM11059'];
        this.defaultScrollLength = 13;
        this.callback = callback;
        this._createForm(id, uri, fields);
        this._ajaxMethod = ajaxMethod || 'POST';
        this._jsonBuilder = jsonBuilder || this._createJSON;
        this._specialValidator = inlineEditValidator || this.validateURI;
        this.hosts = new Array();
        this.validationFlag = true;
        this.member = new Array();
        this.isMember = false;
        console.log("end of form dialog ctr");
    },
    _isBRName: function(element) {
       	return element.value.length == 0 || element.value.match(/^[a-zA-Z_:][a-zA-Z0-9_:\.-]*$/);
    },
    
	_isAlphaNumeric: function(element) {
        return element.value.length == 0 || element.value.match(/^\w+$/);
    },

    _isEmail: function(element) {
        return element.value.length == 0 || element.value.match(/^[a-zA-Z0-9\._%+\-]+@(?:[a-zA-Z0-9\-]+\.)+[a-zA-Z]{2,4}$/);
    },
    
    _isIP: function(element) {
    	console.log("_isIP: element.value: ", element.value);
        var ip = new javascript.widgets.IPAddress(element.value);
        // note, this is not meant to be perfect validation for IPv6
        // which is nearly impossible unless you pick up a Java, or Perl library
        // to do the validation instead
        return ip.bits || element.value.match(/^([:]{2})$/) 
        	  || element.value.match(/^((([0-9]|[a-f]|[A-F]){1,4})([:]{0,2})){0,8}$/)
        	  || element.value.match(/^([:]{2})((([0-9]|[a-f]|[A-F]){1,4}){0,6})$/);
    },
    
    _isValidIP: function(element) {
    	console.log("_isValidIP: element.value: ", element.value);
		return element.value == 0 || dojox.validate.isIpAddress(element.value, {allowDecimal: false, allowIPv6: true, allowHybrid: false}) 
		    || element.value.match(/^([:]{2})$/) 
  	  		|| element.value.match(/^((([0-9]|[a-f]|[A-F]){1,4})([:]{0,2})){0,8}$/)
  	  		|| element.value.match(/^((([0-9]|[a-f]|[A-F]){1,4})([:]{0,2})){0,8}(%\d)?(\/\d)?$/)
  	  		|| element.value.match(/^([:]{2})((([0-9]|[a-f]|[A-F]){1,4}){0,6})$/); 
    },
    
    _isIPMask: function(element) {
        console.log("_isIPMask: element.value: ", element.value);
        var ipString = element.value;
        
		if (element.value.indexOf('/') >= 0) {
                ipString = element.value.split('/')[0];
        } else {
                //Mask is required
                return false;
        }

        var isIP = ipString == 0
                   || dojox.validate.isIpAddress(ipString, {allowDecimal: false, allowIPv6: true, allowHybrid: false})
                   || ipString.match(/^([:]{2})$/)
                   || ipString.match(/^((([0-9]|[a-f]|[A-F]){1,4})([:]{0,2})){0,8}$/)
                   || ipString.match(/^((([0-9]|[a-f]|[A-F]){1,4})([:]{0,2})){0,8}(%\d)?(\/\d)?$/)
                   || ipString.match(/^([:]{2})((([0-9]|[a-f]|[A-F]){1,4}){0,6})$/);

        return isIP && element.value!='';
    },
    
    _isNumeric: function(element) {
        return element.value.length == 0 || element.value.match(/^\d+(\.\d+)?$/);
    },

    _isURI: function(element) {
        return element.value.length == 0 || element.value.match(/^((https?:\/\/)?[\w\d:#@%\/;$()~_?\+\-=\\\.&]*)$/);
    },
    
	_isSNMPURI: function(element) {
        return element.value.length == 0 || element.value.match(/^([^,](https?:\/\/)?[\w\d:#@%\/;$()~_?\+\-=\\\.&\S]*[^,]){1}(,(https?:\/\/)?[\w\d:#@%\/;$()~_?\+\-=\\\.&\S]+)*([^,])$/);
    },
    
    _isHost: function(element) {
    	return element.value.length == 0 || element.value.match(/^([\w\d\/_\-\.]*)$/);
    },

    _isVersion: function(element) {
        return element.value.length == 0 || element.value.match(/^\d+(\.\d+)*$/);
    },

    _notBlank: function(element) {
    	console.log("_notBlank, value is " + element.value);
        return element.value.length > 0;
    },
    _validatePassword: function(element){
    	var passwordVerifyField = element.value;
    	var passwordField = dojo.byId('password');
    	var validPassword = true;
    	if(this._notBlank(element) || this._notBlank(passwordField) ){
	    	var password = passwordField.value
	    	validPassword = (password == passwordVerifyField);
    	}
		return validPassword;    	
    },
    
    _validateEthernetInterface: function(element){
    	var ethernetVerifyField = element.value;
    	var membersField = dojo.byId('member');
    	var validEthernet = true;
    	if(this._notBlank(element) || this._notBlank(membersField) ){
	    	var member = membersField.value
	    	validMember = (member == getEthernetInterfaces().any());
    	}
		return validPassword;    	
    },
    
	_validateURI: function(element) {
		// this is a validation method for fields that takes either a host name (with no scheme)
		// or IPv4 IPv6 address, since out-of-box dojox validation is insufficient for IPv6
		console.log("entered _validateURI with element.id:", element.getAttribute("id"));
        var result = true;
        
        console.log("element: ", element);
        console.log("element.value: ", element.value);
    	console.log("entered _validateURI 1");

    	element.value = Bedrock.strip(element.value);    	    	
		var isValidIP = dojox.validate.isIpAddress(element.value, {allowDecimal: false}); 
		console.log("isValidIP(IPv4 or IPv6 with no ::): ", isValidIP);
		
		// RFC 1738 says that host can be (hostname | hostnumber)
		// hostnumber = digits "." digits "." digits "." digits

		var strArray = element.value.split(".");
		var strArray2 = element.value.split(":");
		var strArray3 = element.value.split("::");
		
		var ipLike = true;
		var startsEndsWithUnsafeChar = false;
		var entryLength = element.value.length;
		
    	console.log("entered _validateURI 2");			
		console.log("entryLength: ", entryLength);
		console.log("index of last dot: ", element.value.lastIndexOf('.'));
		if(element.value.indexOf('/') == 0 || element.value.indexOf('-') == 0
				|| element.value.indexOf('_') == 0 || element.value.indexOf('.') == 0
				|| element.value.lastIndexOf('/') == entryLength - 1 || element.value.lastIndexOf('-') == entryLength - 1
				|| element.value.lastIndexOf('_') == entryLength - 1 || element.value.lastIndexOf('.') == entryLength - 1) {
				startsEndsWithUnsafeChar = true;
		}
		
		console.log("starts or ends with unsafe character: ", startsEndsWithUnsafeChar);

		var ipV4Like = true;
		var ipV6Like = true;
		
		var j=0;
		
		//we check to see if the provided entry is intended as a valid IPv4 address
		for(j=0; j<strArray.length; j+=1) {
			var octet = strArray[j];
			console.log("octet[",j,"]: ", octet);
			
			if (!strArray[j].match(/^([\d]*)$/)) {
				ipV4Like = false;
				break;
			}
		}

		var k=0;
		var hexit = "";
		console.log("strArray length(.): ", strArray.length);
		console.log("strArray2 length(:): ", strArray2.length);
		console.log("strArray3 length(::) (should be 2 or less): ", strArray3.length);
		
		//we check to see if the provided entry is intended as a valid IP address
		var pattern2 = /^([\da-fA-F]{1,4})(%[\w\d]+)?(\/[\d]+)?$/;
		
		// this pattern is kept for reference, we use pattern2, which is the 
		// same pattern, but in a different notation
		var patternIPv6 = "([\\da-fA-F]{1,4})(%[\\w\\d]+)?(\\/[\\d]+)?";
		
		// declaring the regexp object doesn't seem to work			
		var regExpIPv6 = new RegExp(patternIPv6, "g");
		
		for (k=0; k<strArray2.length; k+=1) {
			hexit = strArray2[k];
			console.log("hexit[", k ,"]: ", hexit, ". Matches: ", 
					pattern2.test(strArray2[k]), ". Matches regexp: ",  
					regExpIPv6.test(strArray2[k]), ". Matches blank: ", (hexit==""));
			
			// logic to detect more than one "::", which makes it invalid as IPv6 address
			if(strArray3.length > 2) {
				ipV6Like = false;
				break;
			}
 		
			if (!pattern2.test(hexit) && hexit != "") {
				ipV6Like = false;
				break;
			}
		}
		
    	console.log("entered _validateURI 3");
		console.log("ipV4Like: ", ipV4Like);
		console.log("ipV6Like: ", ipV6Like);
		ipLike = (ipV4Like || ipV6Like);
		console.log("ipLike: ", ipLike);
        /*    			
		// /^((https?:\/\/)?[\w\d:#@%\/;$()~_?\+\-=\\\.&]*)$/
		*/
		
		// the below regex will prevent use of scheme (http(s)://), ":", "?", "&" and 
		// other character that net-snmp would not expect for a hostname or ipaddress
		
		if (!element.value.length == 0 && !ipV6Like 
				&& !element.value.match(/^([\w\d\/_\-\.]*)$/)) {
    		console.log("entered _validateURI 3a");             
            result = false;
        } else if (!element.value.length == 0 && startsEndsWithUnsafeChar) {
    		console.log("entered _validateURI 3b");
            result = false;        	
        } else if (!element.value.length == 0 && ipLike && !(isValidIP || ipV6Like)) {
    		console.log("entered _validateURI 3c");
            result = false;
        } else {
        	console.log("_validateURI 4, validation passes");
        }  

		console.log("exited _validateURI with: ", result);
        return result;
	},

    _nonHidden: function(fields) {
		return dojo.filter(fields, function(field) { return !field.hidden; });
    
        //return fields.findAll(function(field) { return ! field.hidden });
    },

    _onPostFailure: function(response) {
    	console.log("_onPostFailure", response);
        this.isSubmitting = false;
        var _msg = dojo.fromJson(response.responseText);
        this._setText(this.id, _msg.message, true);
    },

    _onPostRequest: function(uri, fields, event) {
    	console.log("_onPostRequest");
        if (!this.validate(fields)) {
        	console.log("_onPostRequest, validate = ", this.validate(fields));
            this.isSubmitting = false;
            return;
        }
        
        var requiredFieldFilled = true;
        // here we are not validating, but checking whether
        // or not required fields have been filled.
        // Before reaching this, we should have already validated
        // all fields in other validation methods.
        for (var col = 0; col < fields.length; col +=1) {
        	if(("required" in fields[col]) && fields[col].required == true) {
        		var input = dojo.byId(fields[col].name);
        		if (fields[col].name != 'hosts') {
        			requiredFieldFilled = (input.value.length > 0);
        		}
        	}
        	/*
        	if(fields[col].inlinetextedit && fields[col].required == true) {
        		requiredFieldFilled = this._specialValidator('hostItemAdd', this, true);
        		break;
        	}
        	*/
        }
        
        if(!requiredFieldFilled) {
        	console.log("_onPostRequest, validate = ", requiredFieldFilled);
            this.isSubmitting = false;
            return;
        }
        
        this._setText(this.id, messages['RM11033'], false);
        this.uri = uri;

		console.log("_onPostRequest: uri: ", uri);
        var json = this._jsonBuilder(fields, uri, this);
        console.log("_onPostRequest: json from _jsonBuilder: ", json);

        var onPostResponse = dojo.hitch(this, this._onPostResponse);
        var onPostFailure = dojo.hitch(this, this._onPostFailure);

        // this sends the information as json
		dojo.rawXhrPost({ 
			url: uri,
			postData: dojo.toJson(json), 
			handleAs: 'json',
		    headers: {
				'Content-Type': 'application/json',
				'Accept' : 'application/json'
		    },
			load: onPostResponse,
			error: onPostFailure
		});
        
    },

    _onPostResponse: function(response) {
    	console.log("_PostResponse");
        this.isSubmitting = false;
        this.hide(dojo.hitch(this, this.cleanup, this.id));

        if (this.callback)
            this.callback(response);
    },

    _onCertificateQuery: function(response) {
    	console.log("_onCertificateQuery");
        //
        // load the certificate in the dialog
        //
        var certificate = response.responseText.evalJSON();
        var dialog = new AcceptCertificateDialog('accept-dialog', messages['RM11034'], certificate, 'I');
        dialog.show();
    },

    submit: function() {
    	console.log("submit");
        if (this.isSubmitting)
            return;
            
        this.isSubmitting = true;
        this.onSubmit();
    },

    validate: function(fields) {
    	console.log("validate");
        for (var i = 0; i < fields.length; ++i)
        {
        	// skipping network for now.
            if (fields[i].hidden || fields[i].name == 'network' || fields[i].name == 'hosts' || fields[i].name == 'member' || fields[i].name == 'policy')
                continue;

            var input = dojo.byId(fields[i].name);
            
            console.log("input name is: ", fields[i].name);
            console.log("input is: ", input);

            if (input && dojo.hasClass(input,'form-prompt'))
                input.value = '';

            if (input.onblur && !input.onblur())
                return false;
        }

        return true;
    },
    connectToDialogOK: function(textwidget){
		var scope = this;    	
		
		dojo.connect(textwidget, "onkeyup", function(e){
			var key = e.keyCode || e.charCode;
		    switch(key){
		       case dojo.keys.ENTER : 
		       						scope.submit(); 
									dojo.stopEvent(e);
		       						break;
		       default:  break;
		    }
		    
		});
	}
});
