/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/


dojo.provide('javascript.widgets.MessageDialog');

dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Button");
dojo.require("javascript.widgets.GenericDialog");

dojo.declare('javascript.widgets.MessageDialog', [dijit._Widget, dijit._Templated], {   

	templatePath: dojo.moduleUrl("javascript","widgets/templates/MessageDialogTemplate.html"),

	_genericDialog: null,
	_dialogOKButton: null,
	_checkbox: null,
	
	type: "info", /* possible types are info, warning, and error */
	showCheckbox: false,
	checkboxCookieName: null,
	message: null,

	constructor: function(args) {
		this.inherited(arguments);
		if(args.type){
			this.type = args.type;
		}
		if(args.showCheckbox){
			this.showCheckbox = args.showCheckbox;
		}
		if(args.checkboxCookieName){
			this.checkboxCookieName = args.checkboxCookieName;
		}
		if(args.message){
			this.message = args.message;
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);
		if(!this.showCheckbox || (this.showCheckbox && !dojo.cookie(this.checkboxCookieName))){
			//create dialog
			this._genericDialog = new javascript.widgets.GenericDialog({id: this.id+"_genericDialog"}, this._messageDialogNode);
						
			//Create the dialog button
			this._dialogOKButton = new dijit.form.Button({
				label: messages["RM11107"],
				type: "button",
				onClick: dojo.hitch(this, function(){
					this.hide();
				})
			}, this._dialogOKButtonNode);
		
			this.setShowCheckbox(this.showCheckbox);
			this.setMessage();
			this.setType();
		}	
    },
	
	setType: function(newType){
		if(newType) this.type = newType;
		if(this.type=="warning"){
			this._imageNode.src = "../../../images/msg_warning.gif";
		} else if(this.type=="error"){
			this._imageNode.src = "../../../images/error-icon.gif";
		} else {
			this._imageNode.src = "";
		}
	},
	
	setMessage: function(newMessage){
		if(newMessage) this.message = newMessage;
		this._nessageNode.innerHTML = this.message;
	},
	
	setShowCheckbox: function(show){
		this.showCheckbox=show;
		if(show) {			
			if(this._checkbox) {
				dojo.style(this._checkbox.domNode.parentNode, "display", "table-cell");
			} else {
				this._checkbox = new dijit.form.CheckBox({
					name: "doNotShowCheckbox",
					value: "show",
					checked: false,
					title: messages["RM11106"],
					onChange: dojo.hitch(this, function(checked){
						if(checked){
							dojo.cookie(this.checkboxCookieName, checked, {expires: 365});
						} else {
							dojo.cookie(this.checkboxCookieName, null, {expires: -1});
						}
					})
				}, this._doNotShowCheckboxNode);
				
				this._doNotShowLabel.innerHTML =  messages["RM11106"];
			}
		} else {
			if(this._checkbox) {
				dojo.style(this._checkbox.domNode.parentNode, "display", "none");
			}
		}
	},
	
	show: function(){
		if(!this.showCheckbox || (this.showCheckbox && !dojo.cookie(this.checkboxCookieName))){
			this._genericDialog.show();
		}
	},
	
	hide: function(){
		this._genericDialog.hide();
	},
	
	destroy: function(){
		
		//Cleanup widgets
		if(this._genericDialog) this._genericDialog.destroyRecursive(true);
		if(this._dialogOKButton) this._dialogOKButton.destroyRecursive(true);
		if(this._checkbox) this._checkbox.destroyRecursive(true);
		
		delete this._genericDialog;
		delete this._dialogOKButton;
		delete this._checkbox;
		
		this.inherited(arguments);
	}
	

});
