/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// ModalDialog is a simple confirmation dialog that disables the rest of the browser until 
// the user makes a selection. It is centered within the screen and uses the Scriptaculous 
// 'appear' effect to show the disabled screen ('overlay') and dialog window.
//

dojo.provide('javascript.widgets.ModalDialog');
dojo.declare('javascript.widgets.ModalDialog', dijit._Templated, {
 
	_initialize: true,
	
    cancel: function() {
		// TODO: fix cleanup...
 		this.hide();	
		this.cleanup(this.id);		
    },
    
    _center: function(id, dimensions) {
        if (!dimensions)
            dimensions = {};

		var dialog = dojo.byId(id);
        //dialog.style.maxWidth = '2000px';

		// Need to save old values, display the node, determine the coordinates, then set it back. 
		// Silly that dojo doesn't do that for you...
		var oldValues = {};
		dojo.forEach(['opacity', 'zIndex', 'display'], function(item){
	    	oldValues[item] = dojo.style(dialog, item);
		});
		dojo.style(dialog, {'opacity': 0, 'zIndex': -10000, 'display': 'block'})

		var needToHideAgain = false;
		if(Bedrock.isHidden (dialog) ){
			Bedrock.show(dialog);	// have to show to get the appropriate size info
			needToHideAgain = true;
		}

        var unrestrictedDim = dojo.coords(dialog);
		dojo.style(dialog, oldValues);
        var height = (dimensions.height)? dimensions.height : unrestrictedDim.h;
        var width = (dimensions.width)? dimensions.width : unrestrictedDim.w;
		
        dialog.style.marginTop = (-1 * height / 2) + 'px';
        dialog.style.marginLeft = (-1 * width / 2) + 'px';
        //dialog.style.maxWidth = width + 'px';

        if(needToHideAgain){
        	Bedrock.hide(dialog);	// then make sure it is hidden!
		}        	
    },
    
	setSubmit:function(onSubmit) {
        if ( dojo.isFunction(onSubmit) ) {
	        this.onSubmit = onSubmit;
        }
        else {
	        this.onSubmit = this._noOp;
        }
        
	},

    cleanup: function(id) {
    },
    
    hide: function(afterFinish, hideOverlay) {
        if ( hideOverlay == null || hideOverlay == true ) {
	        dojo.fadeOut({
			    node: 'page-overlay',
        	    duration: 500, 
		    	onBegin: function(){
					dojo.style('page-overlay', 'opacity', 0.25);
		   		}
        	}).play();

			Bedrock.hide(dojo.byId('page-overlay'));
		}
        this._show(this.id, false, afterFinish);
        document.dialog = null;
    },
    
    initialize: function(id, text, onSubmit) {
        document.dialog = this;
        this.id = id;
        this.setSubmit(onSubmit);
        if (dojo.isString(text))
            this._setText(this.id, text, false);
    },

	constructor: function(id, text, onSubmit) {
    	this.initialize(id, text, onSubmit);
		
	},
	
    _noOp: function() {
    },
    
    _setText: function(baseID, text, error) {        

		//defect 9100: Fix width so that the table size does not change when the text changes
		var tableNode = dojo.byId(baseID + "-table");
		if(tableNode){
			var cs = dojo.getComputedStyle(tableNode);
			dojo.style(tableNode, "width", cs.width);
		}

		var span = dojo.byId(baseID + '-text');
		span.innerHTML = text;

		if (error){
			dojo.addClass(span, 'error');
		}else{
			dojo.removeClass(span, 'error');
		}
    },
    
    _show: function(id, enable, afterFinish) {
    	//console.debug("Modal Dialog, _show", id);
        afterFinish = afterFinish || this._noOp;

        if (enable)
        {
            dojo.fadeIn({
				node: id,
				duration: 500,
				onBegin: function(){
		    		Bedrock.show(id, {'opacity': '0' } );
				},
				onEnd: afterFinish
			    }).play();
        }
        
        else
        {
				dojo.fadeOut({
					node: id,
					duration: 500,
					onEnd: afterFinish
				    }).play();
	            var callback = dojo.hitch(this, this._showScrolling, true);
	            setTimeout(callback, 1000);
	            
	            Bedrock.hide(dojo.byId(id));
        }
        
        
        var inputFields = dojo.query("input", dojo.byId(this.id));
        if(inputFields && inputFields[0] && !inputFields[0].value){	//focus on first item in list (if no default value)
        	dijit.focus(dojo.byId(inputFields[0].id) )
        }else{
	        dijit.focus(dojo.byId(this.id) ) ;
		}
		//set tab index (aria provided support) in the actual dialog
		dojo.byId(this.id).setAttribute("tab-index", 0);
        
    },
    
    _showScrolling: function(enable) {

		dojo.query('div.scrollable').forEach(function(element) {
	        dojo.style(element, 'overflow', enable ? 'auto' : 'hidden');
        });
        
		dojo.query('div.alt-scrollable').forEach(function(element) {
	        dojo.style(element, 'overflow', 'hidden');
	        dojo.style(element, 'overflowY', enable || element.alwaysScroll ? (element.alwaysScroll ? 'auto' : 'visible') : 'hidden');
        });
    },
    
    show: function(dimensions) { 
    	//console.log("modal-dialog.show", arguments);
        this._center(this.id, dimensions);
        
        Bedrock.show('page-overlay');

        dojo.animateProperty({
		    node: 'page-overlay',
		    properties: {
				opacity: {
	        	    start: 0.0, 
        	    	end: 0.25
				}
	    	}
        }).play();
        
        
        this._show(this.id, true);
    },
    
    submit: function() {
		this.hide();
        this.onSubmit();
		this.cleanup(this.id);
    }
    
});
