dojo.provide("xsa.monitor.widgets.dashboard.CustomReport");

dojo.require("dojo.string");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.Tree");
dojo.require("dijit.Menu");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit.MenuSeparator");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.i18n");

dojo.require("dojox.widget.Standby");

dojo.requireLocalization("xsa.monitor.widgets.dashboard", "CustomReport");

dojo.require("xsa.monitor.widgets.dashboard.ChartContainer");

dojo.declare("xsa.monitor.widgets.dashboard.CustomReport", [dijit._Widget, dijit._Templated],{
	// summary:
	//		This widget is a composite widget for containing a tree of 
	//		chartable statistics and a tab container for containing the generated
	//		charts.
	
	// templateString: String 
	//		The string to use for the template of this widget.
	templateString: dojo.cache("xsa", "monitor/widgets/dashboard/templates/CustomReport.html"),

	// chartDialogTemplate: [private] String
	//		The contents of the chart dialog to use for displaying create new chart
	_chartDialogTemplate: "<div>" +
							"<span class='xsaStatisticsNewchartNameRequired'>*</span>&nbsp;" +
							"<label for='${id}_chartNameBox'>${newChartLabel}</label>&nbsp;"+
							"<input id='${id}_chartNameBox' intermediateChanges='true' type='text' dojoType='dijit.form.ValidationTextBox' required='true' promptMessage='${newChartPrompt}' invalidMessage='${newChartNameInvalid}'></input>" +
							"<br><br>" +
							"<center>" +
								"<button id='${id}_createButton' dojoType='dijit.form.Button' type='button'>${createChartButtonLabel}</button>" +
								"&nbsp;&nbsp;" +
								"<button id='${id}_cancelButton' dojoType='dijit.form.Button' type='button'>${cancelChartButtonLabel}</button>" +
							"</center>" +
						"</div>",

	// _saveDialogTemplate: [private] String
	//		The contents of the save dialog to use for displaying save the report
	_saveDialogTemplate: "<div>" +
							"<br>" +
							"<div class='xsaSaveStatusBox'>" +
								"<div class='xsaSaveStatusTitle'>${saveStatusLabel}</div>" +
								"<span id='${id}_saveStatusMessage'>${initialSaveStatus}</span>" +
							"</div>" +
							"<br>" +
							"<label for='${id}_saveReportNameBox'>${newSaveLabel}</label>&nbsp;"+
							"<input id='${id}_saveReportNameBox' intermediateChanges='true' type='text' dojoType='dijit.form.ValidationTextBox' required='true' promptMessage='${newSavePrompt}'></input>" +
							"<br><br>" +
							"<center>" +
								"<button id='${id}_saveButton' dojoType='dijit.form.Button' type='button'>${saveReportButtonLabel}</button>" +
								"&nbsp;&nbsp;" +
								"<button id='${id}_cancelSaveButton' dojoType='dijit.form.Button' type='button'>${cancelChartButtonLabel}</button>" +
							"</center>" +
						"</div>",
	// _loadDialogTemplate: [private] String
	//		The contents of the load dialog to use for displaying the dropdown for loading reports.
	_loadDialogTemplate: "<div>" +
							"<label for='${id}_loadReportNameBox'>${loadReportLabel}</label>&nbsp;"+
							"<div id='${id}_loadReportNameBox' dojoType='dijit.form.FilteringSelect' invalidMessage='${newLoadPrompt}' promptMessage='${newLoadPrompt}'></div>" +
							"<br><br>" +
							"<center>" +
								"<button id='${id}_loadButton' dojoType='dijit.form.Button' type='button'>${loadReportButtonLabel}</button>" +
								"&nbsp;&nbsp;" +
								"<button id='${id}_cancelLoadButton' dojoType='dijit.form.Button' type='button'>${cancelChartButtonLabel}</button>" +
							"</center>" +
						"</div>",
	// _deleteDialogTemplate: [private] String
	//		The contents of the delete dialog to use for confirming deletion of the report.
	_deleteDialogTemplate: "<div>" +
							"<div id='${id}_deleteQuestion'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</div>" +
							"<br>" +
							"<center>" +
								"<button id='${id}_deleteButton' dojoType='dijit.form.Button' type='button'>${yesLabel}</button>" +
								"&nbsp;&nbsp;" +
								"<button id='${id}_cancelDeleteButton' dojoType='dijit.form.Button' type='button'>${noLabel}</button>" +
							"</center>" +
						"</div>",


	// widgetsInTemplate: Boolean
	//		If true, the template as widgets in it.
	widgetsInTemplate: true,

	// _borderContainer: [private] Object
	//		The borderContainer widget used to house all the tree/chart stuff.
	_borderContainer: null,

	// _searchBoxNode: [private] DOMNode
	//		The DOM Node to house the search field, if any.
	_searchBoxNode: null,

	// _treeNode: [private] DOMNode
	//		The DOM Node to house the tree widget.
	_treeNode: null,

	// _treePane: [private] Object
	//		The reference to the content pane holding the tree and other data.
	_treePane: null,

	// _tabPane: [private] Object
	//		The reference to the content pane holding the tab container and other data.
	_tabPane: null,

	// _tree: [private] Object
	//		The tree widget to use to represent the statistics view.
	//		Created in startup.
	_tree: null,

	// _tree: [private] Object
	//		The tree menu widget to use when clicking on a node (and adding to a chart)
	//		Created in startup.
	_treeMenu: null,

	// _newChartDialog: [private] Object
	//		The dialog used for creating the new chart/tab
	//		Created in startup.
	_newChartDialog: null,

	// _treeStore: [private] Object
	//		The backing store for the tree.
	//		Created in startup.
	_treeStore: null,

	// _treeStore: [private] Object
	//		The backing model for the tree.
	//		Created in startup.
	_treeModel: null,

	// statisticsTreeUrl: [public] String
	//		The URL to the service end point that provides the data that 
	//		inflates the tree.
	statisticsTreeUrl: "",

	// statsticsUrl: [public] String
	//		The URL fragment used to assemble the basic data query that
	//		provides the series data.
	statisticsUrl: "",

	// reportUrl: [public] String
	//		The URL fragment used to save and retrieve reports.
	reportUrl: "",

	// _chartMap: [private] array
	//		An associative map of chart names to information about them,
	//		such as tab, etc.
	_chartMap: null,

	// _selectedItem: [private] object
	//		The datastore item selected by tree node click to add to the charts.
	_selectedItem: null,

	postMixInProperties: function(){
		// sunnary:
		//		Function to mixin translations, etc.
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("xsa.monitor.widgets.dashboard", "CustomReport");
		dojo.mixin(this, this._messages);
	},

	startup: function(){
		// summary:
		//		Over-ridden startup function for the widget.
																					
		if(!this._started){
			this.saveButton.attr("disabled", true);
			this.deleteButton.attr("disabled", true);
			this.inherited(arguments);
			// Build and init the tree.
			if(this.statisticsTreeUrl){
				this._createTree();
			}
			this._createNewChartDialog();
			this._chartMap = {};

			this.connect(this.saveButton, "onClick", "_saveCharts");
			this.connect(this.loadButton, "onClick", "_loadCharts");
			this.connect(this.deleteButton, "onClick", "_deleteCharts");
		}
		this.inherited(arguments);
		//
		//  listen for onClicks, so we can update aria-pressed attribute
		//
		var children = this._tabContainer.tablist.getChildren();
		for( var i = 0; i < children.length; ++i )
		{
		    dojo.connect(children[i],"onClick",dojo.hitch(this,this._removeWaiAttributes));
		}	
		this._removeWaiAttributes();
	},

        //
        //	this function removed invalid WAI-ARIA state.
        //  aria-pressed is invalid for a tab
        //
        _removeWaiAttributes: function(){
	    var children = this._tabContainer.tablist.getChildren();
    	    dojo.forEach(children,function(child){
	        dijit.removeWaiState(child.focusNode || child.domNode,"pressed");
            });
        },

	resize: function(dim){
		// summary:
		//		Resize over-ride to allow calling of child widget resize so that
		//		the widget scales properly in a container.
		// dim:
		//		The dimensions object to process.
		if(dim){
			dojo.contentBox(this.domNode, dim);
		}

		var myDims = dojo.contentBox(this.domNode);
		
		// Internal function usage, may not port (but unlikely to change);
		// Remove any internal padding/margins and use that to set the size of the
		// contained border container.
		var pads = dojo._getPadBorderExtents(this.domNode);
		var margins = dojo._getMarginExtents(this.domNode);
		myDims.h -= (pads.h + margins.h);
		myDims.w -= (pads.w + margins.w);
		this._borderContainer.resize(myDims);

		this.inherited(arguments);
	},

	_createTree: function(){
		// summary:
		//		Function to create the tree and its clickable menu to 
		//		display when clicking on tree elements
		// tags:
		//		private.
		this._treeStore = new dojo.data.ItemFileReadStore({url: this.statisticsTreeUrl});
		this._treeModel = new dijit.tree.ForestStoreModel({
			store: this._treeStore,
			query: {},
			rootId: "grids",
			rootLabel: this.treeRootLabel,
            childrenAttrs: ["children"]
		});
		this._tree = new dijit.Tree({model: this._treeModel, showRoot: false, openOnClick: true});
		this._treeNode.appendChild(this._tree.domNode);

		// Turn off the icons.
		this._tree.getIconClass = dojo.hitch(this, function(item, opened){
			if(this._treeStore.isItem(item) && this._treeStore.getValue(item, "chartable")){
				return "xsaStatisticsChartableNode";
			}
			return "xsaStatisticsNonChartableNode";
		});
		
		this.connect(this._tree, "onClick", "_onTreeNodeClick");
		this._tree.startup();

		// Now we want to build the basic menu.
		this._treeMenu = new dijit.Menu({});
		this._treeMenu.addChild(new dijit.MenuItem({
			label: this._messages.addToNewChart, 
			onClick: dojo.hitch(this, this._onAddToNewChart),
			iconClass: "xsaNewChartIcon"
		}));
		this._treeMenu.startup();

		// Listen for focus loss of the popup and close it.
		this.connect(this._treeMenu, "_onBlur", "_closeTreeMenu");

		// Kill the context menus on tree.
		this.connect(this._tree.domNode, "oncontextmenu", function(e){
			dojo.stopEvent(e);
		});
	},

	_connectToDialogOK: function(button, textwidget, handler){
		// summary:
		//		Function to connect to a text box and handle the OK button of the dialog
		//		and enter as 'equivalent'.
		// button:
		//		The OK button on the widget.
		// textwidget:
		//		The textbox widget.
		// handler:
		//		The function to call when clicked/entered.
		// tags:
		//		private
		var f = dojo.hitch(this, handler);
		this.connect(button, "onClick", f);
		this.connect(textwidget.domNode, "onkeypress", function(evt){
			// Bind the enter key
			if(evt && evt.charOrCode == dojo.keys.ENTER){
				f();
			}
		});
	},

	_createNewChartDialog: function(){
		// summary:
		//		Function to create the new chart name dialog
		// tags:
		//		private
		this._newChartDialog = new dijit.Dialog({
			title: this._messages.newChartDialogTitle
		});
		this._newChartDialog.startup();

		var props = {};

		// Config all the translatables for the template for
		// substitution.
		props.id = dijit.getUniqueId(this.id);
		props.newChartPrompt = this._messages.newChartPrompt;
		props.newChartLabel = this._messages.newChartLabel;
		props.createChartButtonLabel = this._messages.createChartButtonLabel;
		props.cancelChartButtonLabel = this._messages.cancelChartButtonLabel;
		props.newChartNameInvalid = this._messages.newChartNameInvalid;

		this._newChartDialog.attr("content", 
			dojo.string.substitute(this._chartDialogTemplate, props));

		// Find all our widgets we created so we can link them up.
		this._createChartButton = dijit.byId(props.id + "_createButton");
		this._cancelChartButton = dijit.byId(props.id + "_cancelButton");
		this._chartNameBox = dijit.byId(props.id + "_chartNameBox");

		//Initial state of createButton should be disabled.  Only enabled when valid.
		this._createChartButton.attr("disabled", true);

		this._chartNameBox.isValid = dojo.hitch(this, function(){
			// summary:
			//		Custom validation function that verifies the name is valid
			//		and that it doesn't already exist in our chart map.
			var value = this._chartNameBox.attr("value");
			if(dojo.isString(value)){
				if(!this._chartMap[value] || value == this._ignoreCheck || dojo.trim(value) == ""){
					return true;
				}
			}
			return false;
		});

		// Connect up the chart dialog to the OK and enter events.
		this._connectToDialogOK(this._createChartButton, this._chartNameBox, function(){
			// Create adds the new chart, then closes the dialog.
			if(this._chartNameBox.isValid()){
				// Okay, lets build up the params object from the item.
				var params = this._itemToObject(this._selectedItem, this._treeStore);
				this._ignoreCheck  = this._chartNameBox.attr("value");
				var check = dojo.connect(this, "_newChart", dojo.hitch(this, function(){
					// Ignore checking valid names while hiding, can cause a blip of 'invalid' to appear.
					dojo.disconnect(check);
					delete this._ignoreCheck;
				}));
				this._newChartDialog.hide();
				this._newChart(this._chartNameBox.attr("value"), params);
			}
		});

		// Connect up the cancel event.
		this.connect(this._cancelChartButton, "onClick", function(){
			// Cancel just closes the dialog.
			this._newChartDialog.hide();
		});

		// Connect up validation checking.
		this.connect(this._chartNameBox, "onChange", "_checkNewChartDialogState");
	},

	_checkNewChartDialogState: function(){
		// summary:
		//		This function checks the state of the new chart dialog, and if
		//		valid, will activate the create button.
		// tags:
		//		private
		if(this._chartNameBox.isValid() && dojo.trim(this._chartNameBox.attr("value")) != ""){
			this._createChartButton.attr("disabled", false);
		}else{
			this._createChartButton.attr("disabled", true);
		}
	},

	_onTreeNodeClick: function(item, treeNode, event){
		// summary:
		//		Function called when a tree node is clicked.
		//		This allows access to the item and to help know 
		//		when to open a menu at the click point.
		// item:
		//		The tree item clicked on.
		// tags:
		//		protected.

		var node = treeNode?treeNode.labelNode:"";
		if(item && 
			this._treeStore.isItem(item) && 
			this._treeStore.getValue(item, "chartable")){
			// Only pop the menu if the item is 'chartable'.
			if(node){
				var x, y, focusIt;
				// Okay, determine if click or key and do the right thing
				// for opening the menu.
				if(event){
					if(node && event.target !== node){
						// Ignore clicks except on the actual label.
						return;
					}
					x = event.pageX;
					y = event.pageY;
					// Shift up and back a bit to ensure the mouse is 
					// hover-over when it appears.
					if(x>1){
						x -= 1;
					}
					if(y>1){
						y -=1;
					}
				}else{
					var nodePos = dojo.position(node, true);
					x = nodePos.x;
					y = nodePos.y;
					focusIt = true;
				}

				// Create a sub menu that only allows adding to valid charts.
				// for this particular node.
				this._selectedItem = item;
				this._createNewSubMenu();
				dijit.popup.open({
					popup: this._treeMenu, 
					x: x, 
					y: y, 
					onCancel: dojo.hitch(this, function(){
						// Hook to cancel (esc) to close the
						// menu and focus it back to the label in 
						// question
						this._closeTreeMenu();
						dijit.focus(node);
					})
				});
				if(focusIt){
					this._treeMenu.focus();
				}
			}
		}
	},

	_onAddToNewChart: function(){
		// summary:
		//		Hook point to add a new chart to the tab 
		//		container.
		// tags:
		//		private
		this._closeTreeMenu();
		this._chartNameBox.attr("value", "");
		this._newChartDialog.show();
		this._createChartButton.attr("disabled", true);
	},

	_newChart: function(name, params){
		// summary:
		//		Function to add a new chart to the TabContainer.
		// name:
		//		The name of the chart to create, user provided.
		// param:
		//		The parameters that define the chart.
		// tags:
		//		private
		if(name){
			this._chartMap[name] = {
				params: [params]
			};
		}

		// Okay, lets create the new tab (chart container)!
		var newTab = new dijit.layout.ContentPane({
			title: name,
			closable: true
		});
		this._tabContainer.addChild(newTab);
		this._tabContainer.selectChild(newTab);
        
		this._chartMap[name].tab = newTab;
		this._chartMap[name].unit = params.unit;

		// Set up a listener to do cleanup when the tab itself is closed.
		// This will clean up the content pane contents and then 
		// recreate the submenu list of all the chars available for update.
		var self = this;
		var cl = dojo.connect(newTab, "destroy", function(){
			dojo.disconnect(cl);
			var chart = self._chartMap[this.title];
			delete self._chartMap[this.title];

			// Test to see if there are still any charts.  If not, 
			// nothing to save.
			var disable = true;
			for(var t in self._chartMap){
				disable = false;
				break;
			}
			self.saveButton.attr("disabled", disable);
		});

		var blocker = new dojox.widget.Standby({
			target: newTab.domNode,
			imageText: this.standbyText,
			zIndex: 500, 
			color: "#FFFFFF",
			image: dojo.moduleUrl("xsa", "monitor/widgets/dashboard/styles/images/progress-indicator-large.gif").toString()
		});
		dojo.body().appendChild(blocker.domNode);
		blocker.startup();
		blocker.show();

		var c = dojo.connect(blocker, "onHide", function(){
			dojo.disconnect(c);
			setTimeout(function(){
				blocker.destroy();
			}, 10);
		});

		var urlData = dojo.fromJson(dojo.toJson(params));
						   
		// Give a little time for the chart to 'show'
		setTimeout(dojo.hitch(this, function(){
			newTab.attr("content", "<span></span>");
			// This is junk that has to be added in later by the ChartContainer, as most is pulled from its inputs, I think.
			urlData.chartScopeType = "all";

			// The name field needs to be a descriptor, not translated name.
			var _oldname=urlData.name;
			urlData.name="fullName";
			
			var url=dojo.string.substitute(this.statisticsUrl, urlData);
			this._chartMap[name].url = url;
			
			// restore translated name.
			urlData.name=_oldname;

			var container = this._chartMap[name].container = new xsa.monitor.widgets.dashboard.ChartContainer({url:url}, newTab.domNode.firstChild);
			if(this.saveButton.attr("disabled")){
				// Enable the save button.
				this.saveButton.attr("disabled", false);
			}
			container.startup();
			container.resize(dojo.contentBox(newTab.domNode));
			var bc = dojo.connect(container.dataPres, "renderChartWidget", function(){
				// On render, close the blocker.  The close will initate its destroy.
				dojo.disconnect(bc);
				blocker.hide();
			});
			container.addMetric(urlData.seriesType, urlData.seriesID, urlData.metricType, urlData.name);
		}), 500);
	},

	_createNewSubMenu: function(){
		// summary:
		//		Function to destroy and rebuild the sub menu
		//		of existing charts.
		if(this._treeMenu){
			if(this._treeMenuSeparator){
				this._treeMenu.removeChild(this._treeMenuSeparator);
				this._treeMenuSeparator.destroyRecursive();
				delete this._treeMenuSeparator;
			}
			if(this._chartSubMenu){
				// Remove old list to rebuild it.
				this._treeMenu.removeChild(this._chartSubMenu);
				this._chartSubMenu.destroyRecursive();
				delete this._chartSubMenu;
			}

			var item = this._selectedItem;
			if(item){
                // Build new sub menu.
				var subPopupMenu = new dijit.Menu({parentMenu: this._treeMenu});
				this._chartSubMenu = new dijit.PopupMenuItem({
					label: this._messages.addToExistingChart,
					popup: subPopupMenu,
					iconClass: "xsaInsertIntoChartIcon"
				});												   

				// Add in all the chart names to the popup sub menu.
				var chart;
				var newMenu = false;
				var self = this;

				var params = this._itemToObject(this._selectedItem, this._treeStore);
				for(chart in this._chartMap){
					if(!params.unit || this._chartMap[chart].unit == params.unit){
						// Only include charts in 'add to existing chart' that are of the same units
						// The initial series defines the unit range for the entire chart on the Y axis.

						// Determine if we already have this series in this chart.
						// If we do, we skip listing that chart.
						var series = this._chartMap[chart].params;
						var matches = [];
						var matched = false;
						var m, a, k, p;
						if(series && series.length > 0){
							for(p = 0; p < series.length; p++){
								var selector = series[p];
								m = true;
								for(a in selector){
									if(selector[a] != params[a]){
										m = false;
										break;
									}
								}
								matches.push(m);
							}
						}
						// See if any of the chart selector data matched exactly
						// implying the series is already charted.
						for(k = 0; k < matches.length; k++){
							if(matches[k]){
								matched = true;
								break;
							}
						}
						if(!matched){
							// Only list charts that do not already include the series.
							subPopupMenu.addChild(new dijit.MenuItem({
								label: chart,
								iconClass: "xsaChartIcon",
								onClick: function(){
									var c = self._chartMap[this.label];
									if(c){
										var tab = self._chartMap[this.label].tab;
										if(tab){
											self._tabContainer.selectChild(tab);

											// Build up the parameters we need to work with and
											// then store them in the chart for later comparison.
											var params = self._itemToObject(self._selectedItem, self._treeStore);
											c.params.push(params);

											// Okay, new series, so set up to add it.
											var blocker = new dojox.widget.Standby({
												target: tab.domNode, 
												zIndex: 500, 
												color: "#FFFFFF",
												image: dojo.moduleUrl("xsa", "monitor/widgets/dashboard/styles/images/progress-indicator-large.gif").toString()
											});
											dojo.body().appendChild(blocker.domNode);
											blocker.startup();
											blocker.show();

											// Enable cleanup action for Standby.
											var shc = dojo.connect(blocker, "onHide", function(){
												dojo.disconnect(shc);
												setTimeout(function(){
													blocker.destroy();
												}, 10);
											});

											// Give a little time for the chart to 'show'
											setTimeout(dojo.hitch(self, function(){
												var bc = dojo.connect(c.container.dataPres, "renderChartWidget", function(){
													// On render, close the blocker.  The close will initate its destroy.
													dojo.disconnect(bc);
													blocker.hide();
												});
												c.container.addMetric(params.seriesType, params.seriesID, params.metricType, params.name);
											}), 500);
										}
									}
									dijit.popup.close(self._treeMenu);
								}
							}));
							newMenu = true;
						}
					}
				}
				if(newMenu){
					// Only add it if there are contents.
					this._treeMenuSeparator = new dijit.MenuSeparator();
					this._treeMenu.addChild(this._treeMenuSeparator);
					this._treeMenu.addChild(this._chartSubMenu);
					this._chartSubMenu.startup();
				}else{

					this._chartSubMenu.destroyRecursive();
					delete this._chartSubMenu;	
				}
			}
		}
	},

	_closeTreeMenu: function(){
		// summary:
		//		Multi-used function to close the menu.
		// tags:
		//		private
		dijit.popup.close(this._treeMenu);
	},

	_itemToObject: function(item, store){
		 // summary:
		//		Function to concert a store item to basic JS params.
		// item:
		//		The store item.
		// store:
		//		The store.
		// tags:
		//		private
		var params = {};
		var attrs = store.getAttributes(item);
		if(attrs && attrs.length> 0){
			dojo.forEach(attrs, function(param){
				params[param] = store.getValue(item, param);
			});
		}
		return params; // Object
	},

	destroy: function(){
		// summary:
		//		Over-ridden destroy to handle internal cleanup.
		if(this._tree){
			this._tree.destroy();
			delete this._tree;
		}
		if(this._chartSubMenu){
			delete this._chartSubMenu;
		}
		if(this._treeMenu){
			this._treeMenu.destroyRecursive();
			delete this._treeMenu;
		}
		if(this._newChartDialog){
			this._newChartDialog.destroyRecursive();
			delete this._newChartDialog;
		}
		if(this._saveReportDialog){
			this._saveReportDialog.destroyRecursive();
			delete this._saveReportDialog;
		}
		if(this._loadReportDialog){
			this._loadReportDialog.destroyRecursive();
			delete this._loadReportDialog;
		}
		if(this._deleteReportDialog){
			this._deleteReportDialog.destroyRecursive();
			delete this._deleteReportDialog;
		}
		if(this._chartMap){
			var i;
			for(i in this._chartMap){
				var chart = this._chartMap[i];
				this._tabContainer.removeChild(chart.tab);
				chart.tab.destroy();
			}
			delete this._chartMap;
		}
		delete this._selectedItem;
		this.inherited(arguments);
	},

	_saveCharts: function(){
		// summary:
		//		Function to handle opening the dialog for saving the current configuration of 
		//		charts.	 If a current 'named' chart is loaded, the default save name is
		//		set to it, else it is set to blank.
		if(!this._saveReportDialog){
			this._createSaveReportDialog();
		}
		if(this._currentReportName){
			this._saveReportNameBox.attr("value", this._currentReportName);
		}else{
			this._saveReportNameBox.attr("value", "");
		}
		this._saveReportDialog.show();
	},

	_loadCharts: function(){
		// summary:
		//		Function to handle opening the dialog for loading the current configuration of 
		//		charts.
		 if(!this._loadReportDialog){
			 this._createLoadReportDialog();
		 }
		 this._loadReportDialog.show();
	},

	_chartMapToJson: function(reportName){
		// summary:
		//		Function to convert the chart map of this instance to a serializable JSON object.
		// reportName: String
		//		The name to associate with this report.
		// tags: 
		//		private
		var name;
		var charts = [];
		for(name in this._chartMap){
			var chart = this._chartMap[name];
			var cCopy = {};
			var attr;
			for(attr in chart){
				if(attr != "container" && attr != "tab"){
					cCopy[attr] = dojo.fromJson(dojo.toJson(chart[attr]));
				}
			}
			cCopy.chartName = name;
            charts.push(cCopy);
        }
		return dojo.toJson({charts: charts, name: reportName});
	},

	_restoreCharts: function(chartArray){
		// summary:
		//		Function to restore charts from a saved state.
		// chartArray: Array
		//		An array of chart parameters to recreate a series of charts.
		// tags:
		//		private

		// Clean up/destroy anything existing.
		if(chartArray && chartArray.length){
			var name;
			for(name in this._chartMap){
				var chart = this._chartMap[name];
				if(chart){
					this._tabContainer.removeChild(chart.tab);
					chart.tab.destroyRecursive();
				}
			}
			this._chartMap = {};

			// Now, restore everything.  This is odd, but it's done this
			// way to avoid spam-recreating charts.  This way it will render one at a time
			// as it restores and avoid conflict issues.
			var blocker = new dojox.widget.Standby({
				target: this.domNode,
				imageText: this.standbyText,
				zIndex: 500, 
				color: "#FFFFFF",
				image: dojo.moduleUrl("xsa", "monitor/widgets/dashboard/styles/images/progress-indicator-large.gif").toString()
			});
			dojo.body().appendChild(blocker.domNode);
			blocker.startup();
			blocker.show();
			chartArray = chartArray.reverse(); // Reverse so we can use pop to restore.
            var renderNext = dojo.hitch(this, function(){
				if(chartArray.length > 0){
					// Delay between each restore to give browser 'breathing' room.
					setTimeout(dojo.hitch(this, function(){
						this._restoreChart(chartArray.pop(), renderNext);
					}), 100);
				}else{
					// Set a hook to cleanup the blocker on hiding it, 
					// then hide it.
					var c = dojo.connect(blocker, "onHide", function(){
						dojo.disconnect(c);
						setTimeout(function(){
							blocker.destroy();
						}, 10);
					});
					blocker.hide();
				}
			});
			renderNext();
		}
	},

	_restoreChart: function(chartInfo, completedCallback){
		// summary:
		//		Function to restore a chart and call an optional callback when done.
		// chartInfo: object
		//		The chart data, including name, url, units, and metrics.
		// completecCallback: function
		//		The function to call when rendering is done.
		// tags:
		//		private
		var name = chartInfo.chartName;
		if(name){
			this._chartMap[chartInfo.chartName] = {
				 params: dojo.fromJson(dojo.toJson(chartInfo.params))
			};
		}

		// Okay, lets create the new tab (chart container)!
		var newTab = new dijit.layout.ContentPane({
			title: name,
			closable: true
		});
		this._tabContainer.addChild(newTab);
		this._tabContainer.selectChild(newTab);

		this._chartMap[name].tab = newTab;
		this._chartMap[name].unit = chartInfo.unit;

		// Set up a listener to do cleanup when the tab itself is closed.
		// This will clean up the content pane contents and then 
		// recreate the submenu list of all the chars available for update.
		var self = this;
		var cl = dojo.connect(newTab, "destroy", function(){
			dojo.disconnect(cl);
			var chart = self._chartMap[this.title];
			delete self._chartMap[this.title];

			// Test to see if there are still any charts.  If not, 
			// nothing to save.
			var disable = true;
			for(var t in self._chartMap){
				disable = false;
				break;
			}
			self.saveButton.attr("disabled", disable);
		});

		// Give a little time for the chart to 'show'
		setTimeout(dojo.hitch(this, function(){
			newTab.attr("content", "<span></span>");
			var url = chartInfo.url;
			this._chartMap[name].url = url;

			var container = this._chartMap[name].container = new xsa.monitor.widgets.dashboard.ChartContainer({url:url}, newTab.domNode.firstChild);
			if(this.saveButton.attr("disabled")){
				// Enable the save button.
				this.saveButton.attr("disabled", false);
			}
			container.startup();
			container.resize(dojo.contentBox(newTab.domNode));

			var series = chartInfo.params.reverse();
			var s = series.pop();
			var bc = dojo.connect(container.dataPres, "renderChartWidget", function(){
				// On render, close the blocker.  The close will initate its destroy.
				if(series.length > 0){
					s = series.pop(); // Go to the next metric on the chart
					if(s){
						container.addMetric(s.seriesType, s.seriesID, s.metricType, s.name);
					}
				}else{
					if(completedCallback){
						dojo.disconnect(bc);
						completedCallback();
					}
				}
			});
			 
			// This loops and after each render, the connection pops the next series to render
			// This avoid spam rendering the browser too much.
			if(s){
				container.addMetric(s.seriesType, s.seriesID, s.metricType, s.name);
			}else{
				if(completedCallback){
					completedCallback();
				}
			}
		}), 500);
	},

	_createSaveReportDialog: function(){
		// summary:
		//		Function to create the 'Save Report' dialog
		// tags:
		//		private
		this._saveReportDialog = new dijit.Dialog({
			title: this._messages.saveReportDialogTitle
		});
		this._saveReportDialog.startup();

		var props = {};

		// Config all the translatables for the template for
		// substitution.
		props.id = dijit.getUniqueId(this.id);
		props.newSavePrompt = this._messages.newSavePrompt;
		props.newSaveLabel = this._messages.newSaveLabel;
		props.saveReportButtonLabel = this._messages.saveReportButtonLabel;
		props.cancelChartButtonLabel = this._messages.cancelChartButtonLabel;
		props.saveStatusLabel = this._messages.saveStatusLabel;
		props.initialSaveStatus = this._messages.initialSaveStatus;

        this._saveReportDialog.attr("content", 
			dojo.string.substitute(this._saveDialogTemplate, props));

		// Find all our widgets we created so we can link them up.
		this._saveReportButton = dijit.byId(props.id + "_saveButton");
		this._cancelSaveReportButton = dijit.byId(props.id + "_cancelSaveButton");
		this._saveReportNameBox = dijit.byId(props.id + "_saveReportNameBox");
		this._saveStatus = dojo.byId(props.id + "_saveStatusMessage");

		//Initial state of createButton should be disabled.  Only enabled when valid.
		this._saveReportButton.attr("disabled", true);
		this._updateSaveButton();

		this.connect(this._saveReportNameBox, "onChange", "_updateSaveButton");

		// Connect up the events.
		this.connect(this._cancelSaveReportButton, "onClick", function(){
			// Cancel just closes the dialog.
			this._saveReportDialog.hide();
			this._saveReportNameBox.attr("value", "");
			this._saveStatus = this._messages.initialSaveStatus;
		});

		// Connect up the events.
		this._connectToDialogOK(this._saveReportButton, this._saveReportNameBox, function(){
			// Create adds the new chart, then closes trhe dialog.
			if(this._saveReportNameBox.isValid()){
				var reportName = dojo.trim(this._saveReportNameBox.attr("value"));
				// Generate the chart data to store.
				var chartJson = this._chartMapToJson(reportName);
				
				// The name is part of the URI, so we have to escape it
				var nameUri = escape(reportName);
				var targetUrl = this.reportUrl + "/" + nameUri;

				// Test to see if the report name exists as it will dictate if it is a PUT
				// or a POST to save it.
				var self = this;
				var args = {
					url: targetUrl,
					preventCache: true,
					handleAs: "text",
					failOk: true,
					load: function(data){
						// Exists, so we need to PUT.
						args.putData = chartJson;
						args.headers = {
							"Content-Type": "application/json"
						};
						delete args.load;
						delete args.error;
						var putDeferred = dojo.xhrPut(args);
						putDeferred.addCallback(function(data){
							self._saveStatus.innerHTML = self._messages.saveSuccessfulStatus;
							setTimeout(function(){
								self._saveReportDialog.hide();
								
								// Set the current report name and enable delete.
								self._currentReportName = reportName;
								self.deleteButton.attr("disabled", false);
								self._title.innerHTML = dojo.string.substitute(self._messages.titleTextWithName, {"0": self._currentReportName});
								self._saveReportNameBox.attr("");

								self._saveStatus.innerHTML = self._messages.initialSaveStatus;
							}, 1500);
						});
						putDeferred.addErrback(function(err){
							 self._saveStatus.innerHTML = self._messages.saveFailedStatus;
						});
					},
					error: function(error, ioargs){
						if(ioargs.xhr.status === 404){
							// Doesn't exist, PUT it.
							args.postData = chartJson;
							args.headers = {
								"Content-Type": "application/json"
							};
							delete args.load;
							delete args.error;
							var postDeferred = dojo.xhrPost(args);
							postDeferred.addCallback(function(data){
								self._saveStatus.innerHTML = self._messages.saveSuccessfulStatus;
								setTimeout(function(){
									self._saveReportDialog.hide();

									// Set the current report name and enable delete.
									self._currentReportName = reportName;
									self.deleteButton.attr("disabled", false);
									self._title.innerHTML = dojo.string.substitute(self._messages.titleTextWithName, {"0": self._currentReportName});
									self._saveReportNameBox.attr("");

									self._saveStatus.innerHTML = self._messages.initialSaveStatus;
								}, 1500);
							});
							postDeferred.addErrback(function(err){
								 self._saveStatus.innerHTML = self._messages.saveFailedStatus;
							});
						}else{
							self._saveStatus.innerHTML = self._messages.failedLookupStatus;
						}
					}
				};
				dojo.xhrGet(args);
			}
		});
	},

	_updateSaveButton: function(){
		// summary:
		//		Simple updater function to toggle the save button on and off.
		var value = this._saveReportNameBox.attr("value") || "";
		if(this._saveReportNameBox.isValid() && dojo.trim(value) !== ""){
			this._saveReportButton.attr("disabled", false);
			if(this._checkName){
				clearTimeout(this._checkName);
				delete this._checkName;
			}

			// Do a delay check on the name entered to see if it exists
			// if it does, then denote such in the status so that people 
			// are aware they will over-write a report.
			this._checkName = setTimeout(dojo.hitch(this, function(){
				delete this._checkName;
				var reportName = dojo.trim(this._saveReportNameBox.attr("value"));
				var nameUri = escape(reportName);
				var targetUrl = this.reportUrl + "/" + nameUri;
				var self = this;
				var args = {
					url: targetUrl,
					preventCache: true,
					handleAs: "text",
					failOk: true,
					load: function(data){
						self._saveStatus.innerHTML = self._messages.saveNameExistsStatus;
					},
					error: function(error, ioargs){
						if(ioargs.xhr.status === 404){
							// Doesn't exist, PUT it.
							self._saveStatus.innerHTML = self._messages.saveNameDoesNotExistStatus;
						}else{
							self._saveStatus.innerHTML = self._messages.failedLookupStatus;
						}
					}
				};
				dojo.xhrGet(args);
			}), 300);
		}else{
			// Empty or invalid, set state back to inital.
			if(this._checkName){
				clearTimeout(this._checkName);
				delete this._checkName;
			}
			this._saveStatus.innerHTML = this._messages.initialSaveStatus;
			this._saveReportButton.attr("disabled", true);
		}
	},

	_createLoadReportDialog: function(){
		// summary:
		//		Function to create the 'Load Report' dialog
		// tags:
		//		private
		this._loadReportDialog = new dijit.Dialog({
			title: this._messages.loadReportDialogTitle
		});
		this._loadReportDialog.startup();

		var props = {};

		// Config all the translatables for the template for
		// substitution.
		props.id = dijit.getUniqueId(this.id);
		props.loadReportLabel = this._messages.loadReportLabel;
		props.loadReportButtonLabel = this._messages.loadReportButtonLabel;
		props.cancelChartButtonLabel = this._messages.cancelChartButtonLabel;
		props.newLoadPrompt = this._messages.newLoadPrompt;

        this._loadReportDialog.attr("content", 
			dojo.string.substitute(this._loadDialogTemplate, props));

		// Find all our widgets we created so we can link them up.
		this._loadReportButton = dijit.byId(props.id + "_loadButton");
		this._cancelLoadReportButton = dijit.byId(props.id + "_cancelLoadButton");
		this._loadReportNameBox = dijit.byId(props.id + "_loadReportNameBox");

		//Initial state of createButton should be disabled.  Only enabled when valid.
		this._createChartButton.attr("disabled", true);
		this._updateLoadButton();

		this.connect(this._loadReportNameBox, "onChange", "_updateLoadButton");

		// Connect up the events.
		this.connect(this._cancelLoadReportButton, "onClick", function(){
			// Cancel just closes the dialog.
			this._loadReportDialog.hide();
			this._loadReportNameBox.attr("displayedValue", "");
		});

		// Connect up the events.
		this._connectToDialogOK(this._loadReportButton, this._loadReportNameBox, function(){
			// Function to pull the data off the selected 'item' and recreate the charts from it.
			if(this._loadReportNameBox.isValid()){
				this._loadReportDialog.hide();
				var item = this._loadReportNameBox.item;
				var charts = this._loadReportNameBox.store.getValues(item, "charts");

				// Get th chart name, enable the delete button, and set the title text to the report name.
				this._currentReportName = this._loadReportNameBox.store.getValue(item, "name");
				this.deleteButton.attr("disabled", false);
				this._title.innerHTML = dojo.string.substitute(this._messages.titleTextWithName, {"0": this._currentReportName});
				
				// Wait a second for the dialog to hide before invoking the restore.
				setTimeout(dojo.hitch(this, function(){
					this._restoreCharts(charts);
				}), 500);
			}
		});

		this.connect(this._loadReportDialog, "onShow", function(){
			// Function to update the store with the latest list of reports from the server.
			var self = this;
			var args = {
				url: this.reportUrl,
				handleAs: "json",
				load: function(reportData){
                    self._loadReportNameBox.store = new dojo.data.ItemFileReadStore({data: {items: reportData}, hierarchical: false});
				},
				error: function(){
					 self._loadReportNameBox.store = new dojo.data.ItemFileReadStore({data: {items: []}, hierarchical: false});
				}
			};
			dojo.xhrGet(args);
		});
	},

	_updateLoadButton: function(){
		// summary:
		//		Simple updater function to toggle the load button on and off.
		// tags:
		//		private
		if(this._loadReportNameBox.isValid()){
			this._loadReportButton.attr("disabled", false);
		}else{
			this._loadReportButton.attr("disabled", true);
		}
	},

	_deleteCharts: function(){
		// summary:
		//		Function to delete the collection of charts from the server.
		// tags:
		//		private
		if(!this._deleteReportDialog){
			this._createDeleteReportDialog();
		}
		this._deleteReportDialog.show();
	},

	_createDeleteReportDialog: function(){
		// summary:
		//		Function to create the 'Load Report' dialog
		// tags:
		//		private
		this._deleteReportDialog = new dijit.Dialog({
			title: this._messages.deleteDialogTitle
		});
		this._deleteReportDialog.startup();

		var props = {};

		// Config all the translatables for the template for
		// substitution.
		props.id = dijit.getUniqueId(this.id);
		props.yesLabel = this._messages.yesLabel;
		props.noLabel = this._messages.noLabel;

        this._deleteReportDialog.attr("content", 
			dojo.string.substitute(this._deleteDialogTemplate, props));

		// Find all our widgets we created so we can link them up.
		this._deleteReportButton = dijit.byId(props.id + "_deleteButton");
		this._cancelDeleteReportButton = dijit.byId(props.id + "_cancelDeleteButton");
		this._deleteText = dojo.byId(props.id + "_deleteQuestion");

		// Connect up the events.
		this.connect(this._cancelDeleteReportButton, "onClick", function(){
			// Cancel just closes the dialog.
			this._deleteReportDialog.hide();
		});

		this.connect(this._deleteReportButton, "onClick", function(){
			var self = this;
			var args = {
				url: this.reportUrl + "/" + escape(this._currentReportName),
				handleAs: "text",
				failOk: true,
				load: function(data){
					delete self._currentReportName;
					self.deleteButton.attr("disabled", true);
					self._title.innerHTML = self._messages.titleText;
					self._deleteReportDialog.hide();
				},
				error: function(){
					 self._deleteReportDialog.hide();
				}
			};
			dojo.xhrDelete(args);
		});

		this.connect(this._deleteReportDialog, "onShow", function(){
			// Function to update the store with the latest list of reports from the server.
			this._deleteText.innerHTML = dojo.string.substitute(this._messages.deleteQuestion, {"0": this._currentReportName});
		});
	}
});
