/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.layout.IndividualDataCacheLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.charting.DataChart");
dojo.require("dojox.charting.widget.Legend");
dojo.require("dojox.grid.DataGrid");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("xsa.monitor.widgets.overview.CannedChart");
dojo.require("xsa.monitor.widgets.overview.MetricSnapshot");
dojo.requireLocalization("xsa.monitor.widgets.layout", "xsa_messages");

dojo.declare("xsa.monitor.widgets.layout.IndividualDataCacheLayout", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/layout/templates/IndividualDataCacheLayout.html"),
	widgetsInTemplate: true,

    xsa_messages : null,
    xsa_messages_overview : null,
    
    constructor: function(kwArgs) {
        console.log("base dashboard constructor called....");
		if(!kwArgs) { 
			kwArgs = {};
		}

        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.layout", "xsa_messages");
        this.xsa_messages_overview = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        if ( this.xsa_messages ) {
            console.log("found messages ", this.xsa_messages);
        }

        this.tabTitle1 = this.xsa_messages_overview.usedBytes;
        this.tabTitle2 = this.xsa_messages_overview.cacheChart;
        this.tabTitle3 = this.xsa_messages_overview.avgTP;
    },
    
    postCreate: function() {
        if ( this.splashScreen ) {
	        Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
	        Bedrock.hide(this.dashboardView);
        }
        var options = {
            contentDefault: messages.RM11006
        };
        this.defaultMessage.innerHTML = "<p align='center'><span class='dashboard-detail-view-default-message-text'>" + options.contentDefault + "</span></p>";
    },

	startup: function() {
	    console.log("IndividualDataCacheLayout:startup");
	},
	    
    destroy: function() {
        console.log("IndividualDataCacheLayout:destroy");
		if(this.chart3) this.chart3.destroy();
		if(this.chart2) this.chart2.destroy();
		if(this.chart) this.chart.destroy();
		if(this.metric) this.metric.destroy();
		if(this._tabDiv) this._tabDiv.destroyRecursive();
    },
    
    //
    // selectResource() modifies the data shown in the detail panels to be that 
    // of the resource with the given URI/ID. The action toolbar will also be 
    // updated to reflect the change.
    //
    selectResource: function(uri, id, dontShowAfter) {
		console.log("select resource " + uri + " id = " + id);
		this.server = id;
        Bedrock.hide(this.defaultMessage);
        
        if (dojo.style(this.splashScreen, 'display') == 'none' && !dontShowAfter)
        {
            console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
        
        //SMP: Bug 19879 & 19880
        //from ActionMenu.js: to make sure focus moves to detail panel after item is selected
        console.log("IndividualDataCacheLayout.selectResource this", this);
        // parent node is the div passed in constructor
        var parentElement = this.dashboardView.parentNode;        
        // set tab index to -1 for screen reader to acknowledge
        parentElement.setAttribute("tabIndex", "-1");
		// set aria live to polite to indicate to screen reader panel updates dynamically
		parentElement.setAttribute("aria-live", "polite");
        dijit.focus(parentElement);
        		
   		this.selectedPanel.innerHTML = this.xsa_messages_overview.XSA30101A+" "+this.server;
		dojo.style( this.selectedPanel, 'display: block;');
		dojo.removeClass(this.selectedPanel, 'heading');
		dojo.removeClass(this.selectedPanel, 'action-menu-default-message');		
		dojo.addClass(this.selectedPanel, 'action-menu-heading');
		
		console.log("now show the dashboard view");
        Bedrock.show(this.dashboardView);
		
		if ( this.currentSelectedId != this.server ) {
	        this.inherited(arguments);
		    console.log("IndividualDataCacheLayout:selectResource");

			this.createMetricSnapshot1();

	        tabs = dijit.byId("_tabDiv");
	        console.debug("*********************_tabs: ", tabs);
		    tabs.startup();
		    //tabs.layout();
		    tabs.resize();
		    this.tabSelected(tabs.selectedChildWidget);
			
	        console.debug("tabs.selectedChildWidget: ", tabs.selectedChildWidget);
	        dojo.connect(tabs,"_transition", dojo.hitch(this, this.tabSelected));
	        
	        //Hack because initial selected tab won't load.
	        //tabs.selectChild(tabs.getChildren()[1]);
	        //tabs.selectChild(tabs.getChildren()[0]);
		}

        this.currentSelectedId = this.server;
        
        // only apply to IE because IE doesn't handle overflow properly.
        if (!/*@cc_on!@*/0) return;

        // find every element to test
        var all = document.getElementsByTagName('*'), i = all.length;

        // fast reverse loop
        while (i--) {
          // if the scrollWidth (the real width) is greater than
          // the visible width, then apply style changes
          if (all[i].scrollWidth > all[i].offsetWidth) {
            all[i].style['paddingRight'] = '20px';
            all[i].style['overflowX'] = 'hidden';
          }
        }
    },
    
	tabSelected: function(newTab, oldTab) {
		console.debug("Previous tab ", oldTab || "nothing");
		console.debug("New Tab ", newTab);
		if (newTab.id == "usedBytes") {
		    this.createCannedChart3();
		} else if (newTab.id == "totalGetCount") {
			this.createCannedChart2();
		} else if (newTab.id == "avgTP") {
			this.createCannedChart();
		}
	},

	createCannedChart3: function() {
	    console.log("create Chart 3 for this grid " + this.server);

	    if (this.chart3) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.chart3.domNode, "before");
	       this.chart3.destroy();
	       this._usedBytesCannedChartDiv = newDiv;
	    }

		var dataSeries = [
		  {metricType: 'usedBytes', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'primary'},
		  {metricType: 'numEntries', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'secondary'},
          {metricType: 'limit', numberDatapoints: '30', timeRange: '1hr', 
              type: 'datagrid', seriesId: this.server, scale: '', axis: 'primary', hideOnNegative: 'true'}
		];
		
		var p_cannedChart = {
	      //chartType: "StackedBars",
		  chartDescriptors: dataSeries,
		  scopeId:'all',
		  type:'datagrid',
		  //dualAxis:true
		  ariaLevel:'3'
		};

		console.log("Attach p:", p_cannedChart);
		this.chart3 = new xsa.monitor.widgets.overview.CannedChart(p_cannedChart, this._usedBytesCannedChartDiv);
		console.log("createCannedChart3", this.chart3);
		//this.chart.render();
		this.chart3.startup();
	},
	
	createCannedChart2: function() {
	    console.log("create Chart 2 for this grid " + this.server);

	    if (this.chart2) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.chart2.domNode, "before");
	       this.chart2.destroy();
	       this._totalGetCountCannedChartDiv = newDiv;
	    }

		var dataSeries = [
		  {metricType: 'totalGetCount', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'primary'},
		  {metricType: 'totalHitCount', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'primary'},
		  {metricType: 'hitRate', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'secondary'}
		];
		
		var p_cannedChart = {
	      //chartType: "StackedBars",
		  chartDescriptors: dataSeries,
		  scopeId:'all',
		  type:'datagrid',
		  //dualAxis:false,
		  titleKey:'cacheChart',
		  ariaLevel:'3'
		};

		console.log("Attach p:", p_cannedChart);
		this.chart2 = new xsa.monitor.widgets.overview.CannedChart(p_cannedChart, this._totalGetCountCannedChartDiv);
		console.log("createCannedChart2", this.chart2);
		//this.chart.render();
		this.chart2.startup();
	},
	
	createCannedChart: function() {
	    console.log("create Chart  for this grid " + this.server);

	    if (this.chart) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.chart.domNode, "before");
	       this.chart.destroy();
	       this._avgTPSparklineDiv = newDiv;
	    }

		var dataSeries = [
		  {metricType: 'avgTP', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'primary'},
		  {metricType: 'avgTranTime', numberDatapoints: '30', timeRange: '1hr', 
			  type: 'datagrid', seriesId: this.server, scale: '', axis: 'secondary'}
		];
		
		var p_cannedChart = {
	      //chartType: "StackedBars",
		  chartDescriptors: dataSeries,
		  scopeId:'all',
		  type:'datagrid',
		  //dualAxis:true
		  ariaLevel:'3'
		};

		console.log("Attach p:", p_cannedChart);
		this.chart = new xsa.monitor.widgets.overview.CannedChart(p_cannedChart, this._avgTPSparklineDiv);
		console.log("createCannedChart", this.chart);
		//this.chart.render();
		this.chart.startup();
	},
	
	createMetricSnapshot1: function() {
	    console.log("create metric snapshot 1 for this grid " + this.server);

	    if (this.metric) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.metric.domNode, "before");
	       this.metric.destroy();
	       this.metricSnapshotDiv1 = newDiv;
	    }

		var dataSeries = [
		  {metricType: 'numEntries', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'avgTranTime', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'avgTP', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'hitRate', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'gridCap', type: 'datagrid', seriesId: this.server, scale: '', hideOnNegative: 'true'}
		];
		
		var p_metric = {
		  titleKey: 'XSA30502',
		  chartDescriptors: dataSeries,
		  scopeId:'all',
		  type:'datagrid',
		  ariaLevel:'3'
		};

		console.log("Attach p:", p_metric);
		this.metric = new xsa.monitor.widgets.overview.MetricSnapshot(p_metric, this.metricSnapshotDiv1);
		console.log("createMetricSnapshot1", this.metric);
		this.metric.startup();
	}
    
});
