/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.HiLowChart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

//dojo.require("dojox.widget.DataPresentation");
dojo.require("dojox.charting.Theme");
dojo.require("dojox.charting.themes.Distinctive");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require("xsa.monitor.widgets.ext.DataPresentation");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.HiLowChart", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/HiLowChart.html"),
     widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,
    
    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the type of metric that will be represented in this sparkline metric */
    metricType : null,
    /* the type of data artifact to aggregate the stats to (ie datacache, grid, map) */
    type : null,
    /* the time range to show */
    timeRange: null,
    /* the scope "all" or fully qualified name of artifact to scope the chart to , like "anngrid" or "anngrid!annMap" */
    scopeId : null,
    /* the number of contributors to the breakdown, the rest will be grouped into an "other" category */
    numContribs: null,
    /* the message key of title of the pool breakdown - default is set on rest call if not set */
    titleKey : null,
    titleKeyParms : null,
    /* the message key of subtitle of the pool breakdown - default is set on rest call if not set */
    subtitleKey : null,
    subtitleKeyParms : null,
    
    /* set by the script */
    responseData : null,
	
	_widgets : null,

    constructor: function(kwArgs) {
        console.log("HiLowChart dashboard constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.metricType = kwArgs.metricType ? kwArgs.metricType : "avgTP";
          this.type = kwArgs.type ? kwArgs.type : "datacache";
          this.timeRange = kwArgs.timeRange ? kwArgs.timeRange : "1hr";
          this.scopeId = kwArgs.scopeId ? kwArgs.scopeId : "all";
          this.numContribs = kwArgs.numContribs ? kwArgs.numContribs : "1";
          this.titleKey = kwArgs.titleKey ? kwArgs.titleKey : "XSA30300A";
          this.titleKeyParms = kwArgs.titleKeyParms ? kwArgs.titleKeyParms : [{parmName: "${_METRICTYPE_}", parmValue: this.metricType}];
          this.subtitleKey = kwArgs.subtitleKey ? kwArgs.subtitleKey : "XSA30300B";
          this.subtitleKeyParms = kwArgs.subtitleKeyParms ? kwArgs.subtitleKeyParms : [{parmName: "${_UNITS_}", parmValue: "u_"+this.metricType}];
          
          this.ariaLevel = kwArgs.ariaLevel ? kwArgs.ariaLevel: '1';
        
          /* set up the messages */
          this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
          if ( this.xsa_messages ) {
               console.log("found messages ", this.xsa_messages);
               this.chartTabTitle = this.xsa_messages.XSA30120;
               this.tableTabTitle = this.xsa_messages.XSA30121;
          }
            
		  this._widgets = [];
    },
     
    postCreate: function() {
        console.log("HiLowChart dashboard postCreate called....");
        if ( this.splashScreen ) {
             console.log("hide splashScreen");
             Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
             console.log("hide dashboardView");
             Bedrock.hide(this.dashboardView);
        }

        /* load up the sparkline data */
        this.loadHiLowData();
        
        //Bug 19537 Hack: not sure why root template
        //title is being set to [object divElement]
        this.hilowRootDiv.title = "";
    },

    //
    //	this function removed invalid WAI-ARIA state.
    //  aria-pressed is invalid for a tab
    //
    _removeWaiAttributes: function(){
    	var children = this.tabContainer.tablist.getChildren();
        dojo.forEach(children,function(child){
            dijit.removeWaiState(child.focusNode || child.domNode,"pressed");
        });
    },

     buildHiLowChart: function(response, ioArgs){
        console.log("HiLowChart dashboard buildHiLowChart called: "+response);
		//create the chart
		try{
			var newTheme = new dojox.charting.Theme({ fillcolors: ["red", "green", "blue"], colors: ["lightpink", "lightgreen", "lightblue"] });
			this._dp = new xsa.monitor.widgets.ext.DataPresentation(this.hilowchart, {
					type: "chart",
					chartType: "ClusteredColumns",
					data: response,
					series: dojo.hitch(this, this.makeseries),
					labelMod: 1,
					gridNode:this.griddiv,
					titleNode: this.title,
					footerNode: this.footer,
					animate: true,
					//theme: "dojox.charting.themes.Distinctive",
                    theme: newTheme
			});
			console.debug("chart",[this._dp]);
            this._widgets.push(new dojox.charting.action2d.Highlight(this._dp.chartWidget, "default"));

            var anim = new dojox.charting.action2d.Tooltip(this._dp.chartWidget, "primary-ClusteredColumns",{text: dojo.hitch(this, this.getToolTipText)});            
            var legend = new dojox.charting.widget.Legend({chart: this._dp.chartWidget, horizontal: false}, this.hilowlegend);
			
			this._widgets.push(newTheme);
			this._widgets.push(anim);
			this._widgets.push(legend);
	    //
	    //  add role to comply with WAI-ARIA
	    //
	    dojo.attr(legend.legendNode, "role", "presentation");
		 }catch(err){
			  console.debug("error",[err]);
		 }
		
        /* var hiLowChart = this.createHiLowChart(response);
        console.debug('render chart',[hiLowChart]);
        hiLowChart.render(); */
        
        Bedrock.fadeCommon(this.splashScreen);
        if ( this.dashboardView ) {
            Bedrock.show(this.dashboardView);
        }
	var children = this.tabContainer.tablist.getChildren();
	for( var i = 0; i < children.length; ++i )
	{
	    dojo.connect(children[i],"onClick",dojo.hitch(this,this._removeWaiAttributes));
	}
        this._removeWaiAttributes();	
     },
	
     getToolTipText: function(field){
         console.debug("field",[field]);
         console.debug("type",[this.xsa_messages[this.type+"_s"]]);
         var typeLabel = this.xsa_messages[this.type+"_s"]+" ";
         var typeValue = this.responseData.series[this.getXIndex(field)].values[parseInt(field.x)].name;
         
         //Handle dynamic data size units.
         var unitLabel = "";
         var units = this.responseData.series[this.getXIndex(field)].units;
         if (units == undefined) {
        	 unitLabel = this.xsa_messages["u_"+this.metricType];
         } else {
        	 unitLabel = this.xsa_messages[units+"_abv"];
         }
        	 
         
         var item = "<b>"+typeLabel+typeValue+"</b><br />";
         console.debug("item",[item]);
         if (typeValue == "all") {
             item = "";
         }
         return item+this.xsa_messages.XSA30122+" "+field.y+" "+unitLabel;
     },
     
     getXIndex: function(field) {
    	 //This will return the wrong index if identical values exist.
         for (var i=0; i<field.plot.series.length; i++) {
		     var data = field.plot.series[i].data;
			 for (var j=0; j<data.length; j++) {
			     if (data[j] == field.y) {
			         return i;
			     }
			 }
         }
         return 0;
     },

     loadHiLowData: function(){
        console.log("HiLowChart dashboard loadHiLowData called....");        
        var deferred = dojo.xhrGet({
           url: "/resources/chartdata/"+this.type,
           handleAs: "json",
           preventCache: true,
           content: {
               scopeId: this.scopeId,
               chartType: "highlow",
               axisLabel: "range",
               maxContributors: this.numContribs,
               chartDescriptor: this.metricType+":all:"+"all"+":"+this.type+":all:;"+this.metricType+":all:"+"all"+":"+this.type+":high:;"+this.metricType+":all:"+"all"+":"+this.type+":low:"
           },
		   headers: {
              'Accept': 'application/json'
           },
           load: dojo.hitch(this, this.buildHiLowChart),
           error: function(error) {
                /* TODO: create a row that will span all cells with the error message */
               console.log("An unexpected error occurred: " + error);
           }
        });
        
        console.log("done, waiting for callback....");

        if (dojo.style(this.splashScreen, 'display') == 'none')
        {
            console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
     },

	 makeseries: function(data) {
	   console.debug("makeseries",[data]);
	   this.responseData = data;
	   
	   //this allows for some manipulation of the data before it really gets processed/used by the DP widget.
	   //override the title and footer
	   var title = this.translate(this.titleKey, this.titleKeyParms, data);
	   console.debug("title",[title]);
	   data.title=title;

	   
	   if (this.responseData.series[0].units) {
           this.subtitleKeyParms = [{parmName: "${_UNITS_}", parmValue: this.responseData.series[0].units}];
	   }
	   
	   var footer = this.translate(this.subtitleKey, this.subtitleKeyParms, data);
	   console.debug("footer",[footer]);
	   data.footer=footer;

	   var name = this.xsa_messages.XSA30123;
	   console.debug("name",[name]);
	   
	   //Set hilow legend labels
       for (var i=0; i<data.series.length; i++) {
           var legend = this.translate("legend_"+data.series[i].legend, [], data);
           data.series[i].legend = legend;
       }
       
       console.debug("data",[data]);
       //If n+1 time period has an average of 0, then it should use the same
       // average as the nth time period.  For example, if the hour,day,week have
       // averages of 4.04,3.03,0 respectively, then the week should use the day
       // average.
       if (data.series[0]) {
		   var timeAvg = data.series[0].values[0].value;
    	   for (var i_avg=1; i_avg<data.series[0].values.length; i_avg++) {
    		   var nextTimeAvg = data.series[0].values[i_avg].value;
           	   console.debug("Time avg / next time avg",[timeAvg, nextTimeAvg]);
                if (nextTimeAvg == 0) {
                    data.series[0].values[i_avg].value = timeAvg;
                    data.series[0].scaledValues[i_avg] = timeAvg;
                }
                timeAvg = nextTimeAvg;
    	   }
       }
       
       //If hour max is greater than day/week/month max than set
       // day/week/month max to hour max.
       if (data.series[1]) {
    	   var hourMax = data.series[1].values[0].value;
    	   for (var i_max=1; i_max<data.series[1].values.length; i_max++) {
    		    var timeMax = data.series[1].values[i_max].value;
           	    console.debug("Hour max / item Max",[hourMax, timeMax]);
                if (hourMax > timeMax) {
                    data.series[1].values[i_max].value = hourMax;
                    data.series[1].values[i_max].fullName = data.series[1].values[0].fullName;
                    data.series[1].values[i_max].name = data.series[1].values[0].name;
                    data.series[1].scaledValues[i_max] = hourMax;
                } else {
                    hourMax = timeMax;
                }
    	   }
       }
       //If hour min is greater than day/week/month min than set
       // day/week/month min to hour min.
       if (data.series[2]) {
    	   var hourMin = data.series[2].values[0].value;
    	   for (var i_min=1; i_min<data.series[2].values.length; i_min++) {
    		   var timeMin = data.series[2].values[i_min].value;
    		   console.debug("Hour min / item min",[hourMin, timeMin]);
               if (hourMin < timeMin) {
                   data.series[2].values[i_min].value = hourMin;
                   data.series[2].values[i_min].fullName = data.series[2].values[0].fullName;
                   data.series[2].values[i_min].name = data.series[2].values[0].name;
                   data.series[2].scaledValues[i_min] = hourMin;
               } else {
                   hourMin = timeMin;
               }
    	   }
       }
       
       
       //Remove last month stats
       //for (var i=0; i<data.series.length; i++) {
           //data.series[i].scaledValues.pop();
           //data.series[i].range.pop();
           //data.series[i].values.pop();
       //}
       //data.range.pop();
       
       //Inject current stats
       /* for (var i=0; i<data.series.length; i++) {
           data.series[i].scaledValues.unshift(data.series[i].current);
           data.series[i].range.unshift("current");
           data.series[i].values.unshift({ fullname: "", name: "", value: data.series[i].current });
       }
       data.range.unshift("current");*/

	   //Set hilow time range labels
       for (var i=0; i<data.series[0].range.length; i++) {
           var range = this.translate("range_"+data.series[0].range[i], [], data);
           data.series[0].range[i] = range;
       }
	   
	   console.debug("data",[data]);
	   
	   var txt = [];
	   txt.push({ datapoints: "series[0].range", name: name, type: "series[0].range", chart: false })
       for (var i=0; i<data.series.length; i++) {
             txt.push({
                 datapoints: "series["+i+"].scaledValues",
                 namefield: "series["+i+"].legend"
             });
       }

	   console.debug("txt", txt);
       return txt;
	 },
	 
	translate: function(key, parms, data){
	   console.debug("translate enter",[key,parms,data]);
	   var message = this.xsa_messages[key];
	   if (parms){
	       var parm;
	       for (parm in parms){
                if (this.xsa_messages[parms[parm].parmValue]){
                    //look for the message key sub parm to be another message key
                    var name = parms[parm].parmName;
                    var value = this.xsa_messages[parms[parm].parmValue];
                    message = message.replace(name,value);
                }else{
                    //check if it is set in the data
                    if (this.responseData[parms[parm].parmValue]){
						var name2 = parms[parm].parmName;
                        console.debug("name2",[name2]);
                        var value2 = this.xsa_messages[this.responseData[parms[parm].parmValue]];
						console.debug("value2",[value2]);
                        message = message.replace(name2,value2);
                    }
              }
	       }
	   }
	   console.debug("translate exit",[message]);
	   return message;
	},
	
	destroy: function() {
		if(this._dp){
			this._dp.destroy();
			this._dp = null;
		}
		dojo.forEach(this._widgets, function(widget){
			if(widget.destroyRecursive) {
				widget.destroyRecursive();
			} else if(widget.destroy){
				widget.destroy();
			}
		});
		this.inherited(arguments);
    }
     
});
