/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.dashboard.BaseDashboard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.charting.DataChart");
dojo.require("dojox.charting.widget.Legend");
dojo.require("dojox.grid.DataGrid");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dojox.charting.action2d.Magnify");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.action2d.Highlight");

dojo.require("xsa.monitor.widgets.dashboard.SelectableLegend");
dojo.require("xsa.monitor.widgets.data.IntervalStore");
dojo.require("xsa.monitor.widgets.dashboard.ScalableDataChart");
dojo.requireLocalization("xsa.monitor.widgets.dashboard", "xsa_messages");

dojo.declare("xsa.monitor.widgets.dashboard.BaseDashboard", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/dashboard/templates/BaseDashboard.html"),
	widgetsInTemplate: true,
    
    xaxis : null,
    yaxis : null,
    layout : null,
    started : false,
    interval : null,
    field : null,
    fieldMap : {},
    fieldMap2 : {},
    selectedItems : {},
    xsa_messages : null,
    
    constructor: function(kwArgs) {
        console.log("base dashboard constructor called....");
		if(!kwArgs) { 
			kwArgs = {};
		}
		this.widgetType = kwArgs.type ? kwArgs.type : "servers";
		this.subType = kwArgs.subtype ? kwArgs.subtype : "procs";
        this.initField = kwArgs.init ? kwArgs.init : "availProcs";
        this.field = this.initField;
        console.log("this.field is " + this.field);
        this.scroll = kwArgs.scroll ? kwArgs.scroll : false;
		this.fieldStore = new dojo.data.ItemFileReadStore({
        	url: "/resources/fields/" + this.widgetType 
        });
        this.selectedItem = null;
        this.toolTip = null;
        this.gridSortedFlag = false;
        
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.dashboard", "xsa_messages");
        if ( this.xsa_messages ) {
        	console.log("found messages ", this.xsa_messages);
        }
    },
    
    postCreate: function() {
        if ( this.splashScreen ) {
	        Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
	        Bedrock.hide(this.dashboardView);
        }
        var options = {
            contentDefault: messages['RM11006']
        };
        this.defaultMessage.innerHTML = "<p align='center'><span class='dashboard-detail-view-default-message-text'>" + options.contentDefault + "</span></p>";        
    	this.startPauseButton.setLabel("Resume Chart Update");
    	this.showMessage.innerHTML = "Show:";
    	//this.autoRefresh.innerHTML = "Auto Refresh";
		this.createGraphSelect();
    },

	startup: function() {
	},
	    
    destroy: function() {
    	this.stopCharting();
    },
    
    //
    // selectResource() modifies the data shown in the detail panels to be that 
    // of the resource with the given URI/ID. The action toolbar will also be 
    // updated to reflect the change.
    //
    selectResource: function(uri, id, dontShowAfter) {
		console.log("select resource " + uri + " id = " + id);
        Bedrock.hide(this.defaultMessage);

        if (dojo.style(this.splashScreen, 'display') == 'none' && !dontShowAfter)
        {
        	console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
        
		this.selectedPanel.innerHTML = id;
		dojo.style( this.selectedPanel, 'display: block;');
		dojo.removeClass(this.selectedPanel, 'heading');
		dojo.removeClass(this.selectedPanel, 'action-menu-default-message');		
		dojo.addClass(this.selectedPanel, 'action-menu-heading');
		
		this.server = id;
		
		console.log("now show the dashboard view");
        Bedrock.show(this.dashboardView);    	
		
		if ( this.currentSelectedId != id ) {
			console.log("we have a new resource, build a new store");
    		// we need a new store
    		this.stopCharting();
	        
	        this.store = new xsa.monitor.widgets.data.IntervalStore({ 
	        	url: '/resources/' + this.widgetType + '/' + id + '/' + this.subType,
	        	clearOnClose : true, 
	        	urlPreventCache: true 
	        });
		    // add on set connection
			dojo.connect(this.store, "onComplete", this, "onComplete");
		}

        this.field = this.initField;
    	if ( this.currentSelectedId && this.currentSelectedId != id ) {
    		console.log("changed resources, destroy charting widgets");
    		if ( this.chart ) {
    			this.chart.destroy();
    			this.chart = null;
    		}
    		if ( this.legend ) {
    			this.legend.destroy();
    			this.legend = null;
    		}
    		if ( this.grid ) {
    			this.grid.destroy();
    			this.grid = null;
    		}
    	}
   		this.currentSelectedId = id;
		this.getFields();
    },

	changeChart: function() {
		this.stopCharting();
		var sel = this.combo.attr('value');
		var mapped = this.fieldMap[sel];
		if ( this.field != mapped ) {
			if ( mapped ) {
				this.field = mapped;
			}
		
			console.log("changeGrid, field is " + this.field);
        	this.chart.setStore(this.store, {name: this.getStoreFilter("*")}, this.field);
			this.refresh();
		}
	},
	
	addFields: function(response, ioArgs) {
		console.log("addFields");
		if ( response ) {
			console.log("we have a response, build the layout and ranges");
    	    // set the layout structure:
        	this.layout = response.items;
        	this.xaxis = {};
        	this.yaxis = {};
	    	for ( i = 0; i < this.layout.length; i++ ) {
    			if ( this.layout[i].formatter) {
    				this.layout[i].formatter = dojo.hitch(this, this.layout[i].formatter);
    			}
    			if ( this.layout[i].xaxis) {
	        		this.xaxis[this.layout[i].history] = this.layout[i].xaxis;
	    			if ( this.xaxis[this.layout[i].history].labelFunc) {
    					this.xaxis[this.layout[i].history].labelFunc = dojo.hitch(this, this.xaxis[this.layout[i].history].labelFunc);
    				}
    			}
    			if ( this.layout[i].yaxis) {
	        		this.yaxis[this.layout[i].history] = this.layout[i].yaxis;
	    			if ( this.yaxis[this.layout[i].history].labelFunc) {
    					this.yaxis[this.layout[i].history].labelFunc = dojo.hitch(this, this.yaxis[this.layout[i].history].labelFunc);
    				}
    			}
    			if ( this.layout[i].historyL ) {
    			    this.fieldMap[this.layout[i].historyL] = this.layout[i].history;
    			    this.fieldMap2[this.layout[i].history] = this.layout[i].historyL;
    			}
    		}
		}
		
		var	selected = this.fieldMap2[this.field];
		
		console.log("now build the grid");
		this.createGrid();
		console.log("now create the chart");
		this.createChart();
		console.log("selected is " + selected + " and field is " + this.field);
		this.combo.setValue(selected);
		
    	this.chart.render();
        this.grid.startup();
	},
	
    getFields: function() {
    	console.log("get fields");
    	if ( this.yaxis ) {
    		console.log("we already have the ranges/layout, call addFields directly");
    		this.addFields();
    	}
    	else {
	    	console.log("get fields from server");
			dojo.xhrGet({
			    url: '/resources/fields/' + this.widgetType,
		    	handleAs: "json",
			    headers: {
					'Accept': 'application/json'
			    },
		    	load: dojo.hitch(this, this.addFields)
        	});
    		console.log("done, waiting for callback....");
    	}
	      
        Bedrock.fadeCommon(this.splashScreen);
    },

	
	getActionLink: function(value, inRowIndex) {
		var item = this.grid.getItem(inRowIndex);
		if ( item.pid ) {
			return "<a href='javascript:details.stopProcess(\"" + inRowIndex + "\")'>Stop</a>";
		}
		else {
			return "<a href='javascript:details.startProcess(\"" + inRowIndex + "\")'>Start</a>";
		}	
	},

	refresh: function() {
		this.fetch();
	},
	    
    onComplete: function() {
    	console.log("onComplete");
    	if ( this.grid ) {
    		this.grid._refresh(true);
    	}
		this.createChart();
    },
    
	fetch: function() {
		//this.combo.setValue(this.field);
      	// reset the store
       	console.log("interval");
       	this.store.close();
        this.store.fetch({query:{name: this.getStoreFilter("*")}, callOnSet : true});
	},

	updateError: function(data) {
		console.log("error updating starting/stopping process");
	},
	
	_updateProcess: function(inRowIndex, request) {
		console.log("update process ", arguments);
		var item = this.grid.getItem(inRowIndex);
		var _item = {};
		_item.name = item.name[0];
		_item.type = item.type[0];
		_item.jvmName = item.jvmName[0];
		_item.operation = request;
		dojo.xhrPut({
		    url: '/resources/' + this.widgetType + '/' + this.server + '/' + this.subType + '/' + _item.name,
		    putData: dojo.toJson(_item),
			headers: {
				'Content-Type': 'application/json'
		    },		    
            load: dojo.hitch(this, this.refresh),
            error: dojo.hitch(this, this.updateError)
        });
	},
		    	    
	startProcess: function(inRowIndex) {
		this._updateProcess(inRowIndex, "start");
	},
	
	stopProcess: function(inRowIndex) {
		var item = this.grid.getItem(inRowIndex);
		var message = this.xsa_messages.XSA10000;
		message = dojo.string.substitute(message,{_PROCESS_:item.name[0]});
        var submitHandler = dojo.hitch(this, this._updateProcess, inRowIndex, "stop");
        this.dialog = new ModalDialog('modal-dialog', message, submitHandler);
        this.dialog.show();
		
		//this._updateProcess(inRowIndex, "stop");
	},

	_convertTime: function(time) {
		var newDate = new Date( );
		newDate.setTime( time );
		var hours = newDate.getHours().toString();
		if ( hours.length == 1 ) {
			hours = "0" + hours;
		}
		var minutes = newDate.getMinutes().toString();
		if ( minutes.length == 1 ) {
			minutes = "0" + minutes;
		}
		var seconds = newDate.getSeconds().toString();
		if ( seconds.length == 1 ) {
			seconds = "0" + seconds;
		}
		//return newDate.toUTCString();
		if(isNaN(hours) || isNaN(minutes) || isNaN(seconds)){
			return "        ";
		}else{
			return hours + ":" + minutes + ":" + seconds;
		}	
	
	},
	
	timeStamp: function(val) {
		var time_range = this.store.getTimeRange();
		// return 1st, last and equals spaceing in between.
		if ( time_range ) {
			if ( val == 1 ) {
				return this._convertTime(time_range[0]);
			}
			else if ( val == 4 ) {
				return this._convertTime(time_range[3]);
			}
			else if ( val == 7 ) {
				return this._convertTime(time_range[6]);
			}
			else if ( val == 10 ) {
				return this._convertTime(time_range[9]);
			}
			else {
				return "        ";
			}
		}
		else {
			return "        ";
		}
	},
	memorySize : function (val){
		if(isNaN(val)){
			return "";
		}else{
			return val + " MB";
		}
	},
	
    createChart: function() {
    	console.log("create Chart for field " + this.field);
    	if ( this.chart ) {
    		this.chart.destroy();
    		this.chart = null;
    	}
        var p = {
            type:dojox.charting.plot2d["Lines"],
            comparative: false,
            scroll: false,
            stretchToFit: true,
            displayRange: 10,
            yaxis : this.yaxis[this.field],
            xaxis : this.xaxis[this.field],
            start:0
        }
	        
        this.chart = new xsa.monitor.widgets.dashboard.ScalableDataChart(this.chartDiv, p);
        this.chart.setStore(this.store, {name:this.getStoreFilter("*")}, this.field);

		if ( !this.legend ) {	
			console.log("add connect for creating legend");        
			this.c_onData = dojo.connect(this.chart, "onData", this, "createLegend");
		}

		this.toolTip = new dojox.charting.action2d.Tooltip(this.chart, "default", { text : dojo.hitch(this, this.getTooltipText, this.field) } );
		var magnify = new dojox.charting.action2d.Magnify(this.chart, "default", {scale: 3.0});
		var highlight = new dojox.charting.action2d.Highlight(this.chart, "default");
		
    	console.log("created chart");
    },
    getTooltipText : function (field, defaultArguments){
		console.log('getTooltipText', arguments);
		var prefix = '' + field + ':';
		var suffix = this.toolTip.defaultParams.text(defaultArguments);
		
		return prefix + suffix;
    },

	createLegend: function() {
		console.log("create legend");
		if ( this.legend ) {
			this.legend.destroy();
			this.legend = null;
		}
		var legendDiv = dojo.create('div', null, this.legendParentDiv);
		dojo.addClass(legendDiv, 'base-dashboard-legend');
		
		if ( this.c_onData ) {
	        dojo.disconnect(this.c_onData);
		}
		this.legend = new dojox.charting.widget.Legend({chart: this.chart, horizontal: false}, legendDiv);
		//this.legend = new xsa.monitor.widgets.dashboard.SelectableLegend({chart: this.chart, horizontal: false}, legendDiv, dojo.hitch(this, this.selectLegendItem) );
    },
    
	selectLegendItem: function(item, selected){
		console.debug('BaseDashboard.selectLegendItem', [item, selected]);
		this.selectedItems[item] = selected;
		this.fetch();
	},

	getStoreFilter : function(currentFilter){
		var selectedItem;
		for (selectedItem in this.selectedItems){
			console.debug('selectedItem', [selectedItem, this.selectedItems[selectedItem] ]);
		}
		return currentFilter;
	},

	createGrid: function() {
		console.log("create grid ", this.layout);
		if ( this.grid ) {
			this.grid.destroy();
			this.grid = null;
		}
		var gridDiv = dojo.create('div', null, this.gridParentDiv);
		dojo.addClass(gridDiv, 'base-dashboard-grid');
		
        this.grid = new dojox.grid.DataGrid({
            query: {
                name: this.getStoreFilter("*")
            },
            store: this.store,
            clientSort: true,
            structure: this.layout,
            selectionMode : 'single'
        },
        gridDiv);
    	
    	dojo.connect(this.grid, "onRowClick", this, "saveItemSelected");
    	dojo.connect(this.grid, "onHeaderCellMouseDown", this, "setGridSortedFlag");
    	dojo.connect(this.grid, "_onFetchComplete", this, "moveSelectionAndScroll");
    	console.log("created grid");
    	
	},
	saveItemSelected: function(e){
		console.log("saveItemSelected entry");
		this.selectedItem = this.grid.selection.getSelected()[0];
	},

	setGridSortedFlag: function(){
		console.log("setGridSortedFlag entry");
		this.gridSortedFlag = true;
	},
	
	moveSelectionAndScroll: function(items, req){
		console.log("moveSelectionAndScroll entry");
		var rowIndex = false;
		this.grid.selection.clear();
		if(this.gridSortedFlag){	// preferred method
			this.gridSortedFlag = false;
			rowIndex = this.grid.getItemIndex(this.selectedItem);
		}else if(this.selectedItem){
			var item;
			var count = 0;
			for (item in items){
				if( this.selectedItem.name == items[item].name[0]){
					rowIndex = count;	
				}
				count ++;
			}		
		}
		console.log("moveSelectionAndScroll rowIndex ", rowIndex);
		if(rowIndex){
			this.grid.selection.select(rowIndex);
			this.grid.scrollToRow(rowIndex);
			console.log("moveSelectionAndScroll updateDrillDown (before) ", rowIndex);
			this.updateDrillDown(rowIndex);
			console.log("moveSelectionAndScroll updateDrillDown (after) ", rowIndex);
		}			
	},

	createGraphSelect: function(display) {
		if ( this.combo ) {
			this.combo.destroy();
			this.combo = null;
		}
		this.combo = new dijit.form.ComboBox({
            name: "data",
            store: this.fieldStore,
            searchAttr: "historyL",
            onChange: dojo.hitch(this, this.changeChart)
        },
        this.selectGraph);
	},
	
		
    startPauseChart: function() {
    	if ( this.started ) {
    		this.stopCharting();
	    }
	    else {
    		this.started = true;
	    	console.log("start chart");
	 		this.startPauseButton.setLabel("Pause Chart Update");
	    	this.interval = setInterval( dojo.hitch(this, "fetch"), 5000);
	    }
    },
    
    stopCharting: function() {
    	console.log("stop chart");
   		if(!this.started){
			this.startPauseButton.setLabel("Start Chart Update");
		}
		else{
			this.startPauseButton.setLabel("Resume Chart Update");
		}
		if ( this.interval ) {
       		clearInterval( this.interval);
       		this.interval = null;
       	}
       	else {
       		console.log("charting was not running");
       	}
       	this.started = false;
    },
        
    setCurrentStatus: function(icon, message) {
        this.currentStatusIcon = icon;
        this.currentStatusMessage = message;
    },
    
    showDefault: function(uri) {
        this.actions.showDefault(uri);
        
        if (this.panel){
            this.panel.showDefault(uri);
		}            
        Bedrock.show(this.defaultMessage);
    }
    
});
