CREATE TABLE Simplecaches (
    ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),	Name VARCHAR(64) NOT NULL,
    Created BIGINT NOT NULL,
    Updated BIGINT NOT NULL,
	OwnerID INTEGER NOT NULL
);


CREATE TABLE Domains (
    ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1), 
	Name VARCHAR(64) NOT NULL UNIQUE,
	jmxUser VARCHAR(64),
	jmxPass VARCHAR(64),
	keystorePassword VARCHAR(64),
	customCredGenerator VARCHAR(256),
	credGeneratorProperties VARCHAR(256),
	OwnerID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE Servers (
    ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1), 
	name VARCHAR(64) NOT NULL,
	ListenerPort INTEGER NOT NULL,
	jmxPort INTEGER NOT NULL,
	OwnerID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);


CREATE TABLE DomainsKeystores (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    DomainID INTEGER NOT NULL,
    KeystoreID INTEGER NOT NULL,
	OwnerID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);


CREATE TABLE DomainsServers (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    DomainID INTEGER NOT NULL,
    ServerID INTEGER NOT NULL,
	OwnerID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);


CREATE TABLE KeyStores (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
	name VARCHAR(64) NOT NULL,
	keyStore VARCHAR(64) NOT NULL,
	trustStore VARCHAR(64) NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);	

CREATE TABLE Users (
    ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1), 
    UserName VARCHAR(64) NOT NULL UNIQUE, 
    Name VARCHAR(64) NOT NULL, 
    Email VARCHAR(128) NOT NULL, 
    Salt VARCHAR(128) NOT NULL, 
    PasswordHash VARCHAR(128) NOT NULL,
    CurrentStatus VARCHAR(8) NOT NULL,
    CurrentMessage VARCHAR(8),
    Created BIGINT NOT NULL,
    Updated BIGINT NOT NULL,
    SystemUser INTEGER
);

CREATE TABLE Groups (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    Name VARCHAR(64) NOT NULL UNIQUE,
    Description VARCHAR(128) NOT NULL,
	OwnerID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE GroupsUsers (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    GroupID INTEGER NOT NULL,
    UserID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE SettingDefinitions (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    Name VARCHAR(32) NOT NULL,
    Description VARCHAR(32) NOT NULL,
    DefaultValueID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE SettingValues (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    SettingID INTEGER NOT NULL,
    Value VARCHAR(32) NOT NULL,
    Description VARCHAR(32) NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE SettingInstances (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    UserID INTEGER NOT NULL,
    SettingID INTEGER NOT NULL,
    ValueID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);

CREATE TABLE UserRoles (
	ID INTEGER PRIMARY KEY GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    UserID INTEGER NOT NULL,
    RoleID INTEGER NOT NULL,
    Created BIGINT NOT NULL, 
    Updated BIGINT NOT NULL 
);
